#include <string.h>

#include "mbs.h"
#include "f2c.h"
#include "rcb.h"
#include "rcb2.h"

#define  STRINGBUFFER  256

void 
rcb_guess (char *filename, void(*init)(double *par), double(*calc)())
{
    static char    name[STRINGBUFFER], outfilename[STRINGBUFFER];
    void           rcb_guess_ini_function_pointers(int, double(*)());
    char         **datafile, **outputfile;
    double       **parameters, x, *y;
    float          value, xval, yval;
    int            i, j, k, np, nx, nvar;
    FILE          *file, **outfile;
    
    file = fopen(filename,"r");
    fscanf (file, "%d %d %d", &np, &nx, &nvar);
    if (!(parameters = (double **)mbs_2D_alloc_array(nx, np, sizeof(double))))
	mbs_alloc_error("RCB GUESS", "Parameters");
    if (!(datafile = (char **)mbs_2D_alloc_array(nx, STRINGBUFFER, sizeof(char))))
	mbs_alloc_error("RCB GUESS", "Data file names");
    if (!(outputfile = (char **)mbs_2D_alloc_array(nx, STRINGBUFFER, sizeof(char))))
	mbs_alloc_error("RCB GUESS", "Output file names");
    if (!(outfile = (FILE **)calloc(nvar, sizeof(FILE*))))
	mbs_alloc_error("RCB GUESS", "Output filehandles");
    if (!(y = (double *)calloc(nvar+2, sizeof(double))))
	mbs_alloc_error("RCB GUESS", "Computed values");
    for (i=0 ; i<np ; i++) 
	for (j=0 ; j<nx ; j++) 
	{
	    fscanf (file, "%s %f", &name, &value);
	    parameters[j][i] = value;
	}
    for (j=0 ; j<nx ; j++)
    {
	fscanf (file, "%s", &name);
	strcpy (datafile[j], name);
    }
    for (j=0 ; j<nx ; j++)
    {
	fscanf (file, "%s", &name);
	strcpy (outputfile[j], name);
    }
    close (file);

    rcb_guess_ini_function_pointers (nvar, calc);
    for (j=0 ; j<nx ; j++)
    {
	(*init)(parameters[j]);

	file = fopen(datafile[j],"r");
	for (k=0 ; k<nvar ; k++)
	{
	    sprintf (outfilename, "%s-%d", outputfile[j], k+1);
	    outfile[k] = fopen(outfilename,"w");
	}
	while (!feof(file)) 
	{
	    if (fscanf(file, "%g %g", &xval, &yval)==2)
	    {
		x = xval;
		(*_RCB_make_guess)(x, y);
		for (k=0 ; k<nvar ; k++) 
		{
		    (*_RCB_write_guess_data)(j, k, x, (double)yval, y[k+1], outfile[k]);
		}
	    }
	}
	close (file);
	for (k=0 ; k<nvar ; k++) 
	    close (outfile[k]);
    }
}
void
make_guess_tcd (double x, double *y)
{
    y[1] = (*_RCB_tcd_y_value)(&x);
}
void
make_guess_mcd (double x, double *y)
{
    (*_RCB_mcd_y_value)(&x, y);
}

void
write_guess_data_custom (int j, int k, double xval, double yval, double y, FILE *outfile)
{
    static double prn[3];

    ++j;    ++k;
    (*_RCB_data_converter)(&j, &k, &yval, &y, prn);
    fprintf (outfile, "%e %e\n", xval, prn[1]);
}
void
write_guess_data (int j, int k, double xval, double yval, double y, FILE *outfile)
{
    fprintf (outfile, "%e %e\n", xval, y);
}
