#ifndef _RCB_H_
#define _RCB_H_

#include <stdio.h>

/*
 * copy current values of model parameters.
 * points to user-specified procedure
 * that has been passed to RCBfit(), rcb_fit(), or rcb_guess()
 */
void    (*_RCB_initialize_parameters)(double*);
/*
 * calculate theoretical value(s).
 * point to user-specified procedures
 * that has been passed to RCBfit(), rcb_fit(), or rcb_guess()
 */
double  (*_RCB_tcd_y_value)(double*);
double  (*_RCB_mcd_y_value)(double*,double*);
/*
 * calculate residuals explicitly (only for traditional code).
 * point to user-specified procedures
 * that has been passed to rcb_fit()
 */
double  (*_RCB_tcd_residual)(double*,double*);
double  (*_RCB_mcd_residual)(double*,double*);
/*
 * compute residual (forward) and simulated data (reverse)
 * on a non-standard scale.
 * point to user-specified procedures
 * that have been passed to rcb_set_customized_distance_measure()
 */
double  (*_RCB_forward)(double*, double*);  
double  (*_RCB_reverse)(double*, double*);  

/*
 * internal pointer channeling proper computation of theoretical values.
 * affected by rcb_select_data_style()
 */
void    (*_RCB_theory)(int);
/*
 * internal pointer channeling proper computation of residuals.
 * affected by rcb_select_data_style() and
 * rcb_set_customized_distance_measure()
 */
double  (*_RCB_residual)(int);
/*
 * internal pointer channeling proper generation of simulated data.
 * affected by rcb_select_data_style() and
 * rcb_set_customized_distance_measure()
 */
void    (*_RCB_adderror)(double*,int*);
/*
 * internal pointer channeling proper computation of theoretical values
 * in rcb_guess().
 * affected by rcb_select_data_style()
 */
void    (*_RCB_make_guess)(double, double*);
/*
 * internal pointer channeling data output.
 * affected by rcb_select_data_style()
 */
int     (*_RCB_output)();
/*
 * internal pointer to channel data writing for RCBfit() [rcb_fit()] 
 * and rcb_guess(), respectively.
 * affected by rcb_set_customized_data_output() and rcb_select_data_style()
 */
void    (*_RCB_write_fit_data)(int, integer*, FILE*);  
void    (*_RCB_write_guess_data)(int, int, double, double, double, FILE*);  
/*
 * change scale of data and theoretical values on output.
 * points to user-specified procedure
 * that has been passed to rcb_set_customized_data_output()
 */
void    (*_RCB_data_converter)(int*, int*, double*, double*, double*);

void   two_column_data_theory(int);
void   multi_column_data_theory(int);

double two_column_data_residual(int);
double multi_column_data_residual(int);
double two_column_data_residual_custom(int);
double multi_column_data_residual_custom(int);
double two_column_data_residual_outdated(int);
double multi_column_data_residual_outdated(int);
  
void   adderror_tcd(double*,int*);
void   adderror_mcd(double*,int*);
void   adderror_tcd_custom(double*,int*);
void   adderror_mcd_custom(double*,int*);

void   make_guess_tcd(double,double*);
void   make_guess_mcd(double,double*);

int    outinf_tcd();
int    outinf_mcd();

void   write_fit_data_tcd(int,integer*,FILE*);
void   write_fit_data_mcd(int,integer*,FILE*);
void   write_fit_data_tcd_custom(int,integer*,FILE*);
void   write_fit_data_mcd_custom(int,integer*,FILE*);

void   write_guess_data(int,int,double,double,double,FILE*);
void   write_guess_data_custom(int,int,double,double,double,FILE*);


#endif
