#include <math.h>
#include <limits.h>
#include <float.h>
#include <stdio.h>

double d1mach_(i)
int *i;
{
  if (*i < 1 || *i > 5) 
    fprintf(stderr,"D1MACH argument out of range: %d\n",*i);
  else {
    if (*i == 1) return DBL_MIN;                 /* Required by FUI */
    else if (*i == 2) return DBL_MAX;
    else if (*i == 3) return pow((double)2,(double)(-DBL_MANT_DIG));
    else if (*i == 4) return DBL_EPSILON;        /* Required by FUI */
    else if (*i == 5) return log10((double)2);
    else fprintf(stderr,"D1MACH argument out of range: %d\n",*i);
  }
  return 0.0;
}

/*******************************************************************

   D1MACH can be used to obtain machine-dependent parameters for the
   local machine environment.  It is a function subprogram with one
   (input) argument, and can be referenced as follows:

        D = D1MACH(I)

   where I=1,...,5.  The (output) value of D above is determined by
   the (input) value of I.  The results for various values of I are
   discussed below.

   D1MACH( 1) = B**(EMIN-1), the smallest positive magnitude.
   D1MACH( 2) = B**EMAX*(1 - B**(-T)), the largest magnitude.
   D1MACH( 3) = B**(-T), the smallest relative spacing.
   D1MACH( 4) = B**(1-T), the largest relative spacing.
   D1MACH( 5) = LOG10(B)

   Assume double precision numbers are represented in the T-digit,
   base-B form

              sign (B**E)*( (X(1)/B) + ... + (X(T)/B**T) )

   where 0 .LE. X(I) .LT. B for I=1,...,T, 0 .LT. X(1), and
   EMIN .LE. E .LE. EMAX.

   The values of B, T, EMIN and EMAX are provided in I1MACH as
   follows:
   I1MACH(10) = B, the base.
   I1MACH(14) = T, the number of base-B digits.
   I1MACH(15) = EMIN, the smallest exponent E.
   I1MACH(16) = EMAX, the largest exponent E.

*******************************************************************/
