#include <math.h>
#include <limits.h>
#include <float.h>
#include <stdio.h>

int i1mach_(i)
int *i;
{
  if (*i < 1 || *i > 16) 
    fprintf(stderr,"I1MACH argument out of range: %d\n",*i);
  else {
    if (*i == 1) return 5;
    else if (*i == 2) return 6;
    else if (*i == 3) return 6;
    else if (*i == 4) return 6;             /* Required by FUI */
    else if (*i == 5) return 8*sizeof(int);
    else if (*i == 6) return sizeof(int);
    else if (*i == 7) return 2;
    else if (*i == 8) return 31;
    else if (*i == 9) return INT_MAX;
    else if (*i == 10) return 2;
    else if (*i == 11) return FLT_MANT_DIG;
    else if (*i == 12) return FLT_MIN_EXP;
    else if (*i == 13) return FLT_MAX_EXP;
    else if (*i == 14) return DBL_MANT_DIG;
    else if (*i == 15) return DBL_MIN_EXP; 
    else if (*i == 16) return DBL_MAX_EXP;
    else fprintf(stderr,"I1MACH argument out of range: %d\n",*i);
  }
  return 0;
}

/*******************************************************************

   I1MACH can be used to obtain machine-dependent parameters for the
   local machine environment.  It is a function subprogram with one
   (input) argument and can be referenced as follows:

        K = I1MACH(I)

   where I=1,...,16.  The (output) value of K above is determined by
   the (input) value of I.  The results for various values of I are
   discussed below.

   I/O unit numbers:
     I1MACH( 1) = the standard input unit.
     I1MACH( 2) = the standard output unit.
     I1MACH( 3) = the standard punch unit.
     I1MACH( 4) = the standard error message unit.

   Words:
     I1MACH( 5) = the number of bits per integer storage unit.
     I1MACH( 6) = the number of characters per integer storage unit.

   Integers:
     assume integers are represented in the S-digit, base-A form

                sign ( X(S-1)*A**(S-1) + ... + X(1)*A + X(0) )

                where 0 .LE. X(I) .LT. A for I=0,...,S-1.
     I1MACH( 7) = A, the base.
     I1MACH( 8) = S, the number of base-A digits.
     I1MACH( 9) = A**S - 1, the largest magnitude.

   Floating-Point Numbers:
     Assume floating-point numbers are represented in the T-digit,
     base-B form
                sign (B**E)*( (X(1)/B) + ... + (X(T)/B**T) )

                where 0 .LE. X(I) .LT. B for I=1,...,T,
                0 .LT. X(1), and EMIN .LE. E .LE. EMAX.
     I1MACH(10) = B, the base.

   Single-Precision:
     I1MACH(11) = T, the number of base-B digits.
     I1MACH(12) = EMIN, the smallest exponent E.
     I1MACH(13) = EMAX, the largest exponent E.

   Double-Precision:
     I1MACH(14) = T, the number of base-B digits.
     I1MACH(15) = EMIN, the smallest exponent E.
     I1MACH(16) = EMAX, the largest exponent E.

   To alter this function for a particular environment, the desired
   set of DATA statements should be activated by removing the C from
   column 1.  Also, the values of I1MACH(1) - I1MACH(4) should be
   checked for consistency with the local operating system.

***REFERENCES  P. A. Fox, A. D. Hall and N. L. Schryer, Framework for
                 a portable library, ACM Transactions on Mathematical
                 Software 4, 2 (June 1978), pp. 177-188.

*********************************************************************/
