/*********************************
 *
 *      FUI header comments
 *
 *  FitDataFormat:    TwoColumn  
 *
 *********************************/
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "RCBfit.h"


int
main (int argc, char *argv[])
{
    void    initialize(double*);
    double  value(double*), f_dist(double*,double*), r_dist(double*,double*);
    
    if (argc>=4 || argc==2) 
    {
	fprintf (stderr,"Wrong number of arguments: %d\n", argc);
	exit(1);
    }
    else
    {
	rcb_set_customized_distance_measure(f_dist, r_dist);
	if (argc==1) 
	    RCBfit ("fittp", initialize, value);
	if (argc==3)
	{
	    if (strcmp(argv[1],"-fit")==0)
	    {
		fprintf (stderr, "Calling fit program with tapefile \"%s\"...\n", argv[2]);
		RCBfit (argv[2], initialize, value);
	    }
	    else if (strcmp(argv[1],"-guess")==0)
		rcb_guess (argv[2], initialize, value);
	    else 
		fprintf (stderr, "Unknown option: %s\n", argv[1]);
	}
	exit(0);
    }
}

double   f1, f2, xp1, xp2;

void
initialize (double *parameters)
{
/*
    Begin: Synonyms

    f1             factor_1            f1
    f2             factor_2            f2
    xp1            exponent_1          x1
    xp2            exponent_2          x2

    End: Synonyms
*/
    f1  = parameters[0];
    f2  = parameters[1];
    xp1 = parameters[2];
    xp2 = parameters[3];
}
/*
 *          theoretical value
 *  Input:
 *          t.........independent data value (X)
 */
double
value (double *t)  {  return (f1*exp(-xp1*(*t)) + f2*exp(-xp2*(*t)));  }

/*
 *          scale transformators
 *  Input:
 *          dy........dependent data value
 *          cy........dependent calculated value
 *          r........(simulated) residual
 */
double
f_dist (double *dy, double *cy)  {  return (log(*dy) - log(*cy));  }
double
r_dist (double *cy, double *r)   {  return (exp(log(*cy) + *r));   }

    
