/*********************************
 *
 *      FUI header comments
 *
 *  FitDataFormat:    MultiColumn  
 *  NumbDepVars:      2
 *
 *********************************/
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "RCBfit.h"

int
main (int argc, char *argv[])
{
    void    initialize(double*);
    double  value(double*,double*), forward(double*,double*), reverse(double*,double*);

    if (argc>=4 || argc==2) 
    {
	fprintf (stderr,"Wrong number of arguments: %d\n", argc);
	exit(1);
    }
    else
    {
	rcb_select_data_style (_RCB_MULTI_COLUMN);
	rcb_set_customized_distance_measure (forward, reverse);
	if (argc==1) 
	    RCBfit ("fittp", initialize, value);
	if (argc==3)
	{
	    if (strcmp(argv[1],"-fit")==0)
	    {
		fprintf (stderr, "Calling fit program with tapefile \"%s\"...\n", argv[2]);
		RCBfit (argv[2], initialize, value);
	    }
	    else if (strcmp(argv[1],"-guess")==0)
		rcb_guess (argv[2], initialize, value);
	    else 
		fprintf (stderr, "Unknown option: %s\n", argv[1]);
	}
	exit(0);
    }
}

double         c1, c2, f, s1, s2;

void
initialize (double *parameters)
{
/*
    Begin: Synonyms

    p      percentage    p
    c1     exponent_1    c1
    c2     exponent_2    c2
    s1     scale_1       s1
    s2     scale_2       s2

    End: Synonyms
*/
    p  = parameters[0];
    c1 = parameters[1];
    c2 = parameters[2];
    s1 = parameters[3];
    s2 = parameters[4];
}
double
value (double *t, double *y)  
{  
    double   aux;
    
    aux = exp(-c2*(*t));

    y[1] = s1*(p*(exp(-c1*(*t)) - aux) + aux);  /*  = f1 * [p*exp(-c1*t) + (1-p)*exp(-c2*t)] */
    y[2] = s2*(1.0-aux);	                /*  = f2 * [1 - exp(-c2*t)]                  */
}
/*
 *          scale transformators
 *  Input:
 *          dy........dependent data value
 *          cy........dependent calculated value
 *          r........(simulated) residual
 */
double
forward (double *dy, double *cy)  {  return (log(*dy) - log(*cy));  }
double
reverse (double *cy, double *r)   {  return (exp(log(*cy) + *r));   }

