/*********************************
 *
 *      FUI header comments
 *
 *  FitDataFormat:    MultiColumn  
 *  NumbDepVars:      2
 *
 *********************************/
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "RCBfit.h"


int
main (int argc, char *argv[])
{
    void    initialize(double*);
    double  function(double*,double*);
    double  f_dist(double*,double*), r_dist(double*,double*);
    
    if (argc>4 || argc==2) 
    {
	fprintf (stderr,"Wrong number of arguments.\n");
	exit(1);
    }
    else
    {
	/*************  put switches here *************/
	rcb_select_data_style(_RCB_MULTI_COLUMN);
	rcb_set_customized_distance_measure(f_dist, r_dist);
	/**********************************************/
	if (argc==1) 
	    RCBfit ("fittp", initialize, function);
	else
	{
	    if (strcmp(argv[1],"-fit")==0)
		RCBfit (argv[2], initialize, function);
	    else if (strcmp(argv[1],"-guess")==0)
		rcb_guess (argv[2], initialize, function);
	    else 
	    {
		fprintf (stderr, "Unknown option: %s\n", argv[1]);
		exit(1);
	    }
	}
	exit(0);
    }
}
/*
 *          scale transformators
 *  Input:
 *          dy........dependent data value
 *          cy........dependent calculated value
 *          r........(simulated) residual
 */
double
f_dist (double *dy, double *cy)  {
    if (*dy < 0.0)   return (0.0);
    else             return (log(*dy) - log(*cy));
}
/*double
f_dist (double *dy, double *cy)  {  return (log(*dy) - log(*cy));  }*/
double
r_dist (double *cy, double *r)   {  return (exp(log(*cy) + *r));   }


double    nn, delta, um, a, fl, ul, vo, to, k;

void
initialize (double *parameters)
{
/*
    Begin: Synonyms

    nn     nn            nn
    delta  delta         delta
    um     um            um
    a      a             a
    fl     fl            fl
    ul     ul            ul
    vo     vo            vo
    to     to            to
    k      k             k 

    End: Synonyms
*/
    nn  = parameters[0];
    delta = parameters[1];
    um = parameters[2];
    a = parameters[3];
    fl = parameters[4];
    ul = parameters[5];
    vo = parameters[6];
    to = parameters[7];
    k = parameters[8];
 
}

/*
   Compute theoretical values for data in the format {X;Y_1,Y_2}
   
   t       pointer to independent variable (*t = X)  [INPUT]
   y       array of theoretical values               [OUTPUT]
           y[1] is matched against Y_1,
	   y[2] against Y_2, etc.
*/
double
function (double *t, double *y)
{
    /* y[1] = Compute theoretical value corresponding to Y_1 */
    /* y[2] = Compute theoretical value corresponding to Y_2 */

    /* e.g., */
    y[1] = ( vo*(nn*k*delta*to
         *(((1-fl)*(delta-
         ul-a)-a*fl)/
       (delta*(3.000-delta))*exp(-delta*(*t))+a*fl*exp((-ul-a)*(*t))/
       ((a+ul)*
       (3.000-a-ul)))/(delta-a-ul)+exp((-um)*(*t))*(3.000-nn*k*to*
             (1-fl+a*fl/(a+ul)))/
         (3.000-um)-exp(-3.000*(*t))*(
         nn*k*delta*to*(((1-fl)*(delta-ul-a)-a*fl)/(delta*(
        3.000-delta))+a*fl/((a+ul)*(3.000-a-ul)))/(delta-a-ul)
       + (3.000-nn*k*to*(1-fl+a*fl/(a+ul)))/(3.000-um)))
        + vo*exp(-3.000*(*t)));
    y[2] =  ( k*vo*to*(((1-fl)*(delta-a-ul)-a*fl)/delta*
       exp(-delta*(*t))+(delta-ul)*fl/(a+ul)*exp(-(a+ul)*(*t)))/
        (delta-a-ul));
}


