/*********************************
 *
 *      FUI header comments
 *
 *  FitDataFormat:    MultiColumn  
 *  NumbDepVars:      2
 *
 *********************************/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "RCBfit.h"

double rho1,rho2,alpha,mu,r,Ton1,Ton2,Toff,X0;

int main (int argc, char *argv[])
{
  void initialize(double*);
  double function(double*,double*), f_dist(double*,double*), r_dist(double*,double*);
    
  if (argc>4 || argc==2) {
    fprintf (stderr,"Wrong number of arguments.\n");
    exit(1);
  } else {
    rcb_select_data_style (_RCB_MULTI_COLUMN);
    rcb_set_customized_distance_measure(f_dist, r_dist);
    if (argc==1) 
      RCBfit ("fittp", initialize, function);
    else {
      if (strcmp(argv[1],"-fit")==0)
        RCBfit (argv[2], initialize, function);
      else if (strcmp(argv[1],"-guess")==0)
        rcb_guess (argv[2], initialize, function);
      else {
        fprintf (stderr, "Unknown option: %s\n", argv[1]);
        exit(1);
      }
    }
  exit(0);
  }
}

void initialize (double *par)
{
/*
   Begin: Synonyms

	rho1	rho1	rho1
	rho2	rho2	rho2
	alpha	alpha	alpha
	mu	mu	mu
	r	r	r
	Ton1	Ton1	Ton1
	Ton2	Ton2	Ton2
	Toff	Toff	Toff
	X0	X0	X0

   End: Synonyms
*/
    rho1 = par[0];
    rho2 = par[1];
    alpha = par[2];
    mu   = par[3];
    r    = par[4];
    Ton1 = par[5];
    Ton2 = par[6];
    Toff = par[7];
    X0 = par[8];
}

double f_dist (double *dy, double *cy)  {  return (log(*dy) - log(*cy));  }
double r_dist (double *cy, double *r)   {  return (exp(log(*cy) + *r));   }

double function (double *time, double *result)
{
  double A1,A2,M1,M2,t,tt,Atoff;
  t = *time;
  M1 = M2 = 0.0;
  A1 = A2 = X0;
  if (t > Ton1 && t < Toff) {
    A1 = X0*exp(rho1*(t-Ton1));
  } else if (t >= Toff) {
    Atoff = X0*exp(rho1*(Toff-Ton1));
    tt = t-Toff;
    A1 = Atoff*exp(-(alpha+r)*tt);
    M1 = (r*Atoff/(alpha+r-mu))*(exp(-mu*tt)-exp(-(alpha+r)*tt));
  }   
  if (t > Ton2 && t < Toff) {
    A2 = X0*exp(rho2*(t-Ton2));
  } else if (t >= Toff) {
    Atoff = X0*exp(rho2*(Toff-Ton2));
    tt = t-Toff;
    A2 = Atoff*exp(-(alpha+r)*tt);
    M2 = (r*Atoff/(alpha+r-mu))*(exp(-mu*tt)-exp(-(alpha+r)*tt));
  } 
  result[1] = A1+M1;
  result[2] = A2+M2;
  /* fprintf(stderr,"%f %f %f %f %f\n",t,A1,M1,A2,M2); */
}
