#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "RCBfit.h"

double rho,alpha,mu,r,Ton,Toff,X0;

int main(int argc, char *argv[])
{
  void initialize(double*);
  double function(double*), f_dist(double*,double*), r_dist(double*,double*);
    

  if (argc>4 || argc==2) {
    fprintf (stderr,"Wrong number of arguments.\n");
    exit(1);
  } else {
    rcb_set_customized_distance_measure(f_dist, r_dist);
    if (argc==1) 
      RCBfit ("fittp", initialize, function);
    else {
      if (strcmp(argv[1],"-fit")==0)
        RCBfit (argv[2], initialize, function);
      else if (strcmp(argv[1],"-guess")==0)
        rcb_guess (argv[2], initialize, function);
      else {
        fprintf (stderr, "Unknown option: %s\n", argv[1]);
        exit(1);
      }
    }
  exit(0);
  }
}

void initialize (double *par)
{
/*
   Begin: Synonyms

	rho	rho	rho
	alpha	alpha	alpha
	mu	mu	mu
	r	r	r
	Ton	Ton	Ton
	Toff	Toff	Toff
	X0	X0	X0

   End: Synonyms
*/
    rho = par[0];
    alpha = par[1];
    mu = par[2];
    r = par[3];
    Ton = par[4];
    Toff = par[5];
    X0 = par[6];
    if (0) printf("%f %f %f\n",par[0],par[4],par[5]);
}

double f_dist (double *dy, double *cy)  {  return (log(*dy) - log(*cy));  }
double r_dist (double *cy, double *r)   {  return (exp(log(*cy) + *r));   }

double function (double *time)
{
  double A,M,t,tt,Atoff;
  t = *time;
  M = 0.0;
  if (t < Ton)
    A = X0; 
  else if (t < Toff)
    A = X0*exp(-rho*(t-Ton));
  else {
    Atoff = X0*exp(-rho*(Toff-Ton));
    tt = t-Toff;
    A = Atoff*exp(-(alpha+r)*tt);
    M = (r*Atoff/(alpha+r-mu))*(exp(-mu*tt)-exp(-(alpha+r)*tt));
  } 
  return (A+M);
}
