
proc displayPartialAverages {} {
    global  numb_exp FUInumbPoints FUIresiduals two_column_data

    if {!$two_column_data} {
        calculateResiduals
    }
    if [info exists FUIresiduals] {
        set chi [paEvaluate partialAverage]
    }
    set FUImaxNumbPoints 0
    for {set ix 0} {$ix<$numb_exp} {incr ix} {
        if {$FUImaxNumbPoints<$FUInumbPoints($ix)} {
            set FUImaxNumbPoints $FUInumbPoints($ix)
        }
    }
    set paChartDepth [expr int(ceil(log($FUImaxNumbPoints)/log(2.0)))]
    if [winfo exists .paav] {
        updatePAdisplay $paChartDepth $chi partialAverage
    } else {
        createPAdisplay $paChartDepth $chi partialAverage
    }
}
proc updatePAdisplay {depth chi average} {
    global  numb_exp 
    upvar  $average  pa

    for {set pai 1; set ir 0} {$ir<$depth} {incr ir; set pai [expr $pai*2]} {
        for {set ix 0} {$ix<$numb_exp} {incr ix} {
            .paav.p($ix,$ir) configure \
                -text [format "%.4f" [expr sqrt($pa($ix,$pai))/$chi]]
        }
    }
}
proc createPAdisplay {depth chi average} {
    global   FUIstandardFont numb_exp
    upvar   $average pa

    if [winfo exists .paav] { raise .paav } else {
        toplevel .paav
        wm title .paav "FUI: partial averages"
    
        frame .paav.f(legend)
        label .paav.l(legend) -width 10 -text "# intervals" -font $FUIstandardFont(pane)
        pack  .paav.l(legend) -padx 4m -side left -in .paav.f(legend)
        for {set ix 0} {$ix<$numb_exp} { } {
            incr ix
            label .paav.p($ix,legend) -width 10 -font $FUIstandardFont(pane) \
                -text "Exp. $ix"
            pack .paav.p($ix,legend) -padx 2m -side left -in .paav.f(legend)
        }
        pack .paav.f(legend) -pady 2m
        for {set pai 1; set ir 0} {$ir<$depth} {incr ir; set pai [expr $pai*2]} {
            frame .paav.f($ir)
            label .paav.l($ir) -width 10 -text [expr int(pow(2,$ir))] \
                -font $FUIstandardFont(pane)
            pack .paav.l($ir) -padx 4m -side left -in .paav.f($ir)
            for {set ix 0} {$ix<$numb_exp} {incr ix} {
                label .paav.p($ix,$ir) -width 10 -font $FUIstandardFont(pane) \
                    -text [format "%.4f" [expr sqrt($pa($ix,$pai))/$chi]]
                pack .paav.p($ix,$ir) -padx 2m -side left -in .paav.f($ir)
            }
            pack .paav.f($ir)
        }
        frame .paav.f(buttons)
        button .paav.update -text Update -font $FUIstandardFont(pane) -command {
            displayPartialAverages
        }
        button .paav.close -text Close -command { destroy .paav } \
            -font $FUIstandardFont(pane)
        pack .paav.update .paav.close -ipadx 6m -padx 4m -side left -in .paav.f(buttons)
        pack .paav.f(buttons) -pady 4m -anchor w
    }
}
proc paEvaluate partialAverage {
    global  numb_exp FUInumbPoints FUIresiduals
    global  FUIsumOfSquares 
    global  two_column_data numb_dependent_vars
    upvar  $partialAverage pa

    for {set ix 0} {$ix<$numb_exp} {incr ix} {
        set ssq 0.0
        if {$two_column_data} {
            for {set ip 0} {$ip<$FUInumbPoints($ix)} {incr ip} {
                set ssq [expr $ssq + $FUIresiduals($ix,$ip)*$FUIresiduals($ix,$ip)]
            }
        } else {
            for {set ip 0} {$ip<$FUInumbPoints($ix)} {incr ip} {
                for {set ic 0} {$ic<$numb_dependent_vars} {incr ic} {
                    set ssq [expr $ssq + $FUIresiduals($ix,$ip,$ic)*$FUIresiduals($ix,$ip,$ic)]
                }
            }
        }
        set rssq [expr sqrt($ssq)]

        set paw [expr $FUInumbPoints($ix)]
        set pai 1
        while {$paw>=1.0} {
            set pa($ix,$pai) 0.0
            set pai [expr $pai*2] 
            set paw [expr $paw/2.0]
        }
        set paw [expr $FUInumbPoints($ix)]
        set pai 1
        while {$paw>=1.0} {
            set k 0
            for {set j $paw; set i 0} {$i<$pai} {incr i; set j [expr $j+$paw]} {
                set sum 0.0
                if {$two_column_data} {
                    while {$k<$j} {
                        set sum [expr $sum+$FUIresiduals($ix,$k)]
                        incr k
                    }
                } else {
                    while {$k<$j} {
                        for {set ic 0} {$ic<$numb_dependent_vars} {incr ic} {
                            set sum [expr $sum+$FUIresiduals($ix,$k,$ic)]
                        }
                        incr k
                    }
                }
                set pa($ix,$pai) [expr $pa($ix,$pai) + $sum*$sum]
            }
            set pai [expr $pai*2]
            set paw [expr $paw/2.0]
        }
    }
    return $rssq
}
proc calculateResiduals {} {
    global  numb_exp numb_dependent_vars
    global  FUIresiduals

    for {set ix 1} {$ix<=$numb_exp} {incr ix} {
        set outfile [open "outdata[expr $ix+1]"]
        set ip 0
        while {[gets $outfile line]>=0} {
            for {set l 1; set ic 0} {$ic<$numb_dependent_vars} {incr ic} {
                set dv [lindex $line $l];  incr l
                set cv [lindex $line $l];  incr l
                set FUIresiduals($ix,$ip,$ic) [expr $dv-$cv]
            }
            incr ip
        }
        close $outfile
    }
}

