
proc doFit {} {
    global env FUIfit_instance FUIcleanup_interval FUItmp

    if [checkConsistency] {
        if {[info exists env(HOSTNAME)] && $env(HOSTNAME)=="loplop"} {
            raise .
        }
        if {![expr $FUIfit_instance % $FUIcleanup_interval] && $FUIfit_instance} {
            eval exec rm [glob ${FUItmp}*]
        }
	if [file exists jacout ] { exec rm jacout }
	if [file exists simout ] { exec rm simout }
	if [file exists output ] { exec rm output }
        writeTapeFile
        if [runFit] {
            analyzeOutputFile
            analyzeSimoutFile
        }
        if {[info exists env(HOSTNAME)] && $env(HOSTNAME)=="loplop"} {
            if [winfo exists .p] { lower . .p }
        }
    } else {
        BSerror "doFit" "Parameter setup inconsistent with model"
    }
}
proc runFit {} {
    global modelname fit_status

    regexp {\.([cf])$} $modelname match model_suffix
    regsub {\.[cf]$} $modelname {} stripped_modelname
    if {$model_suffix == "c"} {
        catch {exec make -k $stripped_modelname >& make.log} 
        set mf [open "make.log"]
        while {[gets $mf line]>=0} { set msg $line }
        if {!([regexp {^gcc} $msg] || [regexp {is up to date.$} $msg])} {
            BSerror "runFit" "$msg\nFor details see \"make.log\""
            return 0
        }
        close $mf
    } elseif {$model_suffix == "f"} {
        if {[file exists Makefile] || [file exists makefile]} {
            catch {exec make -k $stripped_modelname >& make.log}
        } else {
            catch {exec MAKE $stripped_modelname FITNAME=$stripped_modelname >& make.log}
        }
        set mf [open "make.log"]
        while {[gets $mf line]>=0} { set msg $line }
        if {!([regexp {^gcc} $msg] || [regexp {is up to date.$} $msg])} {
            BSerror "runFit" "$msg\nFor details see \"make.log\""
            return 0
        }
        close $mf
    } else {
        BSerror "runFit" "Unknown model suffix: $model_suffix"
	return 0
    }
    if [catch {exec ./$stripped_modelname >& $stripped_modelname.log}] {
        BSerror "runFit" "Data fitting exited abnormally.\n\
            See \"$stripped_modelname.log\" for details"
        return 0
    }
    return 1
}
proc analyzeSimoutFile {} {
    global  par_short par_fixed par_keep numb_par numb_exp
    global  par_sim_mean par_sim_var par_sim_lower par_sim_upper
    global  FloatingpointNumber

    for {set i 0} {$i<$numb_par} {incr i} {
        if $par_keep($i,0) { set top 1 } else { set top $numb_exp }
        for {set j 0} {$j<$top} {incr j} {
            set par_sim_mean($i,$j)  ""
            set par_sim_var($i,$j)   ""
            set par_sim_lower($i,$j) ""
            set par_sim_upper($i,$j) ""
        }
    }
    set simfile [open "simout"]
    while {[gets $simfile line]>=0} {
        if [regexp {^ *sim.#} $line] {
            break
        }
        if {[llength $line]==1} { 
            set name [lindex $line 0]
            for {set i 0} {$i<$numb_par} {incr i} {
                set pattern "^$par_short($i,0)(\[0-9\]*)$"
                if [regexp $pattern $name m j] {
                    if {$j==""}  {
                        set exp_index 0 
                    } else { 
                        set exp_index [expr $j-1]
                    }
                    set par_index $i
                    set par_sim_mean($par_index,$exp_index)  ""
                    set par_sim_var($par_index,$exp_index)   ""
                    set par_sim_lower($par_index,$exp_index) ""
                    set par_sim_upper($par_index,$exp_index) ""
                    break
                }
            }
        } elseif [regexp "average *- *($FloatingpointNumber)" $line match value] {
            set par_sim_mean($par_index,$exp_index) [format "%.3e" $value]
        } elseif [regexp "sigma *- *($FloatingpointNumber)" $line match value] {
            set par_sim_var($par_index,$exp_index)  [format "%.3e" $value]
        } elseif [regexp "upper *- *($FloatingpointNumber)" $line match value] {
            set par_sim_upper($par_index,$exp_index) [format "%.3e" $value]
        } elseif [regexp "lower *- *($FloatingpointNumber)" $line match value] {
            set par_sim_lower($par_index,$exp_index) [format "%.3e" $value]
        }
    }
    close $simfile
}
proc analyzeOutputFile {} {
    global  par_short par_fit numb_par sum_of_squares variance
    global  datafile FUIresiduals FUInumbPoints FUIfit_instance FUItmp
    global  FloatingpointNumber 

    set getfits 0
#    set gf_pattern "^ *(\[0-9\]+) *($FloatingpointNumber) *($FloatingpointNumber) *($FloatingpointNumber) *$"
    set gf_pattern {^ *([0-9]+) *([0-9eE.+-]+) *([0-9eE.+-]+) *([0-9eE.+-]+) *$}
    incr FUIfit_instance
    set outfile [open "output"]
    while {[gets $outfile line]>=0} {
        if [regexp {user +defined +constants +read +from +tinp} $line] {
            while {[gets $outfile line]>=0} {
                if [regexp {^ *$} $line] {
                    break
                }
                if [regexp {([a-zA-Z0-9_]+) *= *([0-9eE.+-]+)} \
                        $line match name value] {
                    for {set i 0} {$i<$numb_par} {incr i} {
                        set pattern "^$par_short($i,0)(\[0-9\]*)$"
                        if [regexp $pattern $name m j] {
                            if {$j==""}  {set j 0 } else { 
                                set j [expr $j-1]}
                            set par_fit($i,$j) [format "%.3e" $value]
                            break
                        }
                    }
                }
            }
        }
        if [regexp {^var=(.*)$} $line match string] {
            if {![regexp {[-]?[0-9]\.[0-9]+[eE][-+]?[0-9]+[ 	]*$} $string]} {
                regexp "=\[ 	\]*(.*)\[ 	\]*$" $string match rv
                BSerror "analyzeOutput" "Data fitting not successful.  \
                    Return value for variance ($rv) not a real number"
                return
            }
        }
        if [regexp {var= +ssq/npts= +([0-9eE.+-]+)} $line match value] {
            set variance [format "%.3e" $value]
        }
        if [regexp {sum of squares = ([0-9eE.+-]+)[ 	]+ssq/\(m-n\) = ([0-9eE.+-]+)} \
            $line match value1 value2] {
            set sum_of_squares [format "%.3e" $value1]
            set variance [format "%.3e" $value2]
        }
        if [regexp {k +name of +guess for +final value} $line] {
            while {[gets $outfile line]>=0} {
                if [regexp {^ *$} $line] {
                    set getfits 1
                    break
                }
                if [regexp {([a-zA-Z0-9_]+) *([0-9eE.+-]+) *([0-9eE.+-]+)} \
                        $line match name guess result] {
                    for {set i 0} {$i<$numb_par} {incr i} {
                        set pattern "^$par_short($i,0)(\[0-9\]*)$"
                        if [regexp $pattern $name m j] {
                            if {$j==""}  {set j 0 } else { 
                                set j [expr $j-1]}
                            set par_fit($i,$j) [format "%.3e" $result]
                            break
                        }
                    }
                }
            }
        }
        if {$getfits} {
            if {[lindex $line 0]=="exp"} {
                if [info exists data_index] {
                    if {$FUInumbPoints($data_index) != $idx} {
                        BSerror "analyzeOutputFile" "Experiment [expr $data_index+1]:\
                            data or output file corrupted.  Number of points on output,\
                            $idx, unequal number of points on input,\
                            $FUInumbPoints($data_index)"
                    }
                }
                set data_index [expr [lindex $line 1]-1]
                if [info exists df] { close $df }
                set df [open "fitdata[expr $data_index+1]"]
                if [info exists sf] { close $sf }
                set sf [open "${FUItmp}plt$FUIfit_instance-$data_index.sdata" w]
                if [info exists pf] { close $pf }
                set pf [open "${FUItmp}plt$FUIfit_instance-$data_index.fit" w]
                if [info exists rf] { close $rf }
                set rf [open "${FUItmp}plt$FUIfit_instance-$data_index.residuals" w]
            } elseif [regexp $gf_pattern $line match idx dy fy residual] {
                if {[gets $df dl]<0} {
                    BSerror "analyzeOutputFile" "Data file corrupted"
                }
                set FUIresiduals($data_index,[expr $idx-1]) $residual
                set x [lindex $dl 0]
                puts $sf "$x $dy"
                flush $sf
                puts $pf "$x $fy"
                flush $pf
                puts $rf "$x $residual"
                flush $rf
            }
        }
    }
    close $outfile
    if [info exists df] { close $df }
    if [info exists sf] { close $sf }
    if [info exists pf] { close $pf }
    if [info exists rf] { close $rf }
}
proc writeTapeFile {} {
    global  data_use_nmax data_skip_first data_skip
    global  data_nmax data_numb_skip_first data_skip_intv
    global  numb_par numb_exp fit_mode 
    global  number_of_simulations sim_noise confidence do_jacknife cc
    global  par_short par_fixed par_keep par_pos par_guess
    global  two_column_data numb_dependent_vars
    global  datafile FUInumbPoints FUIcurrentDataIndex FUIdataChanged
    global  filenameByBrowser

    if {![regexp {^[0-9]+$} $number_of_simulations]} {
        BSerror "writeTapefile" "Number of simulations must be a non-negative integer"
        return
    }
    if {![regexp {^[0-9]+$} $confidence] || $confidence > 100} {
        BSerror "writeTapefile" "Confidence must be in interval [0,100]"
        return
    }
    if {![regexp {^[1-9][0-9]*$} $cc(maxfev)]} {
        BSerror "writeTapeFile" "Maximum number of function evaluations must be a positive integer"
        return
    }
    set par_fixed_numb 0
    set par_free_numb  0
    set par_keep_numb  0
    set par_pos_numb   0
    set par_fixed_list ""
    set par_free_list  ""
    set par_keep_list  ""
    set par_pos_list   ""
    for {set i 0; set j 1} {$i<$numb_par} {incr i} {
        if $par_keep($i,0) { 
            set top 1
        } else {
            set top $numb_exp
        }
        for {set k 0} {$k<$top} {incr k; incr j} {
            if $par_fixed($i,0) {
                incr par_fixed_numb
                set par_fixed_list "$par_fixed_list$j "
            } else {
                incr par_free_numb
                set  par_free_list "$par_free_list$j "
            }
            if $par_keep($i,0) {
                incr par_keep_numb
                set  par_keep_list "$par_keep_list$j "
            }
            if $par_pos($i,0) {
                incr par_pos_numb
                set  par_pos_list "$par_pos_list$j "
            }
        }
    }
    set line_count_list ""
    for {set i 0} {$i<$numb_exp} {incr i} {
        set datafile($i) $filenameByBrowser(data$i)
        set df [open $datafile($i)]
        set fitdata    [open "fitdata[expr $i+1]" w]
        set guessdata  [open "guessdata[expr $i+1]" w]
        set primitive_count -1
        set count 0
        while {[gets $df line]>=0} {
            if [regexp {^[ 	]*([\#\%].*)?$} $line match]  continue
            if [info exists numb_columns] {
                if {[llength $line]!=$numb_columns} {
                    BSerror "writeTapefile" "Experiment [expr $count+1]:\
                        inconsistent data file...  Found $numb_columns columns\
                        previously and [llength $line] columns now"
                    break
                }
            } else {
                set numb_columns [llength $line]
                if [expr ($numb_columns>2) && $two_column_data] {
                    if {$FUIdataChanged } { askForDataColumns $i }
                    set x_index $FUIcurrentDataIndex($i,x)
                    set y_index $FUIcurrentDataIndex($i,y)
                } elseif {$numb_columns<2} {
                    BSerror "writeTapefile" "Experiment [expr $count+1]:\
                        not enough entries in $datafile($i)...  Expecting\
                        at least 2 columns"
                    break
                } else {
                    set x_index 0
                    set y_index 1
                }
            }
            incr primitive_count
            if {$data_skip_first($i) && $primitive_count<$data_numb_skip_first($i)} continue
            if {$data_skip($i) && [expr $primitive_count%$data_skip_intv($i)]} continue
            incr count
            if {$two_column_data} {
                puts $fitdata "[lindex $line $x_index] [lindex $line $y_index]"
                puts $guessdata "[lindex $line $x_index] [lindex $line $y_index]"
            } else {
                puts $fitdata $line
                puts $guessdata "[lindex $line 0] [lindex $line 1]"
            }
        }
        unset numb_columns
        close $df
        close $fitdata
        close $guessdata
        set FUInumbPoints($i) $count
        if $data_use_nmax($i) {
            set c [expr $data_nmax($i)<$count ? $data_nmax($i) : $count]
            set line_count_list "$line_count_list$c "
        } else {
            set line_count_list "$line_count_list$count "
        }
    }
    set FUIdataChanged 0
    set tape [open "fittp" w]
    puts $tape "=>n (# free params), nfix1, nfix2 (# fixed params line 4 and 5 respective)"
    puts $tape "$par_free_numb $par_fixed_numb 0"
    puts $tape "=>parameter numbers (set by list below) listing free first then fixed"
    puts $tape "$par_free_list$par_fixed_list"
    puts $tape " "
    puts $tape "=>how many data files?"
    puts $tape "$numb_exp"
    puts $tape "=>how many points in each data file?"
    puts $tape "$line_count_list"
    puts $tape "=>do you want data fitted simultaneously (0) or separately (1)?"
    puts $tape "$fit_mode"
    puts $tape "=>how many parms must remain positive?"
    puts $tape "$par_pos_numb"
    puts $tape "=> parameter numbers (set by list below) that must remain positive"
    puts $tape "$par_pos_list"
    puts $tape "=> how many parameters will remain the same for each experiment?"
    puts $tape "$par_keep_numb"
    puts $tape "=> list parameters that will remain same"
    puts $tape "$par_keep_list"
    puts $tape "=>how many simulations do you want to do to find parm. errors? (0=none)"
    puts $tape "$number_of_simulations"
    puts $tape "=>If plotting fit, do yau want plots of all simulations? (0=no, 1=yes)"
    puts $tape "0"
    puts $tape "=>in the simulations, what distribution of noise do you want?"
    puts $tape "=> (1=uniform, 2=gaussian, 3=bootstrap)"
    puts $tape "$sim_noise"
    puts $tape "=>do you want option to remove bad simulations before plotting (0=no,1=yes)"
    puts $tape "0"
    puts $tape "=>do you want to perform jacknife fitting?  (0=no, 1=yes)"
    puts $tape "$do_jacknife"
    puts $tape "=>what % confidence interval do you want?"
    puts $tape "$confidence"
    puts $tape "=>nvar (number of columns in file 'fitdata')"
    puts $tape [expr $two_column_data ? 2 : $numb_dependent_vars+1]
    puts $tape "=>convergence criteria: maxfev, ftol, xtol, gtol"
    puts $tape "$cc(maxfev)  $cc(ftol)  $cc(xtol)  $cc(gtol)"
    puts $tape "=>parameter names and guesses (Lbound, Ubound, and scale_val are IMSL only)"
    puts $tape "prname  guess"
    for {set i 0} {$i<$numb_par} {incr i} {
        if {$numb_exp<2 || $par_keep($i,0)} {
            puts $tape [format "%6s  %s" [string range $par_short($i,0) 0 6] \
                    [format "%.4e" $par_guess($i,0)]]
        } else {
            for {set k 0; set j 1} {$k<$numb_exp} {incr k; incr j} {
                set var [string range $par_short($i,0) 0 4]$j
                puts $tape [format "%6s  %s" $var \
                    [format "%.4e" $par_guess($i,$k)]]
            }
        }
    }
    puts $tape "---------------------------------------------------------------------"
    puts $tape "                       GRAPHICS INFORMATION"
    puts $tape "--------------------------------------------------------------------------"
    puts $tape "=>graphics data: 0=no graphics  1=x,y  2=x,logy  3=logx,y or 4=log,log"
    puts $tape "0"
    puts $tape "=>Is graph going to be 1:linear or 2:logarithmic?"
    puts $tape "1"
    puts $tape "=>enter min, step, max for (1) or min, #pts, max for (2), plot every nth pt"
    puts $tape "0.0   1.0   1.0   3"
    puts $tape "=>type title below ending with a \$. (use \" \$\" for no title)"
    puts $tape "test\$"
    puts $tape "=>type x-label below ending with a \$"
    puts $tape "time (seconds)$"
    puts $tape "=>type y-label below ending with a \$"
    puts $tape "relative fluorescence\$"
    puts $tape "=>type x-origin, x-step, x-max separated by spaces. (provide dummy value"
    puts $tape "=>for x-step in cases 2 & 4."
    for {set i 0} {$i<$numb_exp} {incr i} {    puts $tape "0.0   100   400" }
    puts $tape "=>type y-origin, y-step, y-max separated by spaces. (provide dummy value"
    puts $tape "=>for y-step in cases 3 & 4."
    for {set i 0} {$i<$numb_exp} {incr i} { puts $tape "7.0  1.00  10.0" }
    puts $tape "=>do you want parameters on plot?"
    puts $tape "=>0=no, 1=upper left, 2=lower left, 3=upper right, 4=lower right"
    puts $tape "4"
    puts $tape "=>do you wants constants on plot?"
    puts $tape "=>0=no, 1=upper left, 2=lower left, 3=upper right, 4=lower right"
    puts $tape "2"
    close $tape
}
proc checkConsistency {} {
    global modelname par_name par_short numb_par

    set pattern {([a-zA-Z0-9_]+)[ 	]+([a-zA-Z0-9_]+)[ 	]+([a-zA-Z0-9]+)[ 	]*$}
    set file [open $modelname]
    while {[gets $file line] >= 0}  {
        if [regexp -nocase "Begin: *Synonyms" $line] {
            set count 0
            while {[gets $file line] >= 0} {
                if [regexp -nocase "End: *Synonyms" $line] {
                    close $file
		    if {$count == $numb_par} { return 1 } else { return 0 }
                }
                if [regexp $pattern $line match synonym name short_name] {
                    set i 0
                    while {$i<$numb_par} {
                        if {[string compare $par_name($i,0) $name] == 0 &&
                            [string compare $par_short($i,0) $short_name] == 0} { 
                                incr count
                                break 
                        }
			incr i
			if {$i == $numb_par} {
                            BSerror "checkConsistency" \
                                "Unable to find parameter: $name"
                        }
                    }
                }
            }
        }
    }
    close $file
}	
proc calcNdrawGuess {} {
    global modelname fit_status FORTRAN_has_C_main
    global numb_exp datafile FUItmp two_column_data numb_dependent_vars
    global FUInextcolor FUIplotMaxFrame FUIplotMultiple
    global TBScurrentFrame TBS_linecolorList

    regexp {\.([cf])} $modelname match model_suffix
    regsub {\.[cf]} $modelname {} stripped_modelname
    set parfile_name "${FUItmp}par.$stripped_modelname"
    writeParameters $parfile_name
    if { $model_suffix != "c" && $model_suffix != "f" } {
        BSerror "drawGuess" "Unknown model suffix: $model_suffix"
	return 0
    } elseif { $model_suffix=="f" && !$FORTRAN_has_C_main } {
        BSerror "drawGuess" "Command not operating with FORTRAN code"
        return 0
    } else {
        exec make -k $stripped_modelname >&dump
    }
    exec $stripped_modelname -guess $parfile_name >& "$stripped_modelname-guess.log"
    if {[info exists TBScurrentFrame] && 
        [winfo exists .g$TBScurrentFrame]} {
        if {$FUIplotMultiple} { 
            set TBScurrentFrame [expr $TBSplotMaxFrame-$numb_exp] 
        } else {
            set TBScurrentFrame $FUIplotMaxFrame
        }
        for {set j 0} {$j<$numb_exp} {incr j} {
            if {$FUIplotMultiple} { incr TBScurrentFrame }
            if {$two_column_data} { set top 1 } else { set top $numb_dependent_vars }
            for {set k 1} {$k<=$top} {incr k} {
                set guessfile "${FUItmp}guess.fitdata[expr $j+1]-$k"
                TBSloadFile $guessfile -style bezier -color [lindex $TBS_linecolorList $FUInextcolor]
                incr FUInextcolor
                if {$FUInextcolor>=[llength $TBS_linecolorList]} { set FUInextcolor 0 }
            }
        }
    } else { 
        return [BSwarning "drawGuess" "Cannot find current plot frame"]
    }
    return 1
}
proc writeParameters filename {
    global  par_guess par_short par_keep two_column_data
    global  numb_par numb_exp numb_dependent_vars datafile FUItmp

    set pf [open $filename w]
    if {$two_column_data} {
        puts $pf "$numb_par  $numb_exp  1"
    } else {
        puts $pf "$numb_par  $numb_exp  $numb_dependent_vars"
    }
    for {set i 0} {$i<$numb_par} {incr i} {
        for {set j 0} {$j<$numb_exp} {incr j} {
            if {$par_keep($i,0)} {
                set outstring [format "%8s%12.3e" $par_short($i,0) $par_guess($i,0)]
            } else {
                set outstring [format "%8s%12.3e" $par_short($i,0) $par_guess($i,$j)]
            }
            puts $pf "$outstring"
        }
    }
    for {set j 0} {$j<$numb_exp} {incr j} {
        puts $pf "guessdata[expr $j+1]"
    }
    for {set j 0} {$j<$numb_exp} {incr j} {
        puts $pf "${FUItmp}guess.fitdata[expr $j+1]"
    }
    close $pf
}

proc copyFit {} {
    global  numb_par numb_exp
    global  par_guess par_fit par_keep

    for {set i 0} {$i<$numb_par} {incr i} {
        if $par_keep($i,0) { set top 1 } else { set top $numb_exp }
        for {set j 0} {$j<$top} {incr j} {
            set par_guess($i,$j) $par_fit($i,$j)
        }
    }
}
proc askForDataColumns {id} {
    global  FUIstandardFont current_datafile_index datafile

    set current_datafile_index $id
    if [winfo exists .a4dc] { raise .a4dc } else {
        toplevel .a4dc
        wm title .a4dc "FUI: data columns"
        label .a4dc.prompt -text "Data columns in file: $datafile($id)" \
            -font $FUIstandardFont(popup)
        pack .a4dc.prompt -padx 2m -pady 2m -anchor w
        frame .a4dc.f1
        frame .a4dc.f2
        frame .a4dc.f3
        frame .a4dc.f4
        frame .a4dc.f5
        label .a4dc.lx -text "x value" -font $FUIstandardFont(popup)
        label .a4dc.ly -text "y value" -font $FUIstandardFont(popup)
        entry .a4dc.ex -textvariable col_index(x) -width 4 -relief sunken \
            -font $FUIstandardFont(popupEntry)
        entry .a4dc.ey -textvariable col_index(y) -width 4 -relief sunken \
            -font $FUIstandardFont(popupEntry)
        pack .a4dc.lx -ipadx 4m -side left -in .a4dc.f4
        pack .a4dc.ex -padx 2m -side left -in .a4dc.f4
        pack .a4dc.ly -ipadx 4m -side left -in .a4dc.f5
        pack .a4dc.ey -padx 2m -side left -in .a4dc.f5
        pack .a4dc.f4 .a4dc.f5 -padx 2m -side top -in .a4dc.f3
        button .a4dc.ok -text Ok -font $FUIstandardFont(popup) -command {
            global  FUIcurrentDataIndex current_datafile_index
            set id $current_datafile_index
            set FUIcurrentDataIndex($id,x) [expr $col_index(x)-1] 
            set FUIcurrentDataIndex($id,y) [expr $col_index(y)-1]
            destroy .a4dc
            update
        }
        button .a4dc.cancel -text Cancel -font $FUIstandardFont(popup) -command {
            destroy .ad4c
        }
        pack .a4dc.ok .a4dc.cancel -ipadx 6m -pady 2m -side top -fill x -in .a4dc.f2
        pack .a4dc.f3 .a4dc.f2 -padx 3m -side left -in .a4dc.f1
        pack .a4dc.f1 -pady 2m 
        bind .a4dc.ex <Tab>  { focus .a4dc.ey }
        bind .a4dc.ex <Return> {
            global  FUIcurrentDataIndex current_datafile_index
            set id $current_datafile_index
            set FUIcurrentDataIndex($id,x) [expr $col_index(x)-1] 
            set FUIcurrentDataIndex($id,y) [expr $col_index(y)-1]
            destroy .a4dc
            update
        }
        bind .a4dc.ey <Tab>  { focus .a4dc.ex }
        bind .a4dc.ey <Return> {
            global  FUIcurrentDataIndex current_datafile_index
            set id $current_datafile_index
            set FUIcurrentDataIndex($id,x) [expr $col_index(x)-1] 
            set FUIcurrentDataIndex($id,y) [expr $col_index(y)-1]
            destroy .a4dc
            update
        }
        focus .a4dc.ex
        tkwait window .a4dc
    }
}