
proc  getModelSpecifications {} {
    global  filenameByBrowser modelname tk_version
    global  FUIstandardFont FUIbackground

    set filenameByBrowser(model_specs) ""
    if [winfo exists .ms] { raise .ms } else {
        toplevel .ms
        wm title .ms "FUI: model file"
        label .ms.l -text "Select model file" -font $FUIstandardFont(popup)
        frame .ms.f
        button .ms.bd -text "Model File" -font $FUIstandardFont(popup) \
            -command "getFileNameByBrowser . model_specs" -relief flat
        entry .ms.ed -width 35 -relief sunken -bd 2 \
            -textvariable filenameByBrowser(model_specs) -bd 2 \
            -font $FUIstandardFont(popupEntry)
        bind .ms.ed <Return> {
            set fullname "./$filenameByBrowser(model_specs)"
            regexp {^(.*)/([^/]+)$} $fullname match directory modelname
            cd $directory
            wm title . "FUI: $modelname"
            readModelSpecifications $modelname
            destroy .ms } 
        focus .ms.ed
        pack .ms.bd -ipadx 5m -pady 3m -side left -in .ms.f
        pack .ms.ed -padx 5m -pady 3m -side left -in .ms.f
        pack .ms.f
    
        frame .ms.f2
        button .ms.ok -text Ok -font $FUIstandardFont(popup) -command {
            set fullname "./$filenameByBrowser(model_specs)"
            regexp {^(.*)/([^/]+)$} $fullname match directory modelname
            cd $directory
            wm title . "FUI: $modelname"
            readModelSpecifications $modelname
            destroy .ms }
        button .ms.cancel -text Cancel -command { destroy .ms } \
            -font $FUIstandardFont(popup)
	frame .ms.spacer -width 20m
        button .ms.opt -text "Options..." -relief raised -font $FUIstandardFont(1)
        bind .ms.opt <1> {  createModelOptionSelection  }
        pack .ms.ok .ms.cancel -ipadx 6m -padx 6m -pady 3m -side left -in .ms.f2
	pack .ms.spacer -side left -in .ms.f2
	pack .ms.opt -ipadx 3m -padx 2m -pady 3m -side left -in .ms.f2
        pack .ms.f2 -anchor w
        if {$tk_version>=4.0} {
            .ms.bd configure -highlightbackground $FUIbackground
            .ms.ed configure -highlightbackground $FUIbackground
            .ms.ok configure -highlightbackground $FUIbackground
            .ms.cancel configure -highlightbackground $FUIbackground
            .ms.bd configure -takefocus 0
            .ms.ok configure -takefocus 0
            .ms.cancel configure -takefocus 0
        }
    }
}
proc readModelSpecifications filename {
    global  par_name par_short numb_par numb_par_old numb_exp
    global  two_column_data numb_dependent_vars FORTRAN_has_C_main
    global  residual_scale_type

    set nvar ""
    set two_column_data 1
    set numb_par_old $numb_par
    set pattern {([a-zA-Z0-9_]+)[ 	]+([a-zA-Z0-9_]+)[ 	]+([a-zA-Z0-9]+)[ 	]*$}
    set file [open $filename]
    while {[gets $file line] >= 0}  {
        if [regexp {HaveCencodedMain} $line] {
            set FORTRAN_has_C_main 1
        }
        if [regexp {[ 	]*FitDataFormat:[ 	]*([a-zA-Z]+)} $line match format] {
            if [string match TwoColumn $format] {
                set two_column_data 1
            } elseif [string match MultiColumn $format] {
                set two_column_data 0
            } else {
                BSerror "Model Specification" \
                    "Unknown format for fitdata.\nContinuing with two-column format..."
            }
        }
        if [regexp {[ 	]*NumbDepVars:[ 	]*([0-9]+)} $line match nvar] {
            if [regexp {[0-9]+} $nvar] {
                set numb_dependent_vars $nvar
            } else {
                BSerror "Model Specification" \
                    "\"$nvar\" not a proper integer.\nContinuing with NumbDepVars = 1."
            }
        }
        if [regexp {[ 	]*ResScaleType:[ 	]*(abs|rel|log)} $line match nvar] {
            set residual_scale_type $nvar
        } elseif [regexp {[ 	]*ResScaleType:} $line] {
            BSerror "Model Specification" \
                "\"$nvar\" not a proper flag value.\nContinuing with ResScaleType = abs."
        }
        if [regexp -nocase {Begin:[ 	]*Synonyms} $line] {
            set count 0
            while {[gets $file line] >= 0} {
                if [regexp -nocase {End:[ 	]*Synonyms} $line] {
                    set numb_par $count
                    if [winfo exists .p] {
                        updateParDescription $numb_par $numb_par_old \
                                    $numb_exp $numb_exp
                    }
                    return
                }
                if [regexp $pattern $line match synonym name short_name] {
                    set par_name($count,0) $name
                    set par_short($count,0) $short_name
                    incr count
                }
            }
        }
    }
    close $file
}
proc createModelOptionSelection {} {
    global  FUIstandardFont FUIbackground tk_version
    global  two_column_data numb_dependent_vars

    if [winfo exists .mos] { raise .mos } else {
        toplevel .mos
        wm title .mos "FUI: Model Options"
        button .mos.close -text "C L O S E" -font $FUIstandardFont(1) -command {
            destroy .mos
        }
        pack .mos.close -ipadx 3m -padx 3m -pady 4m -side top -anchor e
        frame .mos.f0
        label .mos.res_scale_l1 -text "Compute residuals on" -font $FUIstandardFont(1)
        entry .mos.res_scale -relief sunken -width 5 -font $FUIstandardFont(popupEntry) \
            -textvariable residual_scale_type
        label .mos.res_scale_l2 -text "scale (choose `abs' or `rel' or `log')" -font $FUIstandardFont(1)
        pack .mos.res_scale_l1 -padx 3m -side left -in .mos.f0
        pack .mos.res_scale -side left -in .mos.f0
        pack .mos.res_scale_l2 -padx 3m -side left -in .mos.f0
        frame .mos.vspc0 -height 2m
        pack .mos.f0 .mos.vspc0 -anchor w
        frame .mos.hrule -width 100m -height 1m -relief groove
        pack .mos.hrule
        frame .mos.f1
        label .mos.data_fmt_l1 -text "Enter data in" -font $FUIstandardFont(1)
        radiobutton .mos.tcd -variable two_column_data -value 1 -font $FUIstandardFont(popup) \
            -text "two-column" -command "updateModelOptions"
        radiobutton .mos.mcd -variable two_column_data -value 0 -font $FUIstandardFont(popup) \
            -text "multi-column" -command "updateModelOptions"
        label .mos.data_fmt_l2 -text "format." -font $FUIstandardFont(1)
        pack .mos.data_fmt_l1 .mos.tcd .mos.mcd .mos.data_fmt_l2 -padx 3m -side left -in .mos.f1
        frame .mos.vspc1 -height 1m
        pack .mos.f1 -pady 2m -anchor w
        if {$tk_version>=4.0} {
            .mos.tcd configure -highlightbackground $FUIbackground
            .mos.mcd configure -highlightbackground $FUIbackground
        }
        if {$two_column_data < 1} {
            label .mos.mcd_label -text "MULTI-COLUMN FORMAT:" -font $FUIstandardFont(popup)
            frame .mos.f3
            entry .mos.ndv -relief sunken -width 5 -font $FUIstandardFont(popupEntry) \
                -textvariable numb_dependent_vars
            label .mos.ndv_l -text "dependent variables" -font $FUIstandardFont(popup)
            pack .mos.ndv .mos.ndv_l -padx 3m -side left -in .mos.f3
            frame .mos.vspc2 -height 3m
            pack .mos.mcd_label -padx 3m -side top
            pack .mos.f3 .mos.vspc2 -side top -anchor w
            focus .mos.ndv
        }
    }
}
proc updateModelOptions {} {
    global FUIstandardFont FUIbackground tk_version 
    global two_column_data numb_dependent_vars

    if {$two_column_data < 1} {
        label .mos.mcd_label -text "MULTI-COLUMN FORMAT:" -font $FUIstandardFont(popup)
        frame .mos.f3
        entry .mos.ndv -relief sunken -width 5 -font $FUIstandardFont(popupEntry) \
            -textvariable numb_dependent_vars
        label .mos.ndv_l -text "dependent variables" -font $FUIstandardFont(popup)
        pack .mos.ndv .mos.ndv_l -padx 3m -side left -in .mos.f3
        frame .mos.vspc2 -height 3m
        pack .mos.mcd_label -padx 3m -side top
        pack .mos.f3 .mos.vspc2 -side top -anchor w
        focus .mos.ndv
    } else {
        destroy .mos.mcd_label .mos.f3 .mos.ndv .mos.ndv_l .mos.vspc2
    }
}
