
proc TBSparseCommandLine {argc argv opt filename sty sym wd col} {
    upvar $argv     av
    upvar $filename file
    upvar $sty      style
    upvar $sym      symbol
    upvar $wd       width
    upvar $col      color
    upvar $opt      option

    set top [expr $argc/2]
    for {set l 1; set k 0; set i 0} {$i<$top} {incr i; incr k 2; incr l 2} { 
        if [regexp {\-(file|style|symbol|width|color)([0-9]+)} \
                    [lindex $av $k] m name mi] {
            set ri($mi) 1
            set rv($name,$mi) [lindex $av $l]
        } elseif [regexp {\-(graphtype|xlabel|ylabel|xtitle|ytitle)} \
                    [lindex $av $k] m name] {
            if {$name=="xlabel" || $name=="ylabel"} {
                set value [lindex $av $l]
                if {$value=="decimal" || $value=="power"} {
                    set option($name) $value
                } else {
                    set option($name) decimal
                }
            } else {
                set option($name) [lindex $av $l]
            }
        } else {
            puts "Ignoring unknown option: [lindex $av $k]"
        }
    }
    set index 0
    if [info exists ri] {
        foreach i [lsort [array names ri]] {
            if [info exists rv(file,$i)] { 
                set file($index)   $rv(file,$i) 
            } else { 
                puts "Error: no file specified for set $index" 
                exit
            }
            if [info exists rv(style,$i)] { 
                set style($index)  $rv(style,$i) 
            } else { 
                set style($index)  "" 
            }
            if [info exists rv(symbol,$i)] { 
                set symbol($index) $rv(symbol,$i) 
            } else { 
                set symbol($index) "" 
            }
            if [info exists rv(width,$i)] { 
                set width($index)  $rv(width,$i) 
            } else { 
                set width($index)  "" 
            }
            if [info exists rv(color,$i)] { 
                set color($index)  $rv(color,$i) 
            } else { 
                set color($index)  "" 
            }
            incr index
        }
    }
    return $index
}
proc TBScmdLaunch {opt numbSets filename sty sym wd col} {
    global  TBS_symbolList TBS_linewidthList TBS_linecolorList
    global  TBSgraphType TBSaxisTitle TBSaxisLabelStyle
    upvar $opt      option
    upvar $filename file
    upvar $sty      style
    upvar $sym      symbol
    upvar $wd       width
    upvar $col      color 

    set numbSymbols [llength $TBS_symbolList]
    set numbWidths  [llength $TBS_linewidthList]
    set numbColors  [llength $TBS_linecolorList]
    set nextsymbol 0
    set nextwidth  0
    set nextcolor  0
    for {set i 0} {$i<$numbSets} {incr i} {
        if {$style($i)==""} {
            set c_sty line
        } else {
            set c_sty $style($i)
        }
        if {$symbol($i)==""} {
            set c_sym [lindex $TBS_symbolList $nextsymbol]
            set nextsymbol [expr ($nextsymbol+1)%$numbSymbols]
        } else {
            set c_sym [lindex $TBS_symbolList [expr ($symbol($i)-1)%$numbSymbols]]
        }
        if {$width($i)==""} {
            set c_wd [lindex $TBS_linewidthList $nextwidth]
            set nextwidth [expr ($nextwidth+1)%$numbWidths]
        } else {
            set c_wd [lindex $TBS_linewidthList [expr ($width($i)-1)%$numbWidths]]
        }
        if {$color($i)==""} {
            set c_col [lindex $TBS_linecolorList $nextcolor]
            set nextcolor [expr ($nextcolor+1)%$numbColors]
        } else {
            set c_col [lindex $TBS_linecolorList [expr ($color($i)-1)%$numbColors]]
        }
        setPlotAttributes $i 1 $c_sty $c_sym $c_wd $c_col
    }
    if {$numbSets<1} {
        set c_sty line
        set c_sym $nextsymbol
        set c_wd $nextwidth
        set c_col $nextcolor
        setPlotAttributes 0 1 $c_sty $c_sym $c_wd $c_col
    }
    if [info exists option(graphtype)] {
        set TBSgraphType(1) [expr $option(graphtype)%4]
    } else {
        set TBSgraphType(1) 0
    }
    if [info exists option(xlabel)] {
        set TBSaxisLabelStyle(1,x) $option(xlabel)
    } else {
        set TBSaxisLabelStyle(1,x) decimal
    }
    if [info exists option(ylabel)] {
        set TBSaxisLabelStyle(1,y) $option(ylabel)
    } else {
        set TBSaxisLabelStyle(1,y) decimal
    }
    if [info exists option(xtitle)] {
        set TBSaxisTitle(1,x) $option(xtitle)
    } else {
        set TBSaxisTitle(1,x) ""
    }
    if [info exists option(ytitle)] {
        set TBSaxisTitle(1,y) $option(ytitle)
    } else {
        set TBSaxisTitle(1,y) ""
    }
    mapPlot file $numbSets "TBSplot"
}
