
proc TBSredrawAxesLabels {win frame} {
    $win delete tics_n_labels
    autoScale  as_min as_max tic_incr $frame
    drawAxes $win $frame as_min as_max tic_incr
}
proc TBSdraw {win frame} {
    global  TBSnumbSets

    autoScale  as_min as_max tic_incr $frame
    for {set i 0} {$i<$TBSnumbSets($frame)} {incr i} {
        TBSplot $win $i $frame
    }
    drawAxes $win $frame as_min as_max tic_incr
}
proc TBSredraw {win frame} {
    global TBSnumbSets TBSsetStatus

    autoScale  as_min as_max tic_incr $frame
    for {set i 0} {$i<$TBSnumbSets($frame)} {incr i} {
        if {$TBSsetStatus($frame,$i)!="deleted"} {
            $win delete data$i
            TBSplot $win $i $frame
            if {$TBSsetStatus($frame,$i)=="hidden"} { $win lower data$i priming }
        }
    }
    $win delete tics_n_labels
    drawAxes $win $frame as_min as_max tic_incr
    $win raise viewframe
}
proc TBSrefresh {win index frame} {
    global  TBS_symbol TBS_lineWidth TBS_lineColor TBS_smooth
    global  TBS_plotAttributes

    autoScale as_min as_max tic_incr $frame
    if {$TBS_plotAttributes($index,$frame,style)=="marker"} {
        set TBS_symbol $TBS_plotAttributes($index,$frame,symbol)
        XY_PlotPolymarker $win $index $frame
    } else {
        if {$TBS_plotAttributes($index,$frame,style)=="bezier"} {
            set TBS_smooth 1
        } else {
            set TBS_smooth 0
        }
        set TBS_lineColor $TBS_plotAttributes($index,$frame,color)
        set TBS_lineWidth $TBS_plotAttributes($index,$frame,width)
        XY_PlotPolyline   $win $index $frame
    }
}
proc TBSplot {win index frame} {
    global  TBS_symbol TBS_lineWidth TBS_lineColor TBS_smooth
    global  TBS_plotAttributes

    if {$TBS_plotAttributes($index,$frame,style)=="marker"} {
        set TBS_symbol $TBS_plotAttributes($index,$frame,symbol)
        XY_PlotPolymarker $win $index $frame
    } else {
        if {$TBS_plotAttributes($index,$frame,style)=="bezier"} {
            set TBS_smooth 1
        } else {
            set TBS_smooth 0
        }
        set TBS_lineColor $TBS_plotAttributes($index,$frame,color)
        set TBS_lineWidth $TBS_plotAttributes($index,$frame,width)
        XY_PlotPolyline   $win $index $frame
    }
}
proc XY_PlotPolymarker {win index frame} {
    global  TBSgraphFunction 
    global  TBSnumbPoints TBSdataCoordinates
    global  TBS_symbol

    set fx $TBSgraphFunction($frame,x)
    set fy $TBSgraphFunction($frame,y)
    for {set i 0} {$i<$TBSnumbPoints($index,$frame)} {incr i} {
        DC2VPC [expr ${fx}($TBSdataCoordinates($i,$index,$frame,x))] \
            [expr ${fy}($TBSdataCoordinates($i,$index,$frame,y))] c
        $TBS_symbol $win $c(x) $c(y) $index
    }
}
proc XY_PlotPolyline {win index frame} {
    global  TBSgraphFunction 
    global  TBSnumbPoints TBSdataCoordinates
    global  TBS_lineWidth TBS_lineColor TBS_smooth

    set fx $TBSgraphFunction($frame,x)
    set fy $TBSgraphFunction($frame,y)
    set drawCommand "$win create line"
    for {set i 0} {$i<$TBSnumbPoints($index,$frame)} {incr i} {
        DC2CC [expr ${fx}($TBSdataCoordinates($i,$index,$frame,x))] \
            [expr ${fy}($TBSdataCoordinates($i,$index,$frame,y))] c
        set drawCommand "$drawCommand $c(x) $c(y)"
    }
    set drawCommand "$drawCommand -fill $TBS_lineColor -width $TBS_lineWidth \
        -smooth $TBS_smooth -tags data$index"
    eval $drawCommand
}
proc drawAxes {win frame minimum maximum tic_increment} {
    global   TBSstandardFont TBSgraphFunction 
    global   TBSaxisLabelStyle TBSaxisTitle
    upvar   $minimum       as_min
    upvar   $maximum       as_max
    upvar   $tic_increment tic_incr

        set om [expr log10($tic_incr(x))]
    if {$om < 0.0} { 
            set prec [expr int(ceil(-$om))]
        } else {
            set prec 0
        }
    VPC2CC 0  0    tc2
    VPC2CC 0 -0.05 tc3
    VPC2CC 0 -0.11 tc4
    for {set xtic $as_min(x)} {$xtic<=$as_max(x)} {set xtic [expr $xtic+$tic_incr(x)]} {
        DC2CC $xtic  0 tc1
        $win create line $tc1(x) $tc2(y) $tc1(x) $tc3(y) -tags "xtics tics_n_labels"
        if {$TBSgraphFunction($frame,x)=="log10"} {
            if {$TBSaxisLabelStyle($frame,x)=="decimal"} {
                set t [expr pow(10,$xtic)]
                if {$xtic<0} { set prec [expr int(-1*$xtic)] } else { set prec 0 }
                $win create text $tc1(x) $tc4(y) -text [format "%.${prec}f" $t] \
                    -anchor s -tags "xtic_labels tics_n_labels" \
                    -font $TBSstandardFont(ticLabel)
            } else {
                $win create text $tc1(x) $tc4(y) -text 10 \
                    -anchor e -tags "xtic_labels tics_n_labels" \
                    -font $TBSstandardFont(ticLabel)
                $win create text $tc1(x) $tc4(y) -text [format "%.${prec}f" $xtic] \
                    -anchor sw -tags "xtic_labels tics_n_labels" \
                    -font $TBSstandardFont(ticLabelSuperscript)
            }
        } else {
            $win create text $tc1(x) $tc4(y) -text [format "%.${prec}f" $xtic] \
                -anchor s -tags "xtic_labels tics_n_labels" \
                -font $TBSstandardFont(ticLabel)
        }
    }
    VPC2CC 0.5 -0.22 tc10
    $win create text $tc10(x) $tc10(y) -text $TBSaxisTitle($frame,x) -anchor s \
        -tags "xtitle tics_n_labels" -font $TBSstandardFont(ticLabel)
        set om [expr log10($tic_incr(y))]
    if {$om < 0.0} { 
            set prec [expr int(ceil(-$om))]
        } else {
            set prec 0
        }
    VPC2CC  0    0 tc2
    VPC2CC -0.0333333 0 tc3
    VPC2CC -0.04 0 tc4
    VPC2CC -0.08 0 tc5
    for {set ytic $as_min(y)} {$ytic<=$as_max(y)} {set ytic [expr $ytic+$tic_incr(y)]} {
        DC2CC  0 $ytic tc1
        $win create line $tc2(x) $tc1(y) $tc3(x) $tc1(y) -tags "ytics tics_n_labels"
        if {$TBSgraphFunction($frame,y)=="log10"} {
            if {$TBSaxisLabelStyle($frame,y)=="decimal"} {
                set t [expr pow(10,$ytic)]
                if {$ytic<0} { set prec [expr int(-1*$ytic)] } else { set prec 0 }
                $win create text $tc4(x) $tc1(y) -text [format "%.${prec}f" $t] \
                    -anchor e -tags "ytic_labels tics_n_labels" \
                    -font $TBSstandardFont(ticLabel)
            } else {
                $win create text $tc5(x) $tc1(y) -text 10 \
                    -anchor e -tags "ytic_labels tics_n_labels" \
                    -font $TBSstandardFont(ticLabel)
                $win create text $tc5(x) $tc1(y) -text [format "%.${prec}f" $ytic] \
                    -anchor sw -tags "ytic_labels tics_n_labels" \
                    -font $TBSstandardFont(ticLabelSuperscript)
            }
        } else {
        $win create text $tc4(x) $tc1(y) -text [format "%.${prec}f" $ytic] \
             -anchor e -tags "ytic_labels tics_n_labels" -font $TBSstandardFont(ticLabel)
        }
    }
    VPC2CC -0.16 0.5 tc10
    $win create text $tc10(x) $tc10(y) -text $TBSaxisTitle($frame,y) -anchor s \
        -tags "ytitle tics_n_labels" -font $TBSstandardFont(ticLabel)
}
proc autoScale {a_minimum a_maximum tic_increment frame} {
    global   TBSgraphFunction TBSdataMinimum TBSdataMaximum
    upvar   $a_minimum     an
    upvar   $a_maximum     ax
    upvar   $tic_increment ti

    set fx $TBSgraphFunction($frame,x)
    set fy $TBSgraphFunction($frame,y)
    set range [expr ${fx}($TBSdataMaximum($frame,x))-${fx}($TBSdataMinimum($frame,x))]
    set base [expr pow(10,floor(log10($range)))]
    set b_range [expr ceil($range/$base)]
    set an(x) [expr floor(${fx}($TBSdataMinimum($frame,x))/$base)*$base]
    set ax(x) [expr ceil(${fx}($TBSdataMaximum($frame,x))/$base)*$base]
    if {$b_range < 5} { 
        set ti(x) $base
    } else { 
        set ti(x) [expr $base*2]
    }
    set range [expr ${fy}($TBSdataMaximum($frame,y))-${fy}($TBSdataMinimum($frame,y))]
    set base [expr pow(10,floor(log10($range)))]
    set b_range [expr ceil($range/$base)]
    set an(y) [expr floor(${fy}($TBSdataMinimum($frame,y))/$base)*$base]
    set ax(y) [expr ceil(${fy}($TBSdataMaximum($frame,y))/$base)*$base]
    if {$b_range < 5} { 
        set ti(y) $base
    } else { 
        set ti(y) [expr $base*2]
    }
    setDC2VPC $an(x) $ax(x) $an(y) $ax(y)
}
proc setPlotAttributes {index frame style symbol width color} {
    global  TBS_plotAttributes

    set TBS_plotAttributes($index,$frame,style)  $style
    set TBS_plotAttributes($index,$frame,symbol) $symbol
    set TBS_plotAttributes($index,$frame,width)  $width
    set TBS_plotAttributes($index,$frame,color)  $color
}
proc TBSplaceLegend {x y} {
    global TBScurrentFrame TBSstandardFont
    global TBSlegend TBSnumbSets TBSsetStatus
    global TBS_plotAttributes

    set fid $TBScurrentFrame
    set y_incr 16
    set xx [expr $x+16]
    for {set i 0} {$i<$TBSnumbSets($fid)} {incr i} {
        if {$TBSsetStatus($fid,$i)=="active"} {
            if {$TBS_plotAttributes($i,$fid,style)=="marker"} {
                set TBS_symbol $TBS_plotAttributes($i,$fid,symbol)
                CC2VPC [expr $x+5] $y c
                $TBS_symbol .g$fid.c $c(x) $c(y) $i legend
            } else {
                set TBS_lineColor $TBS_plotAttributes($i,$fid,color)
                set TBS_lineWidth $TBS_plotAttributes($i,$fid,width)
                .g$fid.c create line $x $y [expr $x+10] $y -fill $TBS_lineColor \
                    -width $TBS_lineWidth -tags "data$i legend"
            }
            .g$fid.c create text $xx $y -text $TBSlegend($fid,$i) -anchor w \
                -font $TBSstandardFont(legend) -tags "legendtext legend"
            set y [expr $y+$y_incr]
        }
    }
}
