
proc TBSinitializeStyleConstants {} {
    global TBS_symbolList TBS_linewidthList TBS_linecolorList
    global TBSstandardFont TBSstandardBackground TBSgraphType
    global frame_id env FloatingpointNumber

    set FloatingpointNumber {[-]?[0-9]*(\.[0-9]+)?([eE][-+]?[0-9]+)?}
    if {![info exists frame_id]} { set frame_id 0 }
    if {![info exists TBS_symbolList]} {
        set TBS_symbolList [list filledDiamond filledSquare filledCircle filledTriangleUp \
            filledTriangleDown filledTriangleRight filledTriangleLeft openDiamond openSquare \
            openCircle openTriangleUp openTriangleDown openTriangleRight openTriangleLeft ]
        set TBS_linewidthList [list 1 2 3 4 6 8]
        set TBS_linecolorList [list black red green blue yellow cyan magenta dimGrey grey \
            hotPink cornflowerBlue burlyWood turquoise mediumOrchid]
        set TBSstandardFont(1)  -*-helvetica-bold-r-normal--*-120-*
        set TBSstandardFont(2)  -*-times-medium-r-normal--*-140-*
        set TBSstandardFont(3)  -*-helvetica-bold-r-normal--*-140-*
        set TBSstandardFont(4)  -*-courier-bold-r-normal--*-120-*
        set TBSstandardFont(menubar)             $TBSstandardFont(3)
        set TBSstandardFont(menu)                $TBSstandardFont(3)
        set TBSstandardFont(popup)               $TBSstandardFont(3)
        set TBSstandardFont(popupEntry)          $TBSstandardFont(2)
        set TBSstandardFont(ticLabel)            $TBSstandardFont(3)
        set TBSstandardFont(ticLabelSuperscript) $TBSstandardFont(3)
        set TBSstandardFont(axisTitle)           $TBSstandardFont(3)
        set TBSstandardFont(legend)              $TBSstandardFont(4)

        if {[info exists env(HOSTNAME)] && $env(HOSTNAME)=="lop"} {
            set TBSstandardBackground(viewbox) #ddd
            set TBSstandardBackground(canvas)  #bbb
            set TBSstandardBackground(main)    #aaf
            set TBSstandardBackground(menubar) #aaf
            set TBSstandardBackground(menu)    #aaf
            set TBSstandardBackground(hilite)  cyan
            set TBSstandardBackground(select)  blue
        } else {
            set TBSstandardBackground(viewbox) #eff
            set TBSstandardBackground(canvas)  #d7c7c0
            set TBSstandardBackground(main)    #ccf
            set TBSstandardBackground(menubar) #ccf
            set TBSstandardBackground(menu)    #ccf
            set TBSstandardBackground(hilite)  #d7d7ff
            set TBSstandardBackground(select)  #a7a7ff
        }
    }
    return 1
}
proc TBSfindFrameId {pattern win} {
    global  TBScurrentFrame tk_version
    if {$tk_version<4.0} { tk_butDown $win }
    regexp $pattern $win match TBScurrentFrame
}
proc TBSsetGraphType {frame mode} {
    global TBSgraphType TBSgraphType_old TBSgraphFunction 
    global TBSaxisLabelStyle TBSaxisTitle

    set TBSgraphType($frame) $mode
    set TBSgraphType_old($frame) $TBSgraphType($frame)
    switch $mode {
        0  { set TBSgraphFunction($frame,x) "";      set TBSgraphFunction($frame,y) ""      }
        1  { set TBSgraphFunction($frame,x) "";      set TBSgraphFunction($frame,y) "log10" }
        2  { set TBSgraphFunction($frame,x) "log10"; set TBSgraphFunction($frame,y) ""      }
        3  { set TBSgraphFunction($frame,x) "log10"; set TBSgraphFunction($frame,y) "log10" }
        "" { set TBSgraphFunction($frame,x) "";      set TBSgraphFunction($frame,y) ""      }
    }
    if {![info exists TBSaxisLabelStyle($frame,x)]} { set TBSaxisLabelStyle($frame,x) decimal }
    if {![info exists TBSaxisLabelStyle($frame,y)]} { set TBSaxisLabelStyle($frame,y) decimal }
    if {![info exists TBSaxisTitle($frame,x)]}      { set TBSaxisTitle($frame,x) "" }
    if {![info exists TBSaxisTitle($frame,y)]}      { set TBSaxisTitle($frame,y) "" }
}
proc TBSdataInitialize frame {
    global  TBSdataMinimum TBSdataMaximum

    set TBSdataMinimum($frame,x)  1e300
    set TBSdataMaximum($frame,x) -1e300
    set TBSdataMinimum($frame,y)  1e300
    set TBSdataMaximum($frame,y) -1e300
}

proc TBSchangeGraphType { newValue xscale yscale } {
    global TBSgraphFunction TBSgraphType TBSgraphType_old 
    global TBScurrentFrame TBSnumbSets TBSshowLegends TBSlegendCoords
    global TBSaxisLabelStyle 

    set id $TBScurrentFrame
    if {$TBSgraphType_old($id) != $newValue} { 
        set TBSgraphFunction($id,x)  $xscale
        set TBSgraphFunction($id,y)  $yscale
        if {$xscale==""} { set TBSaxisLabelStyle($id,x) decimal }
        if {$yscale==""} { set TBSaxisLabelStyle($id,y) decimal }
        if {$TBSnumbSets($id)>0} { 
            TBSredraw .g$id.c $id 
            if $TBSshowLegends($id) {
                .g$id.c delete legend
                TBSplaceLegend $TBSlegendCoords($id,x) $TBSlegendCoords($id,y)
            }
        }
    }
    set TBSgraphType_old($id) $newValue
}
proc TBSsetCurrentStyleOptions id {
    global  TBScurrentFile TBScurrentStyle TBScurrentSymbol TBScurrentLinewidth TBScurrentLinecolor
    global  TBS_plotAttributes

    set TBScurrentStyle($id)     $TBS_plotAttributes($TBScurrentFile($id),$id,style)
    set TBScurrentSymbol($id)    $TBS_plotAttributes($TBScurrentFile($id),$id,symbol)
    set TBScurrentLinewidth($id) $TBS_plotAttributes($TBScurrentFile($id),$id,width)
    set TBScurrentLinecolor($id) $TBS_plotAttributes($TBScurrentFile($id),$id,color)
}
proc TBSsetCurrentStyle {id style} {
    global  TBScurrentFile TBScurrentSymbol TBSsetStatus
    global  TBSshowLegends TBSlegendCoords
    global  TBS_plotAttributes
    global  TBS_smooth

    if {$style=="line"} {
        if {$TBS_plotAttributes($TBScurrentFile($id),$id,style)=="bezier"} {
            set TBS_smooth 0
            set TBS_plotAttributes($TBScurrentFile($id),$id,style) line
            .g$id.c itemconfigure data$TBScurrentFile($id) -smooth $TBS_smooth
            if $TBSshowLegends($id) {
                .g$id.c delete legend
                TBSplaceLegend $TBSlegendCoords($id,x) $TBSlegendCoords($id,y)
            }
        } elseif {$TBS_plotAttributes($TBScurrentFile($id),$id,style)=="marker"} {
            .g$id.c delete data$TBScurrentFile($id)
            set TBS_plotAttributes($TBScurrentFile($id),$id,style) line
            if {$TBS_plotAttributes($TBScurrentFile($id),$id,width)==""} {
                set TBS_plotAttributes($TBScurrentFile($id),$id,width) 2
            }
            if {$TBS_plotAttributes($TBScurrentFile($id),$id,color)==""} {
                set TBS_plotAttributes($TBScurrentFile($id),$id,color) red
            }
            TBSrefresh .g$id.c $TBScurrentFile($id) $id
            if {$TBSsetStatus($id,$TBScurrentFile($id))!="active"} {
                .g$id.c lower data$TBScurrentFile($id) priming
            }
            if $TBSshowLegends($id) {
                .g$id.c delete legend
                TBSplaceLegend $TBSlegendCoords($id,x) $TBSlegendCoords($id,y)
            }
            .g$id.c raise viewframe
        }
    } elseif {$style=="bezier"} {
        if {$TBS_plotAttributes($TBScurrentFile($id),$id,style)=="line"} {
            set TBS_smooth 1
            set TBS_plotAttributes($TBScurrentFile($id),$id,style) bezier
            .g$id.c itemconfigure data$TBScurrentFile($id) -smooth $TBS_smooth
            if $TBSshowLegends($id) {
                .g$id.c delete legend
                TBSplaceLegend $TBSlegendCoords($id,x) $TBSlegendCoords($id,y)
            }
        } elseif {$TBS_plotAttributes($TBScurrentFile($id),$id,style)=="marker"} {
            .g$id.c delete data$TBScurrentFile($id)
            set TBS_plotAttributes($TBScurrentFile($id),$id,style) bezier
            if {$TBS_plotAttributes($TBScurrentFile($id),$id,width)==""} {
                set TBS_plotAttributes($TBScurrentFile($id),$id,width) 2
            }
            if {$TBS_plotAttributes($TBScurrentFile($id),$id,color)==""} {
                set TBS_plotAttributes($TBScurrentFile($id),$id,color) red
            }
            TBSrefresh .g$id.c $TBScurrentFile($id) $id
            if {$TBSsetStatus($id,$TBScurrentFile($id))!="active"} {
                .g$id.c lower data$TBScurrentFile($id) priming
            }
            if $TBSshowLegends($id) {
                .g$id.c delete legend
                TBSplaceLegend $TBSlegendCoords($id,x) $TBSlegendCoords($id,y)
            }
            .g$id.c raise viewframe
        }
    } elseif {$style=="marker"} {
        if {$TBS_plotAttributes($TBScurrentFile($id),$id,style)!="marker"} {
            .g$id.c delete data$TBScurrentFile($id)
            set TBS_plotAttributes($TBScurrentFile($id),$id,style) marker
            if {$TBS_plotAttributes($TBScurrentFile($id),$id,symbol)==""} {
                set TBS_plotAttributes($TBScurrentFile($id),$id,symbol) openTriangleUp
                set TBScurrentSymbol($id) openTriangleUp
            }
            TBSrefresh .g$id.c $TBScurrentFile($id) $id
            if {$TBSsetStatus($id,$TBScurrentFile($id))!="active"} {
                .g$id.c lower data$TBScurrentFile($id) priming
            }
            if $TBSshowLegends($id) {
                .g$id.c delete legend
                TBSplaceLegend $TBSlegendCoords($id,x) $TBSlegendCoords($id,y)
            }
            .g$id.c raise viewframe
        }
    }
}
proc TBSsetCurrentSymbol id {
    global  TBS_plotAttributes TBScurrentFile TBScurrentSymbol TBSsetStatus

    if {$TBS_plotAttributes($TBScurrentFile($id),$id,style)=="marker"} {
        global  TBSshowLegends TBSlegendCoords
        .g$id.c delete data$TBScurrentFile($id)
        set TBS_plotAttributes($TBScurrentFile($id),$id,symbol) $TBScurrentSymbol($id)
        TBSrefresh .g$id.c $TBScurrentFile($id) $id
        if {$TBSsetStatus($id,$TBScurrentFile($id))!="active"} {
            .g$id.c lower data$TBScurrentFile($id) priming
        }
        if $TBSshowLegends($id) {
            .g$id.c delete legend
            TBSplaceLegend $TBSlegendCoords($id,x) $TBSlegendCoords($id,y)
        }
        .g$id.c raise viewframe
    } else {
        set TBScurrentSymbol($id) $TBS_plotAttributes($TBScurrentFile($id),$id,symbol)
    }
}
proc TBSsetCurrentLinewidth id {
    global  TBS_plotAttributes TBScurrentFile TBScurrentLinewidth

    if {$TBS_plotAttributes($TBScurrentFile($id),$id,style)!="marker"} {
        set TBS_plotAttributes($TBScurrentFile($id),$id,width) $TBScurrentLinewidth($id)
        .g$id.c itemconfigure data$TBScurrentFile($id) -width $TBScurrentLinewidth($id)
    } else {
        set TBScurrentLinewidth($id) $TBS_plotAttributes($TBScurrentFile($id),$id,width)
    }
}
proc TBSsetCurrentLinecolor id {
    global  TBS_plotAttributes TBScurrentFile TBScurrentLinecolor

    if {$TBS_plotAttributes($TBScurrentFile($id),$id,style)!="marker"} {
        set TBS_plotAttributes($TBScurrentFile($id),$id,color) $TBScurrentLinecolor($id)
        .g$id.c itemconfigure data$TBScurrentFile($id) -fill $TBScurrentLinecolor($id)
    } else {
        set TBScurrentLinecolor($id) $TBS_plotAttributes($TBScurrentFile($id),$id,color)
    }
}
proc TBSreadDatafile {filename index frame} {
    global TBSnumbPoints TBSdataCoordinates TBSdataMinimum TBSdataMaximum
    global TBSsetStatus

#  TBSdataM(ax|in)imum comes in two varieties.
#  partial:  Minima and maxima for individual data set;
#            index is ($dataindex,$frame,(x|y))
#            used to reset the global minima and maxima when a data set is deleted
#  global:   Minima and maxima over all data sets;
#            index is ($frame,(x|y))
#            used for scaling

    set i 0
    set file [open $filename]
    set TBSdataMinimum($index,$frame,x)  1e300
    set TBSdataMaximum($index,$frame,x) -1e300
    set TBSdataMinimum($index,$frame,y)  1e300
    set TBSdataMaximum($index,$frame,y) -1e300
    while {[gets $file line] >= 0} {
	if [regexp {^[ 	]*([\#\%].*)?$} $line match]  continue

        set TBSdataCoordinates($i,$index,$frame,x) [lindex $line 0]
        set TBSdataCoordinates($i,$index,$frame,y) [lindex $line 1]
        set TBSdataMinimum($index,$frame,x) [expr \
            $TBSdataMinimum($index,$frame,x)>$TBSdataCoordinates($i,$index,$frame,x) ? \
            $TBSdataCoordinates($i,$index,$frame,x) : $TBSdataMinimum($index,$frame,x)] 
        set TBSdataMaximum($index,$frame,x) [expr \
            $TBSdataMaximum($index,$frame,x)<$TBSdataCoordinates($i,$index,$frame,x) ? \
            $TBSdataCoordinates($i,$index,$frame,x) : $TBSdataMaximum($index,$frame,x)] 
        set TBSdataMinimum($index,$frame,y) [expr \
            $TBSdataMinimum($index,$frame,y)>$TBSdataCoordinates($i,$index,$frame,y) ? \
            $TBSdataCoordinates($i,$index,$frame,y) : $TBSdataMinimum($index,$frame,y)] 
        set TBSdataMaximum($index,$frame,y) [expr \
            $TBSdataMaximum($index,$frame,y)<$TBSdataCoordinates($i,$index,$frame,y) ? \
            $TBSdataCoordinates($i,$index,$frame,y) : $TBSdataMaximum($index,$frame,y)] 
        incr i
    }
    close $file
    set TBSdataMinimum($frame,x) [expr \
        $TBSdataMinimum($frame,x)>$TBSdataMinimum($index,$frame,x) ? \
        $TBSdataMinimum($index,$frame,x) : $TBSdataMinimum($frame,x)] 
    set TBSdataMaximum($frame,x) [expr \
        $TBSdataMaximum($frame,x)<$TBSdataMaximum($index,$frame,x) ? \
        $TBSdataMaximum($index,$frame,x) : $TBSdataMaximum($frame,x)] 
    set TBSdataMinimum($frame,y) [expr \
        $TBSdataMinimum($frame,y)>$TBSdataMinimum($index,$frame,y) ? \
        $TBSdataMinimum($index,$frame,y) : $TBSdataMinimum($frame,y)] 
    set TBSdataMaximum($frame,y) [expr \
        $TBSdataMaximum($frame,y)<$TBSdataMaximum($index,$frame,y) ? \
        $TBSdataMaximum($index,$frame,y) : $TBSdataMaximum($frame,y)] 
    set TBSnumbPoints($index,$frame) $i
    set TBSsetStatus($frame,$index) active
}
proc TBSresetExtrema frame {
    global  TBSdataMinimum TBSdataMaximum TBSnumbSets TBSsetStatus

    set TBSdataMinimum($frame,x)  1e300
    set TBSdataMaximum($frame,x) -1e300
    set TBSdataMinimum($frame,y)  1e300
    set TBSdataMaximum($frame,y) -1e300
    for {set index 0} {$index<$TBSnumbSets($frame)} {incr index} {
        if {$TBSsetStatus($frame,$index)=="deleted"} continue
        set TBSdataMinimum($frame,x) [expr \
            $TBSdataMinimum($frame,x)>$TBSdataMinimum($index,$frame,x) ? \
            $TBSdataMinimum($index,$frame,x) : $TBSdataMinimum($frame,x)] 
        set TBSdataMaximum($frame,x) [expr \
            $TBSdataMaximum($frame,x)<$TBSdataMaximum($index,$frame,x) ? \
            $TBSdataMaximum($index,$frame,x) : $TBSdataMaximum($frame,x)] 
        set TBSdataMinimum($frame,y) [expr \
            $TBSdataMinimum($frame,y)>$TBSdataMinimum($index,$frame,y) ? \
            $TBSdataMinimum($index,$frame,y) : $TBSdataMinimum($frame,y)] 
        set TBSdataMaximum($frame,y) [expr \
            $TBSdataMaximum($frame,y)<$TBSdataMaximum($index,$frame,y) ? \
            $TBSdataMaximum($index,$frame,y) : $TBSdataMaximum($frame,y)] 
    }
}
proc TBSscratchData {frame index} {
    global  TBSnumbPoints TBSdataCoordinates

    for {set i 0} {$i<$TBSnumbPoints($index,$frame)} {incr i} {
        unset TBSdataCoordinates($i,$index,$frame,x)
        unset TBSdataCoordinates($i,$index,$frame,y)
    }
}
proc TBScheckDatafile filename {
    global  FloatingpointNumber

    set pattern "^\[ 	\]*($FloatingpointNumber\[ 	\]+)+$"
    set file [open $filename]
    while {[gets $file line] >= 0} {
	if [regexp {^[ 	]*([\#\%].*)?$} $line match]  continue
        set line "$line "
	if {![regexp $pattern $line]} {
            BSerror TBScheckDatafile "Non-floatingpoint number in uncommented line"
            return 0
        }
    }
    close $file
    return 1
}