
proc mapHelpWindow {dir top} {
    global  helpDirectory topLevel Links LinkIndex env
	
    if [winfo exists .help] { raise .help } else {
        toplevel .help
        wm title .help "FUI: help"
	frame .help.fb
        button .help.close -text "C L O S E" -font -*-helvetica-bold-r-normal--*-120-* \
            -command { destroy .help }
        button .help.top -text "T O P" -font -*-helvetica-bold-r-normal--*-120-* \
            -command { 
                global  helpDirectory topLevel
                displayDocumentation "$helpDirectory/$topLevel" 1
            }
        button .help.back -text "B A C K" -font -*-helvetica-bold-r-normal--*-120-* \
            -command { 
                global  Links LinkIndex
                incr LinkIndex -1
                if {$LinkIndex<0} {  set LinkIndex [expr [llength $Links] - 1]  }
                displayDocumentation [lindex $Links $LinkIndex] 0
            }
        button .help.forward -text "F O R W A R D" -font -*-helvetica-bold-r-normal--*-120-* \
            -command { 
                global  Links LinkIndex
                incr LinkIndex
                if {$LinkIndex>=[llength $Links]} {  set LinkIndex 0  }
                displayDocumentation [lindex $Links $LinkIndex] 0
            }
        if {[info exists env(HOSTNAME)] && $env(HOSTNAME)=="loplop"} {
            pack .help.top .help.back .help.forward .help.close -side left \
                -ipadx 6m -padx 3m -pady 2m -in .help.fb
        } else {
            pack .help.top .help.back .help.forward .help.close -side left \
                -ipadx 3m -padx 2m -pady 2m -in .help.fb
        }
	pack .help.fb -anchor w
        frame .help.f
        scrollbar .help.scroll -command ".help.text yview"
        if {[info exists env(HOSTNAME)] && $env(HOSTNAME)=="loplop"} {
            text .help.text -width 80 -height 32 -relief sunken -bd 2 \
                -yscrollcommand ".help.scroll set"
        } else {
            text .help.text -width 95 -height 40 -relief sunken -bd 2 \
                -yscrollcommand ".help.scroll set"
        }
        pack .help.scroll -side right -fill y -in .help.f
        pack .help.text -side left -in .help.f
        pack .help.f

        bind .help.text <Any-Key> { }
        bind .help.text <Any-Button> { }

        set helpDirectory $dir
        set topLevel $top
	set Links [list]
	displayDocumentation "$helpDirectory/$topLevel" 1
    }
}

proc displayDocumentation {filename newLink} {
    global  helpDirectory Links LinkIndex helpFile helpRline

    set ihtlnull {\\IHTL\{[^\}]+\}\{[^\}]+\}\{[^\}]+\}}
    set ihtlfull {^([^\]*)\\IHTL\{([^\}]+)\}\{([^\}]+)\}\{([^\}]+)\}(.*)$}
    set xhtlnull {\\XHTL\{[^\}]+\}\{[^\}]+\}\{[^\}]+\}}
    set xhtlfull {^([^\]*)\\XHTL\{([^\}]+)\}\{([^\}]+)\}\{([^\}]+)\}(.*)$}
    set htrefnull {\\HTREF\{[^\}]+\}\{[^\}]+\}\{[^\}]+\}\{[^\}]+\}}
    set htreffull {^([^\]*)\\HTREF\{([^\}]+)\}\{([^\}]+)\}\{([^\}]+)\}\{([^\}]+)\}(.*)$}

    set Links [linsert $Links end $filename]
    if {$newLink} {
        set LinkIndex [expr [llength $Links] - 1]
    }
    set count 1
    set lc 1
    set rc 1
    set xc 1
    set leftmargin "      "
    set text_tag description
    .help.text delete 0.0 end
    set hf [open $filename]
    .help.text insert end "\n"
    incr count
    while {[gets $hf line]>=0} {
        if [regexp {^[     ]*$} $line]  continue
        if [regexp {\\HTL\{([^\}]+)\}\{([^\}]+)\}\{([^\}]+)\}} $line match text source type] {
            set link($lc) "link$lc"
            .help.text insert end "    "
            regexp {[0-9]+.([0-9]+)} [.help.text index insert] match lo
            .help.text insert end "$text\n"
            regexp {[0-9]+.([0-9]+)} [.help.text index {insert -1 c }] match hi
            for {set i $lo} {$i<=$hi} {incr i} {
                set index "$count.$i"
                set helpFile($index) "$helpDirectory/$source"
            }
            .help.text tag add link $count.$lo $count.$hi
            .help.text tag add $type $count.$lo $count.$hi
            .help.text tag add $link($lc) $count.$lo $count.$hi
            incr lc
            incr count
        } elseif [regexp {\\parent\{([^\}]+)\}\{([^\}]+)\}\{([^\}]+)\}} $line match text type source] {
            set link($lc) "link$lc"
            .help.text insert end "    Parent: "
            regexp {[0-9]+.([0-9]+)} [.help.text index insert] match lo
            .help.text insert end "$text "
            regexp {[0-9]+.([0-9]+)} [.help.text index {insert -1 c }] match hi
            for {set i $lo} {$i<=$hi} {incr i} {
                set index "$count.$i"
                set helpFile($index) "$helpDirectory/$source"
            }
            .help.text tag add link $count.$lo $count.$hi
            .help.text tag add $link($lc) $count.$lo $count.$hi
            .help.text insert end "  ($type)\n"
            .help.text tag add subheader $count.0 $count.end
            incr lc
            incr count
        } elseif [regexp {\\par} $line] {
            .help.text insert end "\n"
            incr count
        } elseif [regexp {\\headline\{([^\}]+)} $line match h] {
            .help.text insert end "    $h\n"
            .help.text tag add head $count.0 $count.end
            incr count
        } elseif [regexp {\\subheader\{([^\}]+)} $line match h] {
            .help.text insert end "    $h\n"
            .help.text tag add subheader $count.0 $count.end
            incr count
        } elseif [regexp {\\begin\{notice\}} $line] {
            set text_tag notice
            .help.text insert end "      Notice:\n"
            .help.text tag add note $count.0 $count.end
            incr count
        } elseif [regexp {\\end\{notice\}} $line] {
            set text_tag description
        } elseif [regexp {\\begin\{verbatim\}} $line] {
            set text_tag verbatim
            set leftmargin "  "
        } elseif [regexp {\\end\{verbatim\}} $line] {
            set leftmargin "      "
            set text_tag description
        } elseif [regexp {\\begin\{remarks\}} $line] {
            set text_tag notice
            .help.text insert end "      Remarks:\n"
            .help.text tag add note $count.0 $count.end
            incr count
        } elseif [regexp {\\end\{remarks\}} $line] {
            set text_tag description
        } else {
            set lline $line
            if {![regsub {\\ldots} $lline ... mline]} { set mline $lline }
            while {[regexp {\\name\{([^\{\}]+)\}} $mline match words]} {
                regsub {\\name\{[^\{\}]+\}} $mline "\"$words\"" line
                set mline $line
            }
            while {[regexp {\\command\{([^\}]+)\}} $mline match words]} {
                regsub {\\command\{[^\}]+\}} $mline "`$words'" line
                set mline $line
            } 
            if [regexp $ihtlnull $mline] {
                set lo 0
                set mline $line
                .help.text insert end "      "
                while {[regexp $ihtlfull \
                        $mline match prev text source type aft]} {
                    .help.text insert end $prev
                    .help.text tag add description $count.$lo $count.end
                    set mline $line
                    set link($lc) "link$lc"
                    regexp {[0-9]+.([0-9]+)} [.help.text index insert] match lo
                    .help.text insert end "$text "
                    regexp {[0-9]+.([0-9]+)} [.help.text index {insert -1 c}] match hi
                    for {set i $lo} {$i<=$hi} {incr i} {
                        set index "$count.$i"
                        set helpFile($index) "$helpDirectory/$source"
                    }
                    .help.text tag add link $count.$lo $count.$hi
                    .help.text tag add $type $count.$lo $count.$hi
                    .help.text tag add $link($lc) $count.$lo $count.$hi
                    incr lc
                    set mline $aft
                    set lo $hi
                }
                .help.text insert end "$aft\n"
                .help.text tag add description $count.$lo $count.end
                .help.text tag add $text_tag $count.0 $count.end
                incr count
            } elseif [regexp $xhtlnull $mline] {
                set lo 0
                set mline $line
                .help.text insert end "      "
                while {[regexp $xhtlfull \
                        $mline match prev text source type aft]} {
                    .help.text insert end $prev
                    .help.text tag add description $count.$lo $count.end
                    set mline $line
                    set xlink($xc) "xlink$xc"
                    regexp {[0-9]+.([0-9]+)} [.help.text index insert] match lo
                    .help.text insert end "$text "
                    regexp {[0-9]+.([0-9]+)} [.help.text index {insert -1 c}] match hi
                    for {set i $lo} {$i<=$hi} {incr i} {
                        set index "$count.$i"
                        set helpFile($index) "$helpDirectory/$source"
                    }
                    .help.text tag add link $count.$lo $count.$hi
                    .help.text tag add $type $count.$lo $count.$hi
                    .help.text tag add $xlink($xc) $count.$lo $count.$hi
                    incr xc
                    set mline $aft
                    set lo $hi
                }
                .help.text insert end "$aft\n"
                .help.text tag add description $count.$lo $count.end
                .help.text tag add $text_tag $count.0 $count.end
                incr count
            } elseif [regexp $htrefnull $mline] {
                set lo 0
                set mline $line
                .help.text insert end "      "
                while {[regexp $htreffull \
                        $mline match prev text source line type aft]} {
                    .help.text insert end $prev
                    .help.text tag add description $count.$lo $count.end
                    set mline $line
                    set rlink($rc) "rlink$rc"
                    regexp {[0-9]+.([0-9]+)} [.help.text index insert] match lo
                    .help.text insert end "$text "
                    regexp {[0-9]+.([0-9]+)} [.help.text index {insert -1 c}] match hi
                    for {set i $lo} {$i<=$hi} {incr i} {
                        set index "$count.$i"
                        set helpFile($index) "$helpDirectory/$source"
                        set helpRline($index) $line
                    }
                    .help.text tag add link $count.$lo $count.$hi
                    .help.text tag add $type $count.$lo $count.$hi
                    .help.text tag add $rlink($rc) $count.$lo $count.$hi
                    incr rc
                    set mline $aft
                    set lo $hi
                }
                .help.text insert end "$aft\n"
                .help.text tag add description $count.$lo $count.end
                .help.text tag add $text_tag $count.0 $count.end
                incr count
            } else {
                .help.text insert end "$leftmargin$mline\n"
                .help.text tag add $text_tag $count.0 $count.end
                incr count
            }
        }
    }
    close $hf
    .help.text tag configure main -font -*-times-bold-r-normal--*-240-*
    .help.text tag configure sub1 -font -*-times-bold-r-normal--*-180-*
    .help.text tag configure sub2 -font -*-times-bold-r-normal--*-140-*
    .help.text tag configure head -font -*-times-bold-i-normal--*-180-*
    .help.text tag configure subheader -font -*-times-bold-i-normal--*-140-*
    .help.text tag configure text -font -*-times-medium-r-normal--*-140-*
    .help.text tag configure description -font -*-times-medium-r-normal--*-140-*
    .help.text tag configure notice -font -*-times-medium-i-normal--*-140-*
    .help.text tag configure note -font -*-times-bold-i-normal--*-140-*
    .help.text tag configure verbatim -font -*-courier-medium-r-normal--*-120-*
    .help.text tag configure link -foreground dodgerblue
    for { set i 1 } { $i<$lc } { incr i } {
        .help.text tag bind $link($i) <Button-3> {
            global     helpFile
            set index [.help.text index {@%x,%y}]
            displayDocumentation $helpFile($index) 1
        }
    }
    for { set i 1 } { $i<$xc } { incr i } {
        .help.text tag bind $xlink($i) <Button-3> {
            global     helpFile
            set index [.help.text index {@%x,%y}]
            exec $helpFile($index)
        }
    }
    for { set i 1 } { $i<$rc } { incr i } {
        .help.text tag bind $rlink($i) <Button-3> {
            global     helpFile helpRline
            set index [.help.text index {@%x,%y}]
            displayDocumentation $helpFile($index) 1
            .help.text yview -pickplace $helpRline($index).0
            .help.text tag add sel $helpRline($index).0 $helpRline($index).end
        }
    }
}