#include  "mbs.h"

/*********************************************************************
**
**   MACROS:        
**
**   ROUTINES:      mbs_4D_alloc_array
**                  mbs_3D_alloc_array
**                  mbs_2D_alloc_array
**                  mbs_4D_free_array
**                  mbs_3D_free_array
**                  mbs_2D_free_array
**                 
**********************************************************************/

/*********************************************************************
**
**   Routine Name:  mbs_4D_alloc_array
**                  
**   Purpose:       
**
**   Input:         
**   Output:        
**
**   Called By:     
**
**   Written By:    Bernhard Sulzer, LANL
**   Last Edited:   
**
**********************************************************************/

void ****
mbs_4D_alloc_array(int dim1, int dim2, int dim3, int dim4, int ob_type)
{
        int            i;
	void           ****ptr_4, ***ptr_3, **ptr_2, *ptr_1;

	if (!(ptr_1 = (void *)calloc(dim1*dim2*dim3*dim4, ob_type)))
	    mbs_alloc_error ("MBS ALLOC ARRAY (4D)", "level 0 data");
	if (!(ptr_2 = (void **)calloc(dim1*dim2*dim3, sizeof(void *))))
	    mbs_alloc_error ("MBS ALLOC ARRAY (4D)", "level 1 pointers");
	if (!(ptr_3 = (void ***)calloc(dim1*dim2, sizeof(void *))))
	    mbs_alloc_error ("MBS ALLOC ARRAY (4D)", "level 2 pointers");
	if (!(ptr_4 = (void ****)calloc(dim1, sizeof(void *))))
	    mbs_alloc_error ("MBS ALLOC ARRAY (4D)", "level 3 pointers");

	for (i=0 ; i<dim1*dim2*dim3 ; i++)
	{
	    ptr_2[i] = ptr_1;
	    ptr_1 += (dim4 * ob_type);
	}
	for (i=0 ; i<dim1*dim2 ; i++)
	{
	    ptr_3[i] = ptr_2;
	    ptr_2 += dim3;
	}
	for (i=0 ; i<dim1 ; i++)
	{
	    ptr_4[i] = ptr_3;
	    ptr_3 += dim2;
	}
	return (ptr_4);
}


/*********************************************************************
**
**   Routine Name:  mbs_3D_alloc_array
**                  
**   Purpose:       
**
**   Input:         
**   Output:        
**
**   Called By:     
**
**   Written By:    Bernhard Sulzer, LANL
**   Last Edited:   
**
**********************************************************************/

void ***
mbs_3D_alloc_array(int dim1, int dim2, int dim3, int ob_type)
{
        int            i;
	void           ***ptr_3, **ptr_2, *ptr_1;
	
	if (!(ptr_1 = (void *)calloc(dim1*dim2*dim3, ob_type)))
	    mbs_alloc_error ("MBS ALLOC ARRAY (3D)", "level 0 data");
	if (!(ptr_2 = (void **)calloc(dim1*dim2, sizeof(void *))))
	    mbs_alloc_error ("MBS ALLOC ARRAY (3D)", "level 1 pointers");
	if (!(ptr_3 = (void ***)calloc(dim1, sizeof(void *))))
	    mbs_alloc_error ("MBS ALLOC ARRAY (3D)", "level 2 pointers");

	for (i=0 ; i<dim1*dim2 ; i++)
	{
	    ptr_2[i] = ptr_1;
	    ptr_1 += (dim3 * ob_type);
	}
	for (i=0 ; i<dim1 ; i++)
	{
	    ptr_3[i] = ptr_2;
	    ptr_2 += dim2;
	}
	return (ptr_3);
}


/*********************************************************************
**
**   Routine Name:  mbs_2D_alloc_array
**                  
**   Purpose:       
**
**   Input:         
**   Output:        
**
**   Called By:     
**
**   Written By:    Bernhard Sulzer, LANL
**   Last Edited:   
**
**********************************************************************/

void **
mbs_2D_alloc_array(int dim1, int dim2, int ob_type)
{
        int            i;
	void           **ptr_2, *ptr_1;
	
	if (!(ptr_1 = (void *)calloc(dim1*dim2, ob_type)))
	    mbs_alloc_error ("MBS ALLOC ARRAY (2D)", "level 0 data");
	if (!(ptr_2 = (void **)calloc(dim1, sizeof(void *))))
	    mbs_alloc_error ("MBS ALLOC ARRAY (2D)", "level 1 pointers");

	for (i=0 ; i<dim1 ; i++)
	{
	    ptr_2[i] = ptr_1;
	    ptr_1 += (dim2 * ob_type);
	}
	return (ptr_2);
}


/*********************************************************************
**
**   Routine Name:  mbs_4D_free_array
**                  
**   Purpose:       
**
**   Input:         
**   Output:        
**
**   Called By:     
**
**   Written By:    Bernhard Sulzer, LANL
**   Last Edited:   
**
**********************************************************************/

void    
mbs_4D_free_array(void ****ptr)
{
        free(***ptr);
	free(**ptr);
	free(*ptr);
	free(ptr);
}


/*********************************************************************
**
**   Routine Name:  mbs_3D_free_array
**                  
**   Purpose:       
**
**   Input:         
**   Output:        
**
**   Called By:     
**
**   Written By:    Bernhard Sulzer, LANL
**   Last Edited:   
**
**********************************************************************/

void    
mbs_3D_free_array(void ***ptr)
{
	free(**ptr);
	free(*ptr);
	free(ptr);
}


/*********************************************************************
**
**   Routine Name:  mbs_2D_free_array
**                  
**   Purpose:       
**
**   Input:         
**   Output:        
**
**   Called By:     
**
**   Written By:    Bernhard Sulzer, LANL
**   Last Edited:   
**
**********************************************************************/

void    
mbs_2D_free_array(void **ptr)
{
	free(*ptr);
	free(ptr);
}
