/*********************************************************************
**
**   MACROS:        
**
**   ROUTINES:      mbs_alloc_error
**                  mbs_file_open_error
**                  mbs_out_of_range_error
**                  mbs_out_of_range_warning
**                  mbs_dummy_catch
**                 
**********************************************************************/


#include "mbs.h"


void
mbs_alloc_error (char *routine_name, char *occurrence)
{
    fprintf (stderr, "%s:  Unable to allocate %s.\n", routine_name, occurrence);
    fprintf (stderr, "Exiting now\n\n");
    exit (-1);
}


/*********************************************************************
**
**   Routine Name:  mbs_file_open_error
**                  
**   Purpose:       Print error message and exit
**                  (called when file could not be opened)
**
**   Input:         routine_name  name of routine trying to open file
**                  file_name     name of file that could not be opened
**   Output:        none
**
**   Called By:     anybody
**
**   Written By:    Bernhard Sulzer, LANL
**   Last Edited:   2/10/95
**
**********************************************************************/

void 
mbs_file_open_error(char *routine_name, char *file_name)
{
    fprintf (stderr, "%s: Could not open file %s\n", routine_name, file_name);
    fprintf (stderr, "Exiting now.\n\n");
    exit (-2);
}


void
mbs_out_of_range_error (char *routine_name, char *parameter_name,
			char *lb, char *ub, char *input)
{
    fprintf (stderr, "%s -- ERROR\n", routine_name);
    fprintf (stderr, "Parameter %s:\n", parameter_name);
    fprintf (stderr, "Value %s out of range [%s , %s].\n", input, lb, ub);
    fprintf (stderr, "Exiting now.\n\n");
    exit (-4);
}


void
mbs_out_of_range_warning (char *routine_name, char *parameter_name,
			  char *lb, char *ub, char *input)
{
    fprintf (stderr, "%s -- WARNING\n", routine_name);
    fprintf (stderr, "Parameter %s:\n", parameter_name);
    fprintf (stderr, "Value %s out of range [%s , %s].\n", input, lb, ub);
    fprintf (stderr, "Continuing...\n");
}


void
mbs_dummy_catch (char *routine_name)
{
    fprintf (stderr, "%s:  You have no business being here.\n", routine_name);
    fprintf (stderr, "Exiting now.\n\n");
    exit (-3);
}


/*
void 
_lib_version()
{
    mbs_dummy_catch ("MBS: _LIB_VERSION_");
}
*/
