#include  "mbs.h"

#define  min(a,b)                 ((a)<(b) ? (a) : (b))

#define         POWEXP
#if    defined  POWEXP
#define _ML_power(base,exponent)  (exp((double)(exponent)*log((base))))
#elif  defined  POWOW
#define _ML_power(base,exponent)  (pow((base),(double)(exponent)))
#endif


void
_mbs_global_receptor_conservation (int *n, double *r, double *f, int *iflag)
{
    int     l;
    
    for (l=1 ; l<=*n ; l++)
	f[l-1] = (*__ML_receptor_conservation)(r, &l);
				/* watch out! indices must start with 1 */
				/* reason: compatibility with DSOS */
}

double
_mbs_receptor_conservation_vk (double *r, int *index)
{
    int        i, k, l, m, n;
    double     rb, rl, rt, phi, sum;
    void      _mbs_compute_sigma_vk(int, double*);

    l = *index - 1;		/* watch out! indices start with 1 */
				/* reason: compatibility with DSOS */
    rl = r[l];
    rt = mbs_get_cell_number_of_receptors(l);
    rb = 0.0;
    for (k=0 ; k<_ML_number_of_ligands ; k++)
    {
	_mbs_compute_sigma_vk(k, r);
	n = mbs_get_ligand_valence(k);
	sum = phi = n * _ML_Ikappa_i[k][l][0] * rl;
	for (i=1 ; i<n ; i++)
	{
	    phi *= ((double)(n-i)/(double)i*_ML_Ikappa_i[k][l][i]*rl);
	    sum += phi;
	}
	rb += (mbs_get_ligand_concentration(k)*sum/(1.0+_ML_Isigma[k]));
    }
/*
    for (m=0 ; m<mbs_get_number_of_cell_types ; m++)
	fprintf(stderr, "%e ", r[m]);
    fprintf (stderr, "\n");
    fprintf (stderr, "%d: %e, %e, %e\n", l, r[l], rb/rt, r[l]+rb/rt-1.0);
*/
    return (rl+rb/mbs_get_cell_number_of_receptors(l)-1.0);
}
double
_mbs_receptor_conservation(double *r, int *index)
{
    int        k, l, m, n_x;
    double     rb;
    void       _mbs_compute_rho(double*);
    void       _mbs_compute_auxiliary_parameters(int, int);

    _mbs_compute_rho(r);
    l = *index - 1;		/* watch out! indices start with 1 */
				/* reason: compatibility with DSOS */
    rb = 0.0;
    for (k=0 ; k<_ML_number_of_ligands ; k++)
    {
	_mbs_compute_auxiliary_parameters(k, l);
	rb += (_ML_Ixi[k][l] * 
	       _ML_power(_ML_Irho[k][l], (mbs_get_ligand_effective_valence(k)-1.0)));
    }
/*
    for (m=0 ; m<mbs_get_number_of_cell_types ; m++)
	fprintf(stderr, "%e ", r[m]);
    fprintf (stderr, "\n");
    fprintf (stderr, "%d: %e, %e, %e\n", l, r[l], r[l]*rb, r[l]*(1.0+rb)-1.0);
*/
    return (r[l]*(1.0+rb)-1.0);
}
double
_mbs_compute_signaling_crosslinks(int k, int l)
{
    int     nx, i;
    double  cc=0.0, rt;
    
    nx = mbs_get_ligand_effective_valence(k);
    rt = mbs_get_cell_number_of_receptors(l);
    for (i=(mbs_get_cell_signal_size(l)-1) ; i<nx ; i++)
	cc += (mbs_get_crosslinks(k, l, i)/rt);
    return (cc);
}
double
_mbs_compute_signaling_aggregates(int k, int l)
{
    int     nx, i;
    double  cc=0.0, rt;
    
    nx = mbs_get_ligand_effective_valence(k);
    rt = mbs_get_cell_number_of_receptors(l);
    for (i=(mbs_get_cell_signal_size(l)-1) ; i<nx ; i++)
	cc += (mbs_get_bound_state(k, l, i)/rt);
    return (cc);
}
void
_mbs_compute_rho (double *r)
{
    int       k, l;

    for (k=0 ; k<_ML_number_of_ligands ; k++)
	for (l=0 ; l<_ML_number_of_cell_types ; l++)
	    _ML_Irho[k][l] = _ML_Ikappa_x[k][l]*r[l] + 1.0;
}
void
_mbs_compute_sigma_vk (int k, double *r)
{
    int      i, j, l, n;
    double   psi, sum, sigma, rl;
    
    n = mbs_get_ligand_valence(k);
    sigma = 0.0;
    for (l=0 ; l<mbs_get_number_of_cell_types ; l++)
    {
	rl = r[l];
	sum = psi = n * _ML_Ikappa_i[k][l][0] * rl;
	for (j=2,i=1 ; i<n ; i++,j++)
	{
	    psi *= ((double)(n-i)/(double)j*_ML_Ikappa_i[k][l][i]*rl);
	    sum += psi;
	}
	sigma += (_ML_C[l]->alpha * _ML_C[l]->concentration * sum);
    }
    _ML_Isigma[k] = sigma;
}
void
_mbs_compute_auxiliary_parameters (int k, int l)
{
    ML_Ligand_Attributes  *ligand;
    int                    m, n_x;
    
    ligand = _ML_L[k];
    n_x = ligand->n_x;
    _ML_Isigma[k] = 0.0;
    for (m=0 ; m<_ML_number_of_cell_types ; m++)
	_ML_Isigma[k] += (_ML_Imu[k][m]*(_ML_power(_ML_Irho[k][m],n_x)-1.0));
    _ML_Ixi[k][l] = ligand->n*_ML_Iaf[k][l]*ligand->concentration/(1.0+_ML_Isigma[k]);
}    


double
_mbs_multiply_ligand_concentration (double c)
{
    return (c * _ML_CTRL.ligand_stepsize);
}
double
_mbs_addto_ligand_concentration (double c)
{
    return (c + _ML_CTRL.ligand_stepsize);
}
int
_mbs_select_ligand_scale ()
{
    double   _mbs_multiply_ligand_concentration(double);
    double   _mbs_addto_ligand_concentration(double);
    
    if (_ML_CTRL.ligand_bottom >= _ML_CTRL.ligand_top)
    {
	fprintf(stderr, "MBS: Get Concentration Boundaries -- Error.\n");
	fprintf(stderr, "\tLigand concentration interval:\n");
	fprintf(stderr, "\t Lower boundary (%e) . greater or equal . Upper boundary (%e).\n",
		_ML_CTRL.ligand_bottom, _ML_CTRL.ligand_top);
	return (0);
    }
    switch (_ML_CTRL.ligand_scale_type)
    {
      case _ML_LS_LOGARITHMIC:
	_ML_CTRL.ligand_stepsize = exp(log(_ML_CTRL.ligand_top/_ML_CTRL.ligand_bottom)/
					(double)(_ML_CTRL.ligand_number_of_steps-1));
	__ML_ligand_increment_function = _mbs_multiply_ligand_concentration;
	break;
      case _ML_LS_LINEAR:
	_ML_CTRL.ligand_stepsize = (_ML_CTRL.ligand_top - _ML_CTRL.ligand_bottom)/
					(double)(_ML_CTRL.ligand_number_of_steps-1);
	__ML_ligand_increment_function = _mbs_addto_ligand_concentration;
	break;
      default:
	break;
    }
    return (1);
}


void
_mbs_update_control_variables (int nl, int nc)
{
    double   *d_aux;
    int      *i_aux, n, k, l;

    if (nl != _ML_number_of_ligands)
    {
	n = min(nl, _ML_number_of_ligands);

	if (!(d_aux = (double *)calloc(nl, sizeof(double))))
	    mbs_alloc_error ("Update Control Variables", "Fraction in Mixture");
	for (k=0 ; k<n ; k++)  d_aux[k] = _ML_CTRL_lcf[k];
	if (_ML_CTRL_lcf)  free (_ML_CTRL_lcf);
	_ML_CTRL_lcf = d_aux;

	if (!(i_aux = (int *)calloc(nl, sizeof(int))))
	    mbs_alloc_error ("Update Control Variables", "Vary Ligand Variable");
	for (k=0 ; k<n ; k++)  i_aux[k] = _ML_CTRL_lcv[k];
	if (_ML_CTRL_lcv)  free (_ML_CTRL_lcv);
	_ML_CTRL_lcv = i_aux;

	if (!(i_aux = (int *)calloc(nl, sizeof(int))))
	    mbs_alloc_error ("Update Control Variables", "Ligand Contributes to Signal");
	for (k=0 ; k<n ; k++)  i_aux[k] = _ML_CTRL_lcontrib[k];
	if (_ML_CTRL_lcontrib)  free (_ML_CTRL_lcontrib);
	_ML_CTRL_lcontrib = i_aux;
    }
    if (nc != _ML_number_of_cell_types)
    {
	n = min(nc, _ML_number_of_cell_types);
	
	if (!(d_aux = (double *)calloc(nc, sizeof(double))))
	    mbs_alloc_error ("Update Control Variables", "Cell Fraction in Mixture");
	for (k=0 ; k<n ; k++)  d_aux[k] = _ML_CTRL_ccf[k];
	if (_ML_CTRL_ccf)  free (_ML_CTRL_ccf);
	_ML_CTRL_ccf = d_aux;

	if (!(i_aux = (int *)calloc(nc, sizeof(int))))
	    mbs_alloc_error ("Update Control Variables", "Vary Cell Variable");
	for (k=0 ; k<n ; k++)  i_aux[k] = _ML_CTRL_ccv[k];
	if (_ML_CTRL_ccv)  free (_ML_CTRL_ccv);
	_ML_CTRL_ccv = i_aux;

	if (!(i_aux = (int *)calloc(nc, sizeof(int))))
	    mbs_alloc_error ("Update Control Variables", "Cell Contributes to Signal");
	for (k=0 ; k<n ; k++)  i_aux[k] = _ML_CTRL_ccontrib[k];
	if (_ML_CTRL_ccontrib)  free (_ML_CTRL_ccontrib);
	_ML_CTRL_ccontrib = i_aux;
    }
}
ML_Ligand_Attributes**
_mbs_copy_ligand_attributes (int new, int old, ML_Ligand_Attributes **source)
{
    ML_Ligand_Attributes **aux;
    int                    i;
    
    _ML_ligand_attributes_exist = true;
    if (old<new)
    {
	if (!(aux = (ML_Ligand_Attributes **)calloc(new, sizeof(ML_Ligand_Attributes*))))
	    mbs_alloc_error ("Copy Ligand Attributes", "Ligand Attribute Pointers");
	for (i=0 ; i<old ; i++) 
	{
	    if (!(aux[i] = (ML_Ligand_Attributes *)calloc(1, sizeof(ML_Ligand_Attributes))))
		mbs_alloc_error ("Copy Ligand Attributes", "Ligand Attribute");
	    aux[i]->weight        = source[i]->weight;
	    aux[i]->w_micro       = source[i]->w_micro;
	    aux[i]->w_nano        = source[i]->w_nano;
	    aux[i]->n             = source[i]->n;
	    aux[i]->n_x           = source[i]->n_x;
	    aux[i]->concentration = source[i]->concentration;
	    if (source[i])  free (source[i]);
	}
	if (source)  free (source);
	for ( ; i<new ; i++)
	    if (!(aux[i] = (ML_Ligand_Attributes *)calloc(1, sizeof(ML_Ligand_Attributes))))
		mbs_alloc_error ("Copy Ligand Attributes", "Ligand Attribute");
	return (aux);
    }
    else
    {
	i = new;
	while (i<old)   free (source[i++]);
	return (source);
    }
}
void
_mbs_update_ligand_attributes (int n)
{
    ML_Ligand_Attributes **_mbs_copy_ligand_attributes(int, int, ML_Ligand_Attributes**);
    
    _ML_L = _mbs_copy_ligand_attributes (n, _ML_number_of_ligands, _ML_L);
}
ML_Cell_Attributes**
_mbs_copy_cell_attributes (int new, int old, ML_Cell_Attributes **source)
{
    ML_Cell_Attributes **aux;
    int                    i;
    
    _ML_cell_attributes_exist = true;
    if (old<new)
    {
	if (!(aux = (ML_Cell_Attributes **)calloc(new, sizeof(ML_Cell_Attributes*))))
	    mbs_alloc_error ("Copy Cell Attributes", "Cell Attribute Pointers");
	for (i=0 ; i<old ; i++) 
	{
	    if (!(aux[i] = (ML_Cell_Attributes *)calloc(1, sizeof(ML_Cell_Attributes))))
		mbs_alloc_error ("Copy Cell Attributes", "Cell Attribute");
	    aux[i]->alpha         = source[i]->alpha;
	    aux[i]->signal_size   = source[i]->signal_size;
	    aux[i]->concentration = source[i]->concentration;
	    aux[i]->numb_receptor = source[i]->numb_receptor;
	    if (source[i])  free (source[i]);
	}
	if (source)  free (source);
	for ( ; i<new ; i++)
	    if (!(aux[i] = (ML_Cell_Attributes *)calloc(1, sizeof(ML_Cell_Attributes))))
		mbs_alloc_error ("Copy Cell Attributes", "Cell Attribute");
	return (aux);
    }
    else
    {
	i = new;
	while (i<old)   free (source[i++]);
	return (source);
    }
}
void
_mbs_update_cell_attributes (int n)
{
    ML_Cell_Attributes **_mbs_copy_cell_attributes(int, int, ML_Cell_Attributes**);
    
    _ML_C = _mbs_copy_cell_attributes (n, _ML_number_of_cell_types, _ML_C);
}
double ***
_mbs_copy_X (int nx_new, int nx_old, int ny_new, int ny_old, double ***source)
{
    double  ***aux;
    int        k, l, x=0, y=0, x_top, y_top;
    
    x_top = min(nx_new,nx_old);
    y_top = min(ny_new,ny_old);
    if (!(aux = (double***)mbs_2D_alloc_array(nx_new, ny_new, sizeof(double*))))
	mbs_alloc_error ("Copy Bound States", "Bound State Array");
    for (x=0 ; x<x_top ; x++)
	for (y=0 ; y<y_top ; y++)
	    aux[x][y] = source[x][y];
    for (k=x ; k<nx_old ; k++)
	for (l=0 ; l<ny_old ; l++)  free(source[k][l]);
    for (l=y ; l<ny_old ; l++)
	for (k=0 ; k<x ; k++)       free(source[k][l]);
    for (k=x ; k<nx_new ; k++)
	for (l=0 ; l<ny_new ; l++)
	    if (!(aux[k][l] = (double *)calloc(_ML_L[k]->n_x, sizeof(double))))
		mbs_alloc_error ("Copy Bound States", "Bound State");
    for (l=y ; l<ny_new ; l++)
	for (k=0 ; k<x ; k++)
	    if (!(aux[k][l] = (double *)calloc(_ML_L[k]->n_x, sizeof(double))))
		mbs_alloc_error ("Copy Bound States", "Bound State");
    if (source) mbs_2D_free_array ((void**)source);
    return (aux);
}
double **
_mbs_copy_ip (int nx_new, int nx_old, int ny_new, int ny_old, double **source)
{
    double  **aux;
    int       x, y, x_top, y_top;
    
    x_top = min(nx_new,nx_old);
    y_top = min(ny_new,ny_old);
    if (!(aux = (double**)mbs_2D_alloc_array(nx_new, ny_new, sizeof(double))))
	mbs_alloc_error ("Copy Interaction Parameter", "Parameter Array");
    for (x=0 ; x<x_top ; x++)
	for (y=0 ; y<y_top ; y++)
	    aux[x][y] = source[x][y];
    if (source)  mbs_2D_free_array ((void**)source);
    return (aux);
}
void
_mbs_update_interaction_parameters (int nl, int nc)
{
    double  **_mbs_copy_ip(int, int, int, int, double**);
    double   *aux;
    int       i, top;

    if (nl==_ML_number_of_ligands && nc==_ML_number_of_cell_types)  return;
    
    _ML_Iaf = _mbs_copy_ip (nl, _ML_number_of_ligands, nc, _ML_number_of_cell_types, _ML_Iaf);
    _ML_Icc = _mbs_copy_ip (nl, _ML_number_of_ligands, nc, _ML_number_of_cell_types, _ML_Icc);
    _ML_Ifb = _mbs_copy_ip (nl, _ML_number_of_ligands, nc, _ML_number_of_cell_types, _ML_Ifb);
    _ML_Irb = _mbs_copy_ip (nl, _ML_number_of_ligands, nc, _ML_number_of_cell_types, _ML_Irb);
    _ML_Ifc = _mbs_copy_ip (nl, _ML_number_of_ligands, nc, _ML_number_of_cell_types, _ML_Ifc);
    _ML_Irc = _mbs_copy_ip (nl, _ML_number_of_ligands, nc, _ML_number_of_cell_types, _ML_Irc);

    _ML_Ikappa   = _mbs_copy_ip (nl, _ML_number_of_ligands, nc, _ML_number_of_cell_types, _ML_Ikappa);
    _ML_Ikappa_x = _mbs_copy_ip (nl, _ML_number_of_ligands, nc, _ML_number_of_cell_types, _ML_Ikappa_x);
    _ML_Inu      = _mbs_copy_ip (nl, _ML_number_of_ligands, nc, _ML_number_of_cell_types, _ML_Inu);
    _ML_Inu_x    = _mbs_copy_ip (nl, _ML_number_of_ligands, nc, _ML_number_of_cell_types, _ML_Inu_x);
    _ML_Imu      = _mbs_copy_ip (nl, _ML_number_of_ligands, nc, _ML_number_of_cell_types, _ML_Imu);
    _ML_Irho     = _mbs_copy_ip (nl, _ML_number_of_ligands, nc, _ML_number_of_cell_types, _ML_Irho);
    _ML_Ixi      = _mbs_copy_ip (nl, _ML_number_of_ligands, nc, _ML_number_of_cell_types, _ML_Ixi);
    
    if (nl != _ML_number_of_ligands) 
    {
	top = min(nl,_ML_number_of_ligands);
	if (!(aux = (double *)calloc(nl, sizeof(double))))
	    mbs_alloc_error ("Copy Interaction Parameter", "Sigma");
	for (i=0 ; i<top ; i++)  aux[i] = _ML_Isigma[i];
	if (_ML_Isigma)  free (_ML_Isigma);
	_ML_Isigma = aux;
    }
}


double
_mbs_convert_molar_to_molar (int k, double c)
{
    return (c);
}
double
_mbs_convert_microgram_to_molar (int k, double c)
{
    return (1.0e-03*c/mbs_get_ligand_weight(k));
}
double
_mbs_convert_nanogram_to_molar (int k, double c)
{
    return (1.0e-06*c/mbs_get_ligand_weight(k));
}
double
_mbs_convert_molar_to_microgram (int k, double c)
{
    return (1.0e+03*c*mbs_get_ligand_weight(k));
}
double
_mbs_convert_molar_to_nanogram (int k, double c)
{
    return (1.0e+06*c*mbs_get_ligand_weight(k));
}


int
_mbs_check_dynamics_vs_equilibrium (double eq, double forward, double reverse)
{
    double   ceq;
    
    ceq = forward/reverse;
    if (fabs((ceq-eq)/eq)>1.0e-06)
    {
	fprintf (stderr, " -- Error --\n");
	fprintf (stderr, "Dynamical rate constants inconsistent with equilibrium constant.\n");
	fprintf (stderr, "Forward/Reverse = %e, Equilibrium = %e.\n", ceq, eq);
	return (0);
    }
    else
	return (1);
}
void
_mbs_check_effective_valence()
{
    int  k;
    
    for (k=0 ; k<mbs_get_number_of_ligands ; k++)
	if (mbs_get_ligand_valence(k) != mbs_get_ligand_effective_valence(k))
	{
	    fprintf (stderr, "MBS Initialize Ligand -- Error\n");
	    fprintf (stderr, "Ligand %d: Effective valence %d different from valence %d.\n", 
		     k+1, mbs_get_ligand_effective_valence(k), mbs_get_ligand_valence(k));
	    fprintf (stderr, "Valences must be identical, when you choose to model each\n");
	    fprintf (stderr, "    binding step with an individual equilibrium constant.\n");
	    fprintf (stderr, "Exiting now.\n");
	    exit(2);
	}
}


void
_mbs_ini_flags_n_pointers ()
{
    double   _mbs_receptor_conservation(double*, int*);
    void     _mbs_global_receptor_conservation(int*, double*, double*, int*);
    double   _mbs_convert_molar_to_molar(int, double);
    void      mbs_ini_bound_state_computation(double*);
    void      mbs_compute_bound_state_distribution();
    
    _ML_number_of_ligands    = 0;
    _ML_number_of_cell_types = 0;
    _ML_ligand_attributes_exist      = 0;
    _ML_cell_attributes_exist        = 0;
    _ML_interaction_parameters_exist = 0;
    _ML_ligand_scale   = -1;
    _ML_receptor_scale = -1;
    _ML_cell_scale     = -1;
    
    _ML_L = NULL;
    _ML_C = NULL;
    _ML_X = NULL;
    _ML_r = NULL;
    
    _ML_CTRL_lcontrib = NULL;
    _ML_CTRL_lcv      = NULL;
    _ML_CTRL_lcf      = NULL;
    _ML_CTRL_ccontrib = NULL;
    _ML_CTRL_ccv      = NULL;
    _ML_CTRL_ccf      = NULL;
    
    _ML_Iaf = NULL;
    _ML_Icc = NULL;
    _ML_Ifb = NULL;
    _ML_Irb = NULL;
    _ML_Ifc = NULL;
    _ML_Irc = NULL;
    
    _ML_Ikappa   = NULL;
    _ML_Ikappa_x = NULL;
    _ML_Inu      = NULL;
    _ML_Inu_x    = NULL;
    _ML_Imu      = NULL;
    _ML_Irho     = NULL;
    _ML_Ixi      = NULL;
    _ML_Isigma   = NULL;

    __ML_signal_function = NULL;
    if (!__ML_receptor_conservation)
	__ML_receptor_conservation = _mbs_receptor_conservation;
    if (!__ML_global_receptor_conservation)
	__ML_global_receptor_conservation = _mbs_global_receptor_conservation;
    __ML_ligand_increment_function = NULL;
    if (!__ML_ini_bound_state_computation)
	__ML_ini_bound_state_computation = mbs_ini_bound_state_computation;
    if (!__ML_compute_bound_state_distribution)
	__ML_compute_bound_state_distribution = mbs_compute_bound_state_distribution;
    
    __ML_convert_to_molar   = _mbs_convert_molar_to_molar;
    __ML_convert_from_molar = _mbs_convert_molar_to_molar;
}
