#include "mbs.h"

/*********************************************************************
**
**   MACROS:        
**
**   ROUTINES:      mbs_1d_bisec_froot
**                 
**********************************************************************/



/*********************************************************************
**
**   Routine Name:  mbs_1d_bisec_froot
**                  
**   Purpose:       
**
**   Input:         
**   Output:        
**
**   Called By:     
**
**   Written By:    Bernhard Sulzer, LANL
**   Last Edited:   
**
**********************************************************************/

double 
mbs_1d_bisec_froot(double (*f)(double), double a, double b, 
		   double eps, double tol, int *eflag)
{
    double  fa, fb, fm, m, root;
    
    fa = f(a);
    fb = f(b);
    
    if (a > b)
    {
	fprintf(stderr, "ERROR: lower bound (%e) greater than upper bound (%e).\n", a, b);
	fprintf(stderr, "Aborting root search.\n\n");
	*eflag = 1;
	return(0.0);
    }
    if (fa*fb > 0.0)
    {
	fprintf(stderr, "ERROR: no or multiple roots in [%e , %e].\n", a, b);
	fprintf(stderr, "Aborting root search.\n\n");
	*eflag = 2;
	return(0.0);
    }
    *eflag = 0;
    
    m = (a+b)/2.0;
    fm = f(m);
    
    /*
     * first check whether the absolute value of *f* at any of the three
     * points is less than the required tolerance
     */
    if (fabs(fa) < tol)      return(a);
    if (fabs(fb) < tol)      return(b);
    if (fabs(fm) < tol)      return(m);
    /*
     * root is left of *m*
     */
    if (fa*fm < 0.0)
    {
	if (m-a < eps)
	{
	    if (fabs(fa)<fabs(fm))    return(a);
	    else                      return(m);
	}
	root = mbs_1d_bisec_froot(f, a, m, eps, tol, eflag);
    }
    else if (fm*fb < 0.0)
    {
	if (b-m < eps)
	{
	    if (fabs(fm)<fabs(fb))    return(m);
	    else                      return(b);
	}
	root = mbs_1d_bisec_froot(f, m, b, eps, tol, eflag);
    }
    
    return (root);
}

    
