#include "mbs.h"

enum __CLAMS_SOS_PARAMETERS__
{    __CSOSP__EPS,   __CSOSP__ABS,   __CSOSP__FTOL,  __CSOSP__IFLAG, 
     __CSOSP__IW_1,  __CSOSP__IW_2,  __CSOSP__IW_3,  __CSOSP__RW_1    };
    
double   *__CSOS_rw=NULL;
int      *__CSOS_iw=NULL;
int       __CSOS_lrw, __CSOS_liw;
int       __CSOS_neq, __CSOS_iflag;
double    __CSOS_eps, __CSOS_abs, __CSOS_ftol;
double  (*__CSOS_function)(double*,int*);

void
mbs_clams_dsos (double *x)
{
    extern void   dsos_(double(*)(double*,int*), int*, double*, double*, double*, double*,
			int*, double*, int*, int*, int*);
    
    dsos_(__CSOS_function, &__CSOS_neq, x, &__CSOS_eps, &__CSOS_abs, &__CSOS_ftol,
	  &__CSOS_iflag, __CSOS_rw, &__CSOS_lrw, __CSOS_iw, &__CSOS_liw);
    if (__CSOS_iflag>4)  
    {
	fprintf (stderr, "MBS Clams DSOS -- Error\n");
	fprintf (stderr, "Error Code: %d.\n", __CSOS_iflag);
	fprintf (stderr, "Exiting now.\n");
	exit(2);
    }
}
void
mbs_ini_clams_dsos (int dim, double (*f)(double*,int*))
{
    __CSOS_eps = 1e-10;
    __CSOS_abs = 1e-10;
    __CSOS_ftol = 1e-10;
    
    __CSOS_iflag = 0;

    if (dim != __CSOS_neq)
    {
	__CSOS_lrw = 1 + 6*dim + ((dim+1)*dim)/2;
	__CSOS_liw = 3 + dim;
	if (__CSOS_rw)  free (__CSOS_rw);
	if (!(__CSOS_rw = (double *)calloc(__CSOS_lrw, sizeof(double))))
	    mbs_alloc_error ("MBS INI CLAMS DSOS", "Work array (double)");
	if (__CSOS_iw)  free (__CSOS_iw);
	if (!(__CSOS_iw = (int *)calloc(__CSOS_liw, sizeof(int))))
	    mbs_alloc_error ("MBS INI CLAMS DSOS", "Work array (integer)");
    }
    __CSOS_function = f;
    __CSOS_neq = dim;
}
void
mbs_set_clams_dsos_parameter (int id, void *value)
{
    static char     lower[32], upper[32], input[32];
    double         *dvalue;
    int            *ivalue;
    
    switch (id)
    {
      case  __CSOSP__IFLAG:
	ivalue = value;
	__CSOS_iflag = *ivalue;
	fprintf (stderr, "MBS SET CLAMS DSOS PARAMETER: IFLAG = %d\n", __CSOS_iflag);
	break;
      case  __CSOSP__IW_1:
	ivalue = value;
	__CSOS_iw[0] = *ivalue;
	fprintf (stderr, "MBS SET CLAMS DSOS PARAMETER: IW(1) = %d\n", __CSOS_iw[0]);
	break;
      case  __CSOSP__IW_2:
	ivalue = value;
	if (*ivalue < 1)
	{
	    sprintf (lower, "1"); sprintf (upper, "infinity"); sprintf (input, "%d", *ivalue);
	    mbs_out_of_range_error ("MBS SET CLAMS DSOS PARAMETER", "IW(2)", lower, upper, input);
	}
	__CSOS_iw[1] = *ivalue;
	fprintf (stderr, "MBS SET CLAMS DSOS PARAMETER: IW(2) = %d\n", __CSOS_iw[1]);
	break;
      case  __CSOSP__EPS:
	dvalue = value;
	if (*dvalue < 0.0)
	{
	    sprintf (lower, "0.0"); sprintf (upper, "infinity"); sprintf (input, "%e", *dvalue);
	    mbs_out_of_range_error ("MBS SET CLAMS DSOS PARAMETER", "EPS", lower, upper, input);
	}
	__CSOS_eps = *dvalue;
	fprintf (stderr, "MBS SET CLAMS DSOS PARAMETER: EPS = %e\n", __CSOS_eps);
	break;
      case  __CSOSP__ABS:
	dvalue = value;
	if (*dvalue < 0.0)
	{
	    sprintf (lower, "0.0"); sprintf (upper, "infinity"); sprintf (input, "%e", *dvalue);
	    mbs_out_of_range_error ("MBS SET CLAMS DSOS PARAMETER", "ABS", lower, upper, input);
	}
	__CSOS_abs = *dvalue;
	fprintf (stderr, "MBS SET CLAMS DSOS PARAMETER: ABS = %e\n", __CSOS_abs);
	break;
      case  __CSOSP__FTOL:
	dvalue = value;
	if (*dvalue < 0.0)
	{
	    sprintf (lower, "0.0"); sprintf (upper, "infinity"); sprintf (input, "%e", *dvalue);
	    mbs_out_of_range_error ("MBS SET CLAMS DSOS PARAMETER", "FTOL", lower, upper, input);
	}
	__CSOS_ftol = *dvalue;
	fprintf (stderr, "MBS SET CLAMS DSOS PARAMETER:  FTOL = %e\n", __CSOS_ftol);
	break;
      default:
	fprintf (stderr, "MBS SET CLAMS DSOS PARAMETER -- WARNING\n");
	fprintf (stderr, "Unknown parameter type: %d\n", id);
	fprintf (stderr, "Ignoring request...\n");
	break;
    }
}
void *
mbs_get_clams_dsos_parameter (int id)
{
    switch (id)
    {
      case __CSOSP__IFLAG:
	return (&__CSOS_iflag);
	break;
      case  __CSOSP__IW_3:
	return (__CSOS_iw+2);
	break;
      case  __CSOSP__RW_1:
	return (__CSOS_rw);
	break;
      default:
	fprintf (stderr, "MBS GET CLAMS DSOS PARAMETER -- WARNING\n");
	fprintf (stderr, "Unknown parameter type: %d\n", id);
	fprintf (stderr, "Ignoring request...\n");
	break;
    }
}

enum __CLAMS_NSQ_PARAMETERS__
{    __CNSQP__EPS,   __CNSQP__ABS,   __CNSQP__FTOL,  __CNSQP__IFLAG, 
     __CNSQP__IW_1,  __CNSQP__IW_2,  __CNSQP__IW_3,  __CNSQP__RW_1    };
    
double   *__CNSQ_rw=NULL;
int       __CNSQ_lrw;
int       __CNSQ_neq;
int       __CNSQ_info, __CNSQ_iopt, __CNSQ_nprint;
double    __CNSQ_eps;
void    (*__CNSQ_function)(int*,double*,double*,int*);
void    (*__CNSQ_jacobian)(int*,double*,double*,double*,int*,int*);

void
mbs_clams_dnsqe (double *x, double *y)
{
    extern void   dnsqe_(void(*)(int*,double*,double*,int*), 
			 void(*)(int*,double*,double*,double*,int*,int*),
			 int*, int*, double*, double*, double*, int*, int*, double*, int*);
    
    dnsqe_(__CNSQ_function, __CNSQ_jacobian, &__CNSQ_iopt, &__CNSQ_neq, x, y,
	   &__CNSQ_eps, &__CNSQ_nprint, &__CNSQ_info, __CNSQ_rw, &__CNSQ_lrw);
    if (__CNSQ_info != 1)  
    {
	fprintf (stderr, "MBS Clams DNSQE -- Error\n");
	fprintf (stderr, "Error Code: %d.\n", __CNSQ_info);
	fprintf (stderr, "Exiting now.\n");
	exit(2);
    }
}
void
mbs_ini_clams_dnsq (int dim, void (*f)(int*,double*,double*,int*), 
		    void (*j)(int*,double*,double*,double*,int*,int*))
{
    void    __CNSQ_dummy_jacobian(int*,double*,double*,double*,int*,int*);

    __CNSQ_eps = 1e-10;
    __CNSQ_nprint = -1;
    
    if (dim != __CNSQ_neq)
    {
	__CNSQ_lrw = ((3*dim+13)*dim)/2;
	if (__CNSQ_rw)  free (__CNSQ_rw);
	if (!(__CNSQ_rw = (double *)calloc(__CNSQ_lrw, sizeof(double))))
	    mbs_alloc_error ("MBS INI CLAMS DNSQ", "Work array (double)");
    }
    if (j) 
    {
	__CNSQ_iopt = 1;
	__CNSQ_jacobian = j;
    }
    else
    {
	__CNSQ_iopt = 2;
	__CNSQ_jacobian = __CNSQ_dummy_jacobian;
    }
    __CNSQ_function = f;
    __CNSQ_neq = dim;
}
void
__CNSQ_dummy_jacobian(int *a, double *b, double *c, double *d, int *e, int *f)
{
    mbs_dummy_catch("MBS library: CNSQ dummy jacobian");
}


