#ifndef _LIGAND_H_
#define _LIGAND_H_


#define  min(a,b)    ((a) < (b) ? (a) : (b))
#define  max(a,b)    ((a) > (b) ? (a) : (b))

#define  TRUE          1
#define  FALSE         0

typedef struct _LIGAND_ATTRIBUTES_ 
{
    double  weight;             /* multiplier to convert from molar units to $kg/l$     */
    double  w_micro;		/* multiplier to convert from molar units to $\mu g/ml$ */
    double  w_nano;		/* multiplier to convert from molar units to $ng/ml$    */
    
    int     N;			/* total number of sites */
    int     N_x;		/* effective number of sites (for crosslinks) */
    double  K;			/* binding affinity */
    double  K_x;       		/* crosslinking affinity */
    double  k_plus;		/* forward binding constant */
    double  k_minus;   		/* reverse binding constant */
    double  k_x_plus;   	/* forward crosslinking constant */
    double  k_x_minus; 		/* reverse crosslinking constant */
    double  kappa;		/* normalized binding affinity */
    double  kappa_x;		/* normalized crosslinking affinity */
				/* auxiliary variables */
    double  N_over_N_x;		/* N/N_x */
    double  fk;			/* kappa/kappa_x */
    double  mfk;		/* 1-kappa/kappa_x */
}
LigandAttributes;


typedef struct _STANDARD_LIGAND_VARIABLES_
{
    double      concentration;
    double      percentage_in_collection;
    int         vary_concentration;
}
StandardLigandVariables;


typedef struct _STANDARD_CONTROL_VARIABLES_ 
{
    double      begin;
    double      end;
    double      step_size;
    int         steps;

    int         number_of_ligands;
}
StandardControlVariables;


typedef struct _STANDARD_CELL_ATTRIBUTES_
{
    double      alpha;
    double      concentration;
    double      receptor_total;
    int         signal_size;
}
StandardCellAttributes;


/*
 *   Standard Global Variables
 */
LigandAttributes        **ligands;
StandardCellAttributes    cell;
StandardLigandVariables  *L;
StandardControlVariables  S;


/*********************************************************************
**
**                  Public subroutine interfaces
**
**   Written By:    Bernhard Sulzer, LANL
**
**********************************************************************/

double
determine_number_of_crosslinks (StandardControlVariables*, StandardCellAttributes*,
				StandardLigandVariables*, LigandAttributes**);
double
determine_number_of_aggregates (StandardControlVariables*, StandardCellAttributes*,
				StandardLigandVariables*, LigandAttributes**);
double
default_receptor_conservation(double);
double *
compute_bound_ligand_concentration (double, LigandAttributes*, double*, double);
double
compute_bound_receptors (double, LigandAttributes*, double);
void
initialize_bound_ligand_computation (int);
void
increment_concentrations(StandardControlVariables*, StandardLigandVariables**);

void 
default_get_arguments (char**, StandardControlVariables*, StandardCellAttributes*,
		       StandardLigandVariables**, LigandAttributes ***);
LigandAttributes *
set_ligand_attributes (LigandAttributes*, int, int, double, double, double, double,
		       double, double, double, double, double, double);
void
default_print_usage(char*);


#endif
