#ifndef _MBS_LIGAND_VARIABLES_H_
#define _MBS_LIGAND_VARIABLES_H_

/*
   general information
*/
int     _ML_number_of_ligands;
int     _ML_number_of_cell_types;

int     _ML_ligand_attributes_exist;
int     _ML_cell_attributes_exist;
int     _ML_interaction_parameters_exist;

double  _ML_ligand_scale_factor;
int     _ML_ligand_scale;
double  _ML_receptor_scale_factor;
int     _ML_receptor_scale;
double  _ML_cell_scale_factor;
int     _ML_cell_scale;
/*
   control parameters
*/
typedef struct _ML_CONTROL_PARAMETERS_
{
    double  ligand_bottom;
    double  ligand_top;
    double  ligand_stepsize;
    int     ligand_number_of_steps;
    int     ligand_scale_type;
}
ML_Control_Parameters;

ML_Control_Parameters  _ML_CTRL;

int    *_ML_CTRL_lcv;	      /* 1: vary concentration of ligand `k' */
int    *_ML_CTRL_lcontrib;    /* 1: ligand `k' contributes to signal */
double *_ML_CTRL_lcf;	      /* fraction of ligand `k' in mixture */

int    *_ML_CTRL_ccv;	      /* 1: vary concentration of cell `l' */
int    *_ML_CTRL_ccontrib;    /* 1: cell `l' contributes to signal */
double *_ML_CTRL_ccf;	      /* fraction of cell `l' in mixture */

/*
   ligand parameters
*/
typedef struct _ML_LIGAND_ATTRIBUTES_ 
{
    double  weight;	      /* multiplier to convert from molar units to $kg/l$     */
    double  w_micro;	      /* multiplier to convert from molar units to $\mu g/ml$ */
    double  w_nano;	      /* multiplier to convert from molar units to $ng/ml$    */
    
    int     n;		      /* valence */
    int     n_x;	      /* effective valence */

    double  concentration;    /* ligand concentration (molar units) */
}
ML_Ligand_Attributes;

ML_Ligand_Attributes  **_ML_L;
/*
   cell parameters
*/
typedef struct _ML_CELL_ATTRIBUTES_
{
    double  alpha;	      /* surface/volume concentration converter */
    int     signal_size;      /* number of receptors/ligand to generate signal */
    
    double  concentration;    /* cell concentration */
    double  numb_receptor;    /* total receptor density */
}
ML_Cell_Attributes;

ML_Cell_Attributes    **_ML_C;
/*
   bound ligand states
*/
double ***_ML_X;
/*
   fraction of receptor sites that are free
*/
double   *_ML_r;
double   *_ML_rf;
/*
   interaction parameters
*/
double  **_ML_Iaf;	      /* affinity */
double  **_ML_Icc;	      /* crosslinking constant */
double  **_ML_Ifb;	      /* forward binding rate constant */
double  **_ML_Irb;	      /* reverse binding rate constant */
double  **_ML_Ifc;	      /* forward crosslinking rate constant */
double  **_ML_Irc;	      /* reverse crosslinking rate constant */
/*                               
   interaction parameters --- auxiliary
*/
double  **_ML_Ikappa;	      /* non-dimensional affinity */
double  **_ML_Ikappa_x;	      /* non-dimensional crosslinking constant */
double ***_ML_Ikappa_i;	      /* non-dimensional equilibrium constants */
double  **_ML_Inu;	      /* valence \times kappa */
double  **_ML_Inu_x;	      /* effective valence \times kappa_x */
double  **_ML_Imu;	      /* _ML_Inu/_ML_Inu_x */
double ***_ML_Ipi;	      /* product of K_kl{j} from j=1 to j=i */

double  **_ML_Irho;	      /* kappa_x*r + 1.0 (receptor conservation) */
double  **_ML_Ixi;	      /* n*K*X_T/(1+sigma) (receptor conservation) */
double   *_ML_Isigma;	      /* number of ligand bound (receptor conservation) */

/*
   function pointers
*/
double  (*__ML_signal_function)(int, int);
double  (*__ML_receptor_conservation)(double*, int*);
void    (*__ML_global_receptor_conservation)(int*, double*, double*, int*);
double  (*__ML_ligand_increment_function)(double);
double  (*__ML_convert_to_molar)(int, double);
double  (*__ML_convert_from_molar)(int, double);
void    (*__ML_write_signal)(FILE*, double);
void    (*__ML_ini_bound_state_computation)(double*);  
void    (*__ML_compute_bound_state_distribution)();  

#endif
