#ifndef _MBS_LIGAND_H_
#define _MBS_LIGAND_H_

#include "mbs-ligand-variables.h"
#include "mbs-ligand-prototypes.h"

enum _ML_LIGAND_SCALES_
{    _ML_MOLAR,    _ML_MICROGRAM_PER_MILLILITER,    _ML_NANOGRAM_PER_MILLILITER    };
enum _ML_CELL_SCALES_
{    _ML_ONE_PER_MILLILITER    };
enum _ML_RECEPTOR_SCALES_
{    _ML_ONE_PER_CELL,    _ML_ONE_PER_CM_SQUARED    };
enum _ML_LIGAND_SCALE_TYPES_
{    _ML_LS_LINEAR, _ML_LS_LOGARITHMIC   };
enum _ML_SIGNALS_
{    _ML_CROSSLINKS, _ML_AGGREGATES   };
enum _ML_SIGNAL_MODES_
{    _ML_GLOBAL_SIGNAL, _ML_SIGNAL_PER_CELL   };


#define  mbs_get_number_of_ligands                         (_ML_number_of_ligands)
#define  mbs_get_number_of_cell_types                      (_ML_number_of_cell_types)
#define  mbs_get_ligand_weight(k)                          (_ML_L[(k)]->weight)
#define  mbs_get_ligand_valence(k)                         (_ML_L[(k)]->n)
#define  mbs_get_ligand_effective_valence(k)               (_ML_L[(k)]->n_x)
#define  mbs_get_ligand_concentration(k)                   (_ML_L[(k)]->concentration)
#define  mbs_get_cell_scale(l)                             (_ML_C[(l)]->alpha)
#define  mbs_get_cell_signal_size(l)                       (_ML_C[(l)]->signal_size)
#define  mbs_get_cell_concentration(l)                     (_ML_C[(l)]->concentration)
#define  mbs_get_cell_number_of_receptors(l)               (_ML_C[(l)]->numb_receptor)
#define  mbs_get_affinity(k, l)                            (_ML_Iaf[(k)][(l)])
#define  mbs_get_crosslinking_constant(k, l)               (_ML_Icc[(k)][(l)])
#define  mbs_get_forward_binding(k, l)                     (_ML_Ifb[(k)][(l)])
#define  mbs_get_reverse_binding(k, l)                     (_ML_Irb[(k)][(l)])
#define  mbs_get_forward_crosslinking(k, l)                (_ML_Ifc[(k)][(l)])
#define  mbs_get_reverse_crosslinking(k, l)                (_ML_Irc[(k)][(l)])
#define  mbs_get_nondimensional_binding_constant(k, l, i)  (_ML_Ikappa_i[k][l][i])

#define  mbs_get_free_receptor_sites(l)                    (_ML_r[(l)])
#define  mbs_get_bound_state(k, l, i)                      (_ML_X[(k)][(l)][(i)])
#define  mbs_get_crosslinks(k, l, i)                       ((double)i*_ML_X[(k)][(l)][(i)])
#define  mbs_get_receptor_sites_bound(k, l, i)             ((double)(i+1)*_ML_X[(k)][(l)][(i)])

#define  mbs_set_ligand_concentration(k, c)                (_ML_L[(k)]->concentration =        \
				            		    (*__ML_convert_to_molar)((k),(c)))
#define  mbs_set_cell_concentration(l, c)                  (_ML_C[(l)]->concentration = c)
#define  mbs_set_cell_scale(scale)                         (_ML_cell_scale = (scale))
#define  mbs_set_receptor_scale(scale)                     (_ML_receptor_scale = (scale))

#define  mbs_vary_ligand_concentration(k)                  (_ML_CTRL_lcv[(k)])
#define  mbs_ligand_contributes(k)                         (_ML_CTRL_lcontrib[(k)])
#define  mbs_get_ligand_fraction(k)                        (_ML_CTRL_lcf[(k)])
#define  mbs_vary_cell_concentration(l)                    (_ML_CTRL_ccv[(l)])
#define  mbs_cell_type_contributes(l)                      (_ML_CTRL_ccontrib[(l)])
#define  mbs_get_cell_fraction(l)                          (_ML_CTRL_ccf[(l)])

#define  false   0
#define  true    1

    

#endif
