#include "mbs.h"

/*
	for (k=0 ; k<mbs_get_number_of_ligands ; k++)
	    for (l=0 ; l<mbs_get_number_of_cell_types ; l++)
		for (i=0 ; i<mbs_get_ligand_effective_valence(k) ; i++)
		    printf ("%.15e ", mbs_get_bound_state(k, l, i));
	printf ("\n");
*/

#define   min(a,b)    ((a)<(b) ? (a) : (b))

void
mbs_initialize_1L1C_aggregation (int type, int scale)
{
    mbs_initialize_receptor_aggregation (1, 1);
    mbs_set_ligand_scale_converters (scale);
    _ML_cell_scale     = _ML_ONE_PER_MILLILITER;
    _ML_receptor_scale = _ML_ONE_PER_CELL;
    mbs_select_signal_function (type);
    (*__ML_ini_bound_state_computation)(NULL);
}
double
mbs_1L1C_get_aggregates (double concentration)
{
    mbs_set_ligand_concentration(0, concentration);
    (*__ML_compute_bound_state_distribution)();
/*    (*__ML_write_signal)(stdout, concentration); */
    return (mbs_get_signal_strength());
}


void
mbs_initialize_receptor_aggregation (int nl, int nc)
{
    double   fl, fc;
    int      k, l;
    void    _mbs_ini_flags_n_pointers();
    void     mbs_set_ligand_control_variables(int, int, int, double);
    void     mbs_set_cell_control_variables(int, int, int, double);
    
    _mbs_ini_flags_n_pointers();
    mbs_set_number_of_ligands (nl);
    mbs_set_number_of_cell_types (nc);
    
    fl = 1.0/(double)nl;
    for (k=0 ; k<nl ; k++)
	mbs_set_ligand_control_variables (k, 1, 1, fl);
    fc = 1.0/(double)nc;
    for (l=0 ; l<nc ; l++)
	mbs_set_cell_control_variables (l, 1, 1, fc);
}


int
mbs_ligand_binding_curve (char *filename, double *guess)
{
    double    ligand_conc, ligand_top, signal;
    void      mbs_select_output_function(int);
    int       l;
    FILE     *out;
	
    (*__ML_ini_bound_state_computation)(guess);
    if (!mbs_get_concentration_boundaries(&ligand_conc, &ligand_top))  return(0);
    if (!(out = fopen(filename,"w")))
	mbs_file_open_error ("Ligand Binding Curve", filename);
    while (ligand_conc <= ligand_top)
    {
	(*__ML_compute_bound_state_distribution)();
	(*__ML_write_signal)(out, ligand_conc);
	mbs_increment_ligand_concentration (&ligand_conc);
    }
    fclose (out);
}
void
mbs_write_signal_strength (FILE *outfile, double concentration)
{
    int    i, k, l, nx;
    double rt;
    
    fprintf (outfile, "%e ", concentration);
    for (l=0 ; l<mbs_get_number_of_cell_types ; l++)
	fprintf (outfile, "%e ", mbs_get_signal_strength(l));
    for (l=0 ; l<mbs_get_number_of_cell_types ; l++)
    {
	fprintf(outfile, " %e", mbs_get_free_receptor_sites(l));
/*	rt = mbs_get_cell_number_of_receptors(l);
	for (k=0 ; k<mbs_get_number_of_ligands ; k++)
	{
	    nx = mbs_get_ligand_effective_valence(k);
	    for (i=0 ; i<nx ; i++)
		fprintf(outfile, " %e", mbs_get_bound_state(k, l, i)/rt);
	}
*/
    }
    fprintf (outfile, "\n");
}
void
mbs_write_signal_strength_per_cell (FILE *outfile, double concentration)
{
    int   l;
    
    fprintf (outfile, "%e ", concentration);
    for (l=0 ; l<mbs_get_number_of_cell_types ; l++)
	fprintf (outfile, "%e ", mbs_get_signal_strength_per_cell(l));
    for (l=0 ; l<mbs_get_number_of_cell_types ; l++)
	fprintf(outfile, " %e", mbs_get_free_receptor_sites(l));
    fprintf (outfile, "\n");
}
double
mbs_get_signal_strength ()
{
    int     k, l;
    double  signal = 0.0;
    
    for (k=0 ; k<_ML_number_of_ligands ; k++)
	if (mbs_ligand_contributes(k))
	    for (l=0 ; l<_ML_number_of_cell_types ; l++)
		if (mbs_cell_type_contributes(l))
		    signal += (mbs_get_cell_fraction(l)*(*__ML_signal_function)(k, l));
    return (signal);
}
double
mbs_get_signal_strength_per_cell (int l)
{
    int     k;
    double  signal = 0.0;
    
    for (k=0 ; k<_ML_number_of_ligands ; k++)
	if (mbs_ligand_contributes(k))
	    signal += (*__ML_signal_function)(k, l);
    return (signal);
}
void 
mbs_ini_bound_state_computation_vk(double *guess)
{
    void     mbs_ini_clams_dsos(int, double(*)(double*,int*));
    void     mbs_ini_clams_dnsq(int, void(*)(int*,double*,double*,int*), 
				void(*)(int*,double*,double*,double*,int*,int*));
    void     mbs_set_receptor_conservation(double(*)(double*,int*));
    double  _mbs_receptor_conservation_vk(double*,int*);
    int      k;

    mbs_set_receptor_conservation(_mbs_receptor_conservation_vk);
/*    
    mbs_ini_clams_dsos (_ML_number_of_cell_types, __ML_receptor_conservation);
*/
    mbs_ini_clams_dnsq (_ML_number_of_cell_types, __ML_global_receptor_conservation, NULL);

    if (guess)  
	for (k=0 ; k<_ML_number_of_cell_types ; k++)  _ML_r[k] = guess[k];
    else
	for (k=0 ; k<_ML_number_of_cell_types ; k++)  _ML_r[k] = 0.5;
}
void
mbs_compute_bound_state_distribution_vk ()
{
    int       i, j, k, l, n;
    double    rl, x0;
    void      mbs_clams_dsos(double*);
    void      mbs_clams_dnsqe(double*, double*);
    void     _mbs_compute_sigma_vk (int, double*);
/*    
    mbs_clams_dsos(_ML_r);
*/
    mbs_clams_dnsqe(_ML_r, _ML_rf);

    for (k=0 ; k<_ML_number_of_ligands ; k++)
    {
	_mbs_compute_sigma_vk (k, _ML_r);
	x0 = mbs_get_ligand_concentration(k)/(1.0+_ML_Isigma[k]);
	n = _ML_L[k]->n;
	for (l=0 ; l<_ML_number_of_cell_types ; l++)
	{
	    rl = _ML_r[l];
	    _ML_X[k][l][0] = x0 * n * _ML_Ikappa_i[k][l][0] * rl;
	    for (j=2,i=1 ; i<n ; i++,j++)
	        _ML_X[k][l][i] = rl*(double)(n-i)/(double)j*_ML_Ikappa_i[k][l][i]*_ML_X[k][l][i-1];
	}
    }
}
void 
mbs_ini_bound_state_computation(double *guess)
{
    void     mbs_ini_clams_dsos(int, double(*)(double*,int*));
    void     mbs_ini_clams_dnsq(int, void(*)(int*,double*,double*,int*), 
				void(*)(int*,double*,double*,double*,int*,int*));
    int      k;
/*    
    mbs_ini_clams_dsos (_ML_number_of_cell_types, __ML_receptor_conservation);
*/
    mbs_ini_clams_dnsq (_ML_number_of_cell_types, __ML_global_receptor_conservation, NULL);

    if (guess)  
	for (k=0 ; k<_ML_number_of_cell_types ; k++)  _ML_r[k] = guess[k];
    else
	for (k=0 ; k<_ML_number_of_cell_types ; k++)  _ML_r[k] = 0.5;
}
void
mbs_compute_bound_state_distribution ()
{
    int       n_x, a, b, i, k, l;
    double    kr;
    void      mbs_clams_dsos(double*);
    void      mbs_clams_dnsqe(double*, double*);
    void     _mbs_compute_rho(double*);
    void     _mbs_compute_auxiliary_parameters(int,int);
/*    
    mbs_clams_dsos(_ML_r);
*/
    mbs_clams_dnsqe(_ML_r, _ML_rf);

    _mbs_compute_rho(_ML_r);
    for (k=0 ; k<_ML_number_of_ligands ; k++)
    {
	n_x = _ML_L[k]->n_x;
	for (l=0 ; l<_ML_number_of_cell_types ; l++)
	{
	    _mbs_compute_auxiliary_parameters(k, l);
	    _ML_X[k][l][0] = _ML_Ixi[k][l]*_ML_C[l]->numb_receptor*_ML_r[l];
	    kr = _ML_Ikappa_x[k][l]*_ML_r[l];
	    for (b=n_x-1,a=2,i=1 ; i<n_x ; i++,a++,b--)
		_ML_X[k][l][i] = _ML_X[k][l][i-1]*(double)b/(double)a*kr;
	}
    }
}


int
mbs_get_concentration_boundaries (double *bottom, double *top)
{
    if (_ML_CTRL.ligand_bottom >= _ML_CTRL.ligand_top)
    {
	fprintf(stderr, "MBS: Get Concentration Boundaries -- Error.\n");
	fprintf(stderr, "\tLigand concentration interval:\n");
	fprintf(stderr, "\t Lower boundary (%e) . greater or equal . Upper boundary (%e).\n",
		_ML_CTRL.ligand_bottom, _ML_CTRL.ligand_top);
	return (0);
    }
    *bottom = _ML_CTRL.ligand_bottom;
    *top    = _ML_CTRL.ligand_top;
    return (1);
}
void
mbs_ini_ligand_concentrations ()
{
    double  c;
    int     k;
    
    c = _ML_CTRL.ligand_bottom;
    for (k=0 ; k<_ML_number_of_ligands ; k++)
	if (_ML_CTRL_lcv[k])
	    mbs_set_ligand_concentration(k, c*_ML_CTRL_lcf[k]);
	    
}
void
mbs_increment_ligand_concentration (double *current)
{
    int      k;
    
    for (k=0 ; k<_ML_number_of_ligands ; k++)
	if (mbs_vary_ligand_concentration(k))
	    _ML_L[k]->concentration = (*__ML_ligand_increment_function)(_ML_L[k]->concentration);
    *current = (*__ML_ligand_increment_function)(*current);
}    


void
mbs_select_output_function(int mode)
{
    void    mbs_write_signal_strength(FILE*, double);
    void    mbs_write_signal_strength_per_cell(FILE*, double);
    
    if      (mode == _ML_GLOBAL_SIGNAL)
	__ML_write_signal = mbs_write_signal_strength;
    else if (mode == _ML_SIGNAL_PER_CELL)
	__ML_write_signal = mbs_write_signal_strength_per_cell;
    else
    {
	fprintf (stderr, "MBS Ligand Binding Curve -- Error\n");
	fprintf (stderr, "Unknown data collection mode: %d.\n", mode);
	fprintf (stderr, "Must be %d or %d.\n", _ML_GLOBAL_SIGNAL, _ML_SIGNAL_PER_CELL);
	fprintf (stderr, "Exiting now.\n");
	exit (2);
    }
}
void 
mbs_select_signal_function(int type)
{
    double   _mbs_compute_signaling_crosslinks(int, int);
    double   _mbs_compute_signaling_aggregates(int, int);
    
    switch (type)
    {
      case _ML_CROSSLINKS:
	__ML_signal_function = _mbs_compute_signaling_crosslinks;
	break;
      case _ML_AGGREGATES:
	__ML_signal_function = _mbs_compute_signaling_aggregates;
	break;
      default:
	break;
    }
}
void
mbs_set_signal_function(double(*f)(int,int))
{
    __ML_signal_function = f;
}
void
mbs_set_receptor_conservation (double(*f)(double*,int*))
{
    __ML_receptor_conservation = f;
}
void 
mbs_set_bound_state_computation (void(*ini)(double*), void(*calc)())
{
    __ML_ini_bound_state_computation      = ini;
    __ML_compute_bound_state_distribution = calc;
}


void
mbs_set_ligand_control_variables (int index, int vary, int contrib, double frac)
{
    _ML_CTRL_lcf[index] = frac;
    _ML_CTRL_lcv[index] = vary;
    _ML_CTRL_lcontrib[index] = contrib;
}
void
mbs_set_cell_control_variables (int index, int vary, int contrib, double frac)
{
    _ML_CTRL_ccf[index] = frac;
    _ML_CTRL_ccv[index] = vary;
    _ML_CTRL_ccontrib[index] = contrib;
}
void
mbs_set_interaction_parameters_vk (int k, int l, double *affinity)
{
    double                 rt;
    int                    i, nx;
    
    rt = _ML_C[l]->numb_receptor;
    nx = mbs_get_ligand_effective_valence(k);
    for (i=0 ; i<nx ; i++)
	_ML_Ikappa_i[k][l][i] = rt * affinity[i];
}

void
mbs_set_interaction_parameters (int k, int l, double affinity, double crosslinking_constant,
				double fb, double rb, double fc, double rc)
{
    ML_Ligand_Attributes  *ligand;
    ML_Cell_Attributes    *cell;
    
    ligand = _ML_L[k];
    cell   = _ML_C[l];
    
    _ML_Iaf[k][l]      = affinity;
    _ML_Icc[k][l]      = crosslinking_constant;
    _ML_Ifb[k][l]      = fb;
    _ML_Irb[k][l]      = rb;
    _ML_Ifc[k][l]      = fc;
    _ML_Irc[k][l]      = rc;

    _ML_Ikappa[k][l]   = cell->alpha * cell->concentration * cell->numb_receptor * affinity;
    _ML_Ikappa_x[k][l] = cell->numb_receptor * crosslinking_constant;
    _ML_Inu[k][l]      = ligand->n * _ML_Ikappa[k][l];
    _ML_Inu_x[k][l]    = ligand->n_x * _ML_Ikappa_x[k][l];
    _ML_Imu[k][l]      = _ML_Inu[k][l]/_ML_Inu_x[k][l];
}
void
mbs_set_ligand_attributes (int index, double weight, int n, int n_x, double concentration)
{
    int        i, l, nx;
    double    *aux;
    
    if (n_x != _ML_L[index]->n_x)
    {
	nx = min(n_x, _ML_L[index]->n_x);
	for (l=0 ; l<_ML_number_of_cell_types ; l++) 
	{
	    if (_ML_X[index][l])  free (_ML_X[index][l]);
	    if (!(_ML_X[index][l] = (double *)calloc(n_x, sizeof(double))))
		mbs_alloc_error ("Set Ligand Attributes", "Bound State");
	    if (!(aux = (double *)calloc(n_x, sizeof(double))))
		mbs_alloc_error ("Set Ligand Attributes", "Auxiliary Array");
	    if (_ML_Ikappa_i[index][l])
	    {
		for (i=0 ; i<nx ; i++)   aux[i] = _ML_Ikappa_i[index][l][i];
		free (_ML_Ikappa_i[index][l]);
	    }
	    _ML_Ikappa_i[index][i] = aux;
	}
    }
    _ML_L[index]->weight        = weight;
    _ML_L[index]->w_micro       = weight*1e3;
    _ML_L[index]->w_nano        = weight*1e6;
    _ML_L[index]->n             = n;
    _ML_L[index]->n_x           = n_x;
    mbs_set_ligand_concentration(index, concentration);

    for (l=0 ; l<_ML_number_of_cell_types ; l++) 
    {
	_ML_Inu[index][l] = n * _ML_Ikappa[index][l];
	_ML_Inu_x[index][l] = n_x * _ML_Ikappa_x[index][l];
	_ML_Imu[index][l] = _ML_Inu[index][l]/_ML_Inu_x[index][l];
    }
}
void
mbs_set_cell_attributes (int index, double alpha, int signal_size, 
			 double concentration, double numb_receptor)
{
    ML_Ligand_Attributes  *ligand;
    int                    k;
    
    _ML_C[index]->alpha         = alpha;
    _ML_C[index]->signal_size   = signal_size;
    _ML_C[index]->concentration = concentration;
    _ML_C[index]->numb_receptor = numb_receptor;

    for (ligand=*_ML_L,k=0 ; k<_ML_number_of_ligands ; k++,ligand++)
    {
	_ML_Ikappa[k][index] = alpha * concentration * numb_receptor * _ML_Iaf[k][index];
	_ML_Ikappa_x[k][index] = numb_receptor * _ML_Icc[k][index];
	_ML_Inu[k][index] = ligand->n * _ML_Ikappa[k][index];
	_ML_Inu_x[k][index] = ligand->n_x * _ML_Ikappa_x[k][index];
	_ML_Imu[k][index] = _ML_Inu[k][index]/_ML_Inu_x[k][index];
    }
}
void
mbs_set_number_of_ligands (int n)
{
    void       _mbs_update_control_variables(int, int);
    void       _mbs_update_ligand_attributes(int);
    void       _mbs_update_interaction_parameters(int, int);
    double  ***_mbs_copy_X(int, int, int, int, double***);

    if (_ML_number_of_ligands != n)
    {
	_mbs_update_control_variables(n, _ML_number_of_cell_types);
	_mbs_update_ligand_attributes(n);
	_mbs_update_interaction_parameters(n, _ML_number_of_cell_types);
	_ML_Ikappa_i = _mbs_copy_X (n, _ML_number_of_ligands, _ML_number_of_cell_types, 
				    _ML_number_of_cell_types, _ML_Ikappa_i);
	_ML_X = _mbs_copy_X (n, _ML_number_of_ligands, _ML_number_of_cell_types, 
			     _ML_number_of_cell_types, _ML_X);
	_ML_number_of_ligands = n;
    }
}
void
mbs_set_number_of_cell_types (int n)
{
    void       _mbs_update_control_variables(int, int);
    void       _mbs_update_cell_attributes(int);
    void       _mbs_update_interaction_parameters(int, int);
    double  ***_mbs_copy_X(int, int, int, int, double***);

    if (_ML_number_of_cell_types != n)
    {
	_mbs_update_control_variables(_ML_number_of_ligands, n);
	_mbs_update_cell_attributes(n);
	_mbs_update_interaction_parameters(_ML_number_of_ligands, n);
	_ML_Ikappa_i = _mbs_copy_X (_ML_number_of_ligands, _ML_number_of_ligands, n,
				    _ML_number_of_cell_types, _ML_Ikappa_i);
	_ML_X = _mbs_copy_X (_ML_number_of_ligands, _ML_number_of_ligands, n,
			     _ML_number_of_cell_types, _ML_X);
	if (_ML_r)  free (_ML_r);
	if (!(_ML_r = (double *)calloc(n, sizeof(double))))
	    mbs_alloc_error("Set Number of Cell Types", "Free Receptor Sites");
	if (_ML_rf)  free (_ML_rf);
	if (!(_ML_rf = (double *)calloc(n, sizeof(double))))
	    mbs_alloc_error("Set Number of Cell Types", "Free Receptor Site Functions");
	_ML_number_of_cell_types = n;
    }
}


int
mbs_file_ini_ligand_binding_curve_vk (char *control_file, char *ligand_file,
				      char *cell_file, char *interaction_file)
{
    void   _mbs_ini_flags_n_pointers ();
    void   _mbs_check_effective_valence ();
    void    mbs_ini_ligand_concentrations ();
    
    _mbs_ini_flags_n_pointers();
    if (!mbs_read_control_variable_file (control_file, false))   return (0);
    if (!mbs_read_ligand_file (ligand_file, true))               return (0);
    _mbs_check_effective_valence();
    if (!mbs_read_cell_type_file (cell_file, true))              return (0);
    if (!mbs_read_interaction_file_vk (interaction_file, true))  return (0);
    mbs_ini_ligand_concentrations();
    return(1);
}
int
mbs_file_ini_ligand_binding_curve (char *control_file, char *ligand_file,
				   char *cell_file, char *interaction_file)
{
    void   _mbs_ini_flags_n_pointers ();
    void   mbs_ini_ligand_concentrations ();
    
    _mbs_ini_flags_n_pointers();
    if (!mbs_read_control_variable_file (control_file, false))  return (0);
    if (!mbs_read_ligand_file (ligand_file, true))              return (0);
    if (!mbs_read_cell_type_file (cell_file, true))             return (0);
    if (!mbs_read_interaction_file (interaction_file, true))    return (0);
    mbs_ini_ligand_concentrations();
    return(1);
}

char    dummy[256];

int
mbs_read_ligand_file (char *filename, int check_compatibility)
{
    FILE        *Lfile;
    void         mbs_set_ligand_concentration_scale (int, double);
    static char  lscale[128];
    float        weight, concentration;
    int          k, nl, n, n_x, ls=-1;
    
    if (!(Lfile = fopen (filename, "r")))
	mbs_file_open_error ("Read Ligand Attributes", filename);
    fscanf (Lfile, "%s %s", &dummy, &lscale);
    if (check_compatibility)
    {
	if      (strcmp(lscale,"default")==0 || strcmp(lscale,"molar")==0)
	    ls = _ML_MOLAR;
	else if (strcmp(lscale,"microgram")==0 || strcmp(lscale, "microgram_per_milliliter")==0)
	    ls = _ML_MICROGRAM_PER_MILLILITER;
	else if (strcmp(lscale,"nanogram")==0 || strcmp(lscale, "nanogram_per_milliliter")==0)
	    ls = _ML_NANOGRAM_PER_MILLILITER;
	
	if (ls != _ML_ligand_scale)
	{
	    fprintf(stderr, "Read Ligand Attributes -- Error.\n");
	    fprintf(stderr, "Ligand scale incompatible with previous definition.\n");
	    fprintf(stderr, "Previous value: %d\tnew value: %d\n", _ML_ligand_scale, ls);
	    fprintf(stderr, "Exiting now.\n");
	    exit (2);
	}
    }
    fscanf (Lfile, "%s %d", &dummy, &nl);
    if (check_compatibility && nl != _ML_number_of_ligands)
    {
	fprintf(stderr, "Read Ligand Attributes -- Error.\n");
	fprintf(stderr, "Number of ligands incompatible with previous definition.\n");
	fprintf(stderr, "Previous value: %d\tnew value: %d\n", _ML_number_of_ligands, nl);
	fprintf(stderr, "Exiting now.\n");
	exit (2);
    }
    for (k=0 ; k<nl ; k++)
    {
	fscanf(Lfile, "%s %e %d %d %e", &dummy, &weight, &n, &n_x, &concentration);
	mbs_set_ligand_attributes (k, (double)weight, n, n_x, (double)concentration);
    }
    fclose (Lfile);
    
    return (1);
}

#define  ALPHA    1.666666666666667E-21

int
mbs_read_cell_type_file (char *filename, int check_compatibility)
{
    static char  cscale[128], rscale[128];
    FILE        *Cfile;
    double       total=0.0;
    float        alpha, concentration, number_of_receptors;
    int          l, nc, signal_size, cs=-1, rs=-1;
    
    if (!(Cfile = fopen (filename, "r")))
	mbs_file_open_error ("Read Cell Attributes", filename);
    fscanf (Cfile, "%s %s %s", &dummy, &cscale, &rscale);
    if (check_compatibility)
    {
	if      (strcmp(cscale,"default")==0 || strcmp(cscale,"per_ml")==0)
	    cs = _ML_ONE_PER_MILLILITER;
	if (cs != _ML_cell_scale)
	{
	    fprintf(stderr, "Read Cell Attributes -- Error.\n");
	    fprintf(stderr, "Cell scale incompatible with previous definition.\n");
	    fprintf(stderr, "Previous value: %d\tnew value: %d\n", _ML_cell_scale, cs);
	    fprintf(stderr, "Exiting now.\n");
	    exit (2);
	}
	if      (strcmp(rscale,"default")==0 || strcmp(rscale,"per_cell")==0)
	    rs = _ML_ONE_PER_CELL;
	else if (strcmp(rscale,"per_cm_squared")==0)
	    rs = _ML_ONE_PER_CM_SQUARED;
	if (rs != _ML_receptor_scale)
	{
	    fprintf(stderr, "Read Cell Attributes -- Error.\n");
	    fprintf(stderr, "Receptor scale incompatible with previous definition.\n");
	    fprintf(stderr, "Previous value: %d\tnew value: %d\n", _ML_receptor_scale, rs);
	    fprintf(stderr, "Exiting now.\n");
	    exit (2);
	}
    }
    fscanf (Cfile, "%s %d", &dummy, &nc);
    if (check_compatibility && nc != _ML_number_of_cell_types)
    {
	fprintf(stderr, "Read Cell Attributes -- Error.\n");
	fprintf(stderr, "Number of cell types incompatible with previous definition.\n");
	fprintf(stderr, "Previous value: %d\tnew value: %d\n", _ML_number_of_cell_types, nc);
	fprintf(stderr, "Exiting now.\n");
	exit (2);
    }
    for (l=0 ; l<nc ; l++)
    {
	fscanf(Cfile, "%s %e %d %e %e", &dummy, &alpha, &signal_size, 
	       &concentration, &number_of_receptors);
	if (_ML_cell_scale==_ML_ONE_PER_MILLILITER && _ML_receptor_scale==_ML_ONE_PER_CELL &&
	    fabs((alpha-ALPHA)/ALPHA)>1.0e-06)
	{
	    fprintf (stderr, "Read Cell Attributes -- Error.\n");
	    fprintf (stderr, "For cell scale [1/ml] and receptor scale [1/cell],\n");
	    fprintf (stderr, "the scale factor must be alpha = 1.6666666666E-21,\n");
	    fprintf (stderr, "not alpha = %e.\n", alpha);
	    fprintf (stderr, "Exiting now.\n");
	    exit (2);
	}
	mbs_set_cell_attributes (l, (double)alpha, signal_size, 
				 (double)concentration, (double)number_of_receptors);
	total += concentration;
    }
    fclose (Cfile);

    for (l=0 ; l<nc ; l++)
	_ML_CTRL_ccf[l] = mbs_get_cell_concentration(l)/total;
    
    return (1);
}
int
mbs_read_interaction_file_vk (char *filename, int check_compatibility)
{
    FILE        *Ifile;
    double      *affinity;
    float        K, Kx, fb, rb, fc, rc, ec;
    int          i, k, l, nl, nc, nx, n, count=0;
    void         mbs_set_interaction_parameters_vk (int, int, double*);

    if (!(Ifile = fopen (filename, "r")))
	mbs_file_open_error ("Read Interaction Parameters", filename);
    fscanf (Ifile, "%s %d %d", &dummy, &nl, &nc);
    if (check_compatibility && 
	(nl != _ML_number_of_ligands || nc != _ML_number_of_cell_types))
    {
	fprintf(stderr, "Read Interaction Parameters -- Error.\n");
	if (nl != _ML_number_of_ligands)
	{
	    fprintf(stderr, "Number of ligands incompatible with previous definition.\n");
	    fprintf(stderr, "Previous value: %d\tnew value: %d\n", _ML_number_of_ligands, nl);
	}
	if (nc != _ML_number_of_cell_types)
	{
	    fprintf(stderr, "Number of cell types incompatible with previous definition.\n");
	    fprintf(stderr, "Previous value: %d\tnew value: %d\n", _ML_number_of_cell_types, nc);
	}
	fprintf(stderr, "Exiting now.\n");
	exit (2);
    }
    affinity = NULL;
    n = nl*nc;
    while (1)
    {
	fscanf (Ifile, "%d %d %d", &k, &l, &nx);
	if (feof(Ifile))  break;
	if (++count>n)
	{
	    fprintf (stderr, "Read Interaction Parameters -- Error.\n");
	    fprintf (stderr, "Number of interaction parameters (%d)\n", count);
	    fprintf (stderr, "greater than number of ligand-cell pairs (%d).\n", n);
	    fprintf (stderr, "Returning...\n");
	    fclose (Ifile);
	    return (0);
	}
	if (check_compatibility && nx != mbs_get_ligand_effective_valence(k-1))
	{
	    fprintf(stderr, "Valence incompatible with previous definition.\n");
	    fprintf(stderr, "Previous value: %d\tnew value: %d\n", 
		    mbs_get_ligand_effective_valence(k-1), nx);
	    fprintf(stderr, "Exiting now.\n");
	    exit (2);
	}
	else 
	{
	    if (affinity)  free (affinity);
	    if (!(affinity = (double *)calloc(nx, sizeof(double))))
		mbs_alloc_error("Read Interaction Parameters", "Affinities");
	    for (i=0 ; i<nx ; i++)
	    {
		fscanf(Ifile, "%e", &ec);
		affinity[i] = ec;
	    }
	    mbs_set_interaction_parameters_vk (k-1, l-1, affinity);
	}
    }
    fclose (Ifile);
    
    return (1);
}
int
mbs_read_interaction_file (char *filename, int check_compatibility)
{
    FILE        *Ifile;
    float        K, Kx, fb, rb, fc, rc;
    int          k, l, nl, nc, n, count=0;
    int          _mbs_check_dynamics_vs_equilibrium(double, double, double);
    void         mbs_set_interaction_parameters(int, int, double, double,
						double, double, double, double);
    
    if (!(Ifile = fopen (filename, "r")))
	mbs_file_open_error ("Read Interaction Parameters", filename);
    fscanf (Ifile, "%s %d %d", &dummy, &nl, &nc);
    if (check_compatibility && 
	(nl != _ML_number_of_ligands || nc != _ML_number_of_cell_types))
    {
	fprintf(stderr, "Read Interaction Parameters -- Error.\n");
	if (nl != _ML_number_of_ligands)
	{
	    fprintf(stderr, "Number of ligands incompatible with previous definition.\n");
	    fprintf(stderr, "Previous value: %d\tnew value: %d\n", _ML_number_of_ligands, nl);
	}
	if (nc != _ML_number_of_cell_types)
	{
	    fprintf(stderr, "Number of cell types incompatible with previous definition.\n");
	    fprintf(stderr, "Previous value: %d\tnew value: %d\n", _ML_number_of_cell_types, nc);
	}
	fprintf(stderr, "Exiting now.\n");
	exit (2);
    }
    n = nl*nc;
    while (1)
    {
	fscanf (Ifile, "%d %d %e %e %e %e %e %e", &k, &l, &K, &Kx, &fb, &rb, &fc, &rc);
	if (feof(Ifile))  break;
	if (++count>n)
	{
	    fprintf (stderr, "Read Interaction Parameters -- Error.\n");
	    fprintf (stderr, "Number of interaction parameters (%d)\n", count);
	    fprintf (stderr, "greater than number of ligand-cell pairs (%d).\n", n);
	    fprintf (stderr, "Returning...\n");
	    fclose (Ifile);
	    return (0);
	}
	if (_mbs_check_dynamics_vs_equilibrium (K, fb, rb) &&
	    _mbs_check_dynamics_vs_equilibrium (Kx, fc, rc))
	    mbs_set_interaction_parameters (k-1, l-1, K, Kx, fb, rb, fc, rc);
	else
	{
	    fprintf (stderr, "Exiting now.\n");
	    exit(2);
	}
    }
    fclose (Ifile);
    
    return (1);
}
int 
mbs_read_control_variable_file (char *filename, int check_compatibility)
{
    static char  scale[128], cscale[128], lscale[128], rscale[128];
    FILE        *CVfile;
    float        bottom, top, fraction;
    double      _mbs_convert_molar_to_molar(int, double);
    double      _mbs_convert_molar_to_microgram(int, double);
    double      _mbs_convert_microgram_to_molar(int, double);
    double      _mbs_convert_molar_to_nanogram(int, double);
    double      _mbs_convert_nanogram_to_molar(int, double);
    void         mbs_set_ligand_control_variables(int, int, int, double);
    void         mbs_set_cell_control_variables(int, int, int, double);
    double       total;
    int          steps, k, l, n, vary, contrib;
    
    if (!(CVfile = fopen (filename, "r")))
	mbs_file_open_error ("Read Control Variables", filename);
    fscanf (CVfile, "%s %s %s %s", &dummy, &lscale, &cscale, &rscale);
    if      (strcmp(lscale,"default")==0 || strcmp(lscale,"molar")==0)
    {
	__ML_convert_to_molar   = _mbs_convert_molar_to_molar;
	__ML_convert_from_molar = _mbs_convert_molar_to_molar;
	_ML_ligand_scale = _ML_MOLAR;
    }
    else if (strcmp(lscale,"microgram")==0 || strcmp(lscale, "microgram_per_milliliter")==0)
    {
	__ML_convert_to_molar   = _mbs_convert_microgram_to_molar;
	__ML_convert_from_molar = _mbs_convert_molar_to_microgram;
	_ML_ligand_scale = _ML_MICROGRAM_PER_MILLILITER;
    }
    else if (strcmp(lscale,"nanogram")==0 || strcmp(lscale, "nanogram_per_milliliter")==0)
    {
	__ML_convert_to_molar   = _mbs_convert_nanogram_to_molar;
	__ML_convert_from_molar = _mbs_convert_molar_to_nanogram;
	_ML_ligand_scale = _ML_NANOGRAM_PER_MILLILITER;
    }
    else
    {
	fprintf (stderr, "Unknown ligand scale: %s.\n", scale);
	fprintf (stderr, "Must be: \"molar\", \"microgram\", \"microgram_per_milliliter\",\n");
	fprintf (stderr, "\t\"nanogram\", or \"nanogram_per_milliliter\".\n");
	fprintf (stderr, "Returning...\n");
	return (0);
    }
    if      (strcmp(cscale,"default")==0 || strcmp(cscale,"per_ml")==0)
	_ML_cell_scale = _ML_ONE_PER_MILLILITER;
    else
    {
	fprintf (stderr, "Unknown cell scale: %s.\n", scale);
	fprintf (stderr, "Must be: \"molar\", \"microgram\", \"microgram_per_milliliter\",\n");
	fprintf (stderr, "\t\"nanogram\", or \"nanogram_per_milliliter\".\n");
	fprintf (stderr, "Returning...\n");
	return (0);
    }
    if      (strcmp(rscale,"default")==0 || strcmp(rscale,"per_cell")==0)
	_ML_receptor_scale = _ML_ONE_PER_CELL;
    else if (strcmp(rscale,"per_cm_squared")==0)
	_ML_receptor_scale = _ML_ONE_PER_CM_SQUARED;
    else
    {
	fprintf (stderr, "Unknown receptor scale: %s.\n", scale);
	fprintf (stderr, "Must be: \"molar\", \"microgram\", \"microgram_per_milliliter\",\n");
	fprintf (stderr, "\t\"nanogram\", or \"nanogram_per_milliliter\".\n");
	fprintf (stderr, "Returning...\n");
	return (0);
    }
    fscanf (CVfile, "%s %e %e %d %s", &dummy, &bottom, &top, &steps, &scale);
    _ML_CTRL.ligand_bottom          = bottom;
    _ML_CTRL.ligand_top             = top;
    _ML_CTRL.ligand_number_of_steps = steps;
    if      (strcmp(scale,"linear")==0      || strcmp(scale,"lin")==0)
	_ML_CTRL.ligand_scale_type = _ML_LS_LINEAR;
    else if (strcmp(scale,"logarithmic")==0 || strcmp(scale,"log")==0)
	_ML_CTRL.ligand_scale_type = _ML_LS_LOGARITHMIC;
    else
    {
	fprintf (stderr, "Unknown scale type: %s.\n", scale);
	fprintf (stderr, "Must be: \"linear\", \"lin\", \"logarithmic\", or \"log\".\n");
	fprintf (stderr, "Returning...\n");
	return (0);
    }
    if (!_mbs_select_ligand_scale())   return (0);

    fscanf (CVfile, "%s %d", &dummy, &n);
    if (check_compatibility && n != _ML_number_of_ligands)
    {
	fprintf(stderr, "Read Control Variables -- Error.\n");
	fprintf(stderr, "Number of ligands incompatible with previous definition.\n");
	fprintf(stderr, "Previous value: %d\tnew value: %d\n", _ML_number_of_ligands, n);
	fprintf(stderr, "Exiting now.\n");
	exit (2);
    }
    mbs_set_number_of_ligands (n);
    for (total=0.0,k=0 ; k<n ; k++)
    {
	fscanf (CVfile, "%s %d %d %e", &dummy, &vary, &contrib, &fraction);
	mbs_set_ligand_control_variables (k, vary, contrib, fraction);
	if (vary)  total += fraction;
    }
    for (k=0 ; k<n ; k++)
	if (_ML_CTRL_lcv[k])  _ML_CTRL_lcf[k] /= total;
	else                  _ML_CTRL_lcf[k] = 0.0;
    
    fscanf (CVfile, "%s %d", &dummy, &n);
    if (check_compatibility && n != _ML_number_of_cell_types)
    {
	fprintf(stderr, "Read Control Variables -- Error.\n");
	fprintf(stderr, "Number of cell types incompatible with previous definition.\n");
	fprintf(stderr, "Previous value: %d\tnew value: %d\n", _ML_number_of_cell_types, n);
	fprintf(stderr, "Exiting now.\n");
	exit (2);
    }
    mbs_set_number_of_cell_types (n);
    for (total=0.0,l=0 ; l<n ; l++)
    {
	fscanf (CVfile, "%s %d %d %e", &dummy, &vary, &contrib, &fraction);
	mbs_set_cell_control_variables (l, vary, contrib, fraction);
	if (vary)  total += fraction;
    }
    for (l=0 ; l<n ; l++)
	if (_ML_CTRL_ccv[l])  _ML_CTRL_ccf[l] /= total;
	else                  _ML_CTRL_ccf[l] = 0.0;
    
    fclose (CVfile);

    return (1);
}


int
mbs_set_ligand_scale_parameters (double bottom, double top, double stepsize,
				 int number_of_steps, int scale_type)
{
    int    _mbs_select_ligand_scale();
    
    if (scale_type < 0 || scale_type > 1)
    {
	fprintf (stderr, "MBS Set Ligand Scale Parameters -- Error\n");
	fprintf (stderr, "Unknown scale type: %d.\n", scale_type);
	fprintf (stderr, "Must be 0 or 1.\n");
	fprintf (stderr, "Returning without action...\n");
	return (0);
    }
	
    _ML_CTRL.ligand_bottom     = bottom;
    _ML_CTRL.ligand_top        = top;
    _ML_CTRL.ligand_scale_type = scale_type;
    if (number_of_steps)
    {
	_ML_CTRL.ligand_number_of_steps = number_of_steps;
	if (!_mbs_select_ligand_scale())  return(0);
	
	if (stepsize > 0.0)
	{
	    fprintf (stderr, "MBS Set Ligand Scale Parameters -- Warning\n");
	    fprintf (stderr, "Overridden STEPSIZE.\n");
	    fprintf (stderr, "Old value: %e; new value: %e.\n", stepsize, _ML_CTRL.ligand_stepsize);
	    fprintf (stderr, "Continuing...\n");
	}
    }
    else
    {
	_ML_CTRL.ligand_number_of_steps = 1; /* preliminary value used in _mbs_select_ligand_scale */
	if (!_mbs_select_ligand_scale())  return(0);
	_ML_CTRL.ligand_stepsize = stepsize;
	if (scale_type==_ML_LS_LOGARITHMIC)
	    _ML_CTRL.ligand_number_of_steps = log(top/bottom)/log(stepsize) + 1;
	else
	    _ML_CTRL.ligand_number_of_steps = (top - bottom)/stepsize + 1;
    }
    return (1);
}
int
mbs_set_ligand_scale_converters (int scale)
{
    double      _mbs_convert_molar_to_molar(int, double);
    double      _mbs_convert_molar_to_microgram(int, double);
    double      _mbs_convert_microgram_to_molar(int, double);
    double      _mbs_convert_molar_to_nanogram(int, double);
    double      _mbs_convert_nanogram_to_molar(int, double);

    if (scale < 0 || scale > 2)
    {
	fprintf (stderr, "MBS Set Ligand Scale Converter -- Error\n");
	fprintf (stderr, "Unknown scale: %d.\n", scale);
	fprintf (stderr, "Must be 0, 1, or 2.\n");
	fprintf (stderr, "Returning without action...\n");
	return (0);
    }
    _ML_ligand_scale = scale;
    switch (scale)
    {
      case _ML_MOLAR:
	__ML_convert_to_molar   = _mbs_convert_molar_to_molar;
	__ML_convert_from_molar = _mbs_convert_molar_to_molar;
	break;
      case _ML_MICROGRAM_PER_MILLILITER:
	__ML_convert_to_molar   = _mbs_convert_microgram_to_molar;
	__ML_convert_from_molar = _mbs_convert_molar_to_microgram;
	break;
      case _ML_NANOGRAM_PER_MILLILITER:
	__ML_convert_to_molar   = _mbs_convert_nanogram_to_molar;
	__ML_convert_from_molar = _mbs_convert_molar_to_nanogram;
	break;
      default:
	break;
    }
    return (0);
}

